/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2003-2004 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

//
// sample.c
//
// This is sample coding showing a simple use of Syntrillium Software's Cool
// Edit Pro 2.0 Controller SDK.
#include <windows.h>
#include <commctrl.h>
#include "..\ctrlmsg.h"
#include "internal.h"
#include "resource.h"


// Module scope variables
static HINSTANCE hInstance_;


//! DllMain *****************************************************************/
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD fdwReason, LPVOID reserved)
{
	switch(fdwReason)
	{
		case DLL_PROCESS_ATTACH:
			// LoadLibrary has just been called
			hInstance_ = hInstance;
			return TRUE;

		case DLL_PROCESS_DETACH:
			// Clean up any resources
			
		default:
			return TRUE;
	}

	return TRUE;	
}


//////////////////////////////////////////////////////////////////////////////
/*! CEP calls this as needed to determine if this controller is ready to be
	used.  You should release any resources allocated in this function.  This
	may be called any number of times and should be as simple as possible.
*/
__declspec(dllexport) BOOL QueryController(DevInfo *pDevInfo)
{
	// Fill out the device structure for CEP.
	pDevInfo->dwDevID = 0; // 0 is reserved for testing
	pDevInfo->dwVersion = CTRL_SDK_VER;
	pDevInfo->dwNUmTracks = 4;

	// No more than 19 characters!
	strcpy(pDevInfo->szDevName, "Controller Sample");

	// Since this is a software implementation, it is always ready
	return TRUE;
}


//////////////////////////////////////////////////////////////////////////////
/*! CEP calls this when the user selects this device as the active device (or
	on startup if this is the previously selected device).
	You should do all you initialization in this function.  CEP will call
	CloseController when finished with the controller.
*/
__declspec(dllexport) BOOL OpenController(ControllerFunctions *pcfn)
{
	// Initialize hardware
	return Hardware_Open(hInstance_, pcfn);
}



//////////////////////////////////////////////////////////////////////////////
/*! CEP calls this when anything happens in CEP that the controller needs
	to be aware of.  Ex: active track changes, play start/stop, etc.
*/
__declspec(dllexport) void UpdateController(
	COOLEDIT_CTRLMSG dwMsg, void *pVal1, void *pVal2, void *pVal3)
{
#ifdef _DEBUG
	// This whole code section will notify us if this controller
	// gets any unexpected messages from Cool Edit.
	int ignoremessage[] = 
	{
		CTRL_DECK_REW,			CTRL_DECK_FFW,
		CTRL_DECK_STOP,			CTRL_DECK_PLAY,
		CTRL_DECK_PLAYTOEND,	CTRL_DECK_PLAYLOOPED,
		CTRL_DECK_MONITOR,		CTRL_DECK_RECORD,
		CTRL_DECK_PAUSE,		CTRL_DECK_MASTERFADER,
		CTRL_REGISTRY_CHANGE,	CTRL_DECK_CLOCK,
		CTRL_TRACK_FADER,		CTRL_TRACK_DEVOUT,
		CTRL_TRACK_DEVIN,		CTRL_TRACK_SELECT,
		CTRL_TRACK_MUTE,		CTRL_TRACK_SOLO,
		CTRL_TRACK_RECORD_ARM,	CTRL_TRACK_REC_LEVEL,
		CTRL_TRACK_TITLE,		CTRL_TRACK_PAN,
		CTRL_TRACK_INPUTGAIN,	CTRL_BANK_LEFTENABLE,
		CTRL_BANK_RIGHTENABLE,
	};
	int i, found = 0;
	for(i = 0; !found &&
		i < sizeof(ignoremessage) / sizeof(ignoremessage[0]);
		++i)
		if(ignoremessage[i] == (int)dwMsg)
			found = 1;
	if(!found)
	{
		char szT[1024];
		wsprintf(szT, "Controller message #%d: %d, %d, %d\n",
			dwMsg, pVal1, pVal2, pVal3);
		OutputDebugString(szT);
		_asm int 3
	}
#endif
	Hardware_Update(dwMsg, pVal1, pVal2, pVal3);
} // UpdateController


//////////////////////////////////////////////////////////////////////////////
/*! CEP calls this when it is finished with this controller.  This typcially
	will be because the program is closing or the user selected a different
	controller.
*/
__declspec(dllexport) void CloseController(BOOL bDisconnected)
{
	Hardware_Close();
} // CloseController


//////////////////////////////////////////////////////////////////////////////
/*! OPTIONAL - Implement this only if your device is user-configurable.

	CEP calls this when the user clicks the "configure" button in the "Device
	Properties" window.  This can be called even if this is not the currently
	active controller.  You should release any resources allocated in this
	function.
*/
__declspec(dllexport) void ConfigureController(
	HINSTANCE hInst,
	HWND hWndParent,
	int cCommands,
	GET_COMMAND pfnGetCommand
	)
{
	if(pfnGetCommand)
	{
		ConfigDialogProcParam param = {cCommands, pfnGetCommand};
		DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_CONFIG),
			hWndParent, ConfigDlgProc, (LPARAM)&param);
	}
} // ConfigureController
