/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2003-2004 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

#if _MSC_VER > 1000
#pragma once
#endif

#ifndef SYNTRILLIUM_CTRLMSG___
#define SYNTRILLIUM_CTRLMSG___

#define CTRL_SDK_VER	0x0200

#define MINTRACKVOLUME_DB -300.0
#define MAXTRACKVOLUME_DB 15.0
#define MINMASTERVOLUME_DB -300.0
#define MAXMASTERVOLUME_DB 15.0

#define FADER_MAXIMUM	15.0

#define CONTROLLER_KEYDOWN 0x90
#define CONTROLLER_KEYUP 0x80

#define MAX_TRACK_TITLE 64


typedef struct 
{
	DWORD dwDevID;
	DWORD dwVersion;
	DWORD dwReserved;
	char  szDevName[20];
	DWORD dwNUmTracks;
} DevInfo;

typedef enum
{
	// Transport Controls
	CTRL_DECK_REW = 0,
	CTRL_DECK_FFW,
	CTRL_DECK_STOP,
	CTRL_DECK_PLAY,		  // play selected section
	CTRL_DECK_PLAYTOEND,  // play to end of the file
	CTRL_DECK_PLAYLOOPED, // play looped to the selected section
	CTRL_DECK_MONITOR,
	CTRL_DECK_RECORD,
	CTRL_DECK_PAUSE,
	CTRL_DECK_SET,
	CTRL_DECK_LOCATE_RIGHT, 
	CTRL_DECK_LOCATE_LEFT,
	CTRL_DECK_DIAL,       // same as mouse wheel
	CTRL_DECK_MASTERFADER,
	CTRL_DECK_MASTERFADER_TOUCH,
	CTRL_DECK_METRONOME,

	// Track Controls
	CTRL_TRACK_MUTE = 16,
	CTRL_TRACK_SOLO,
	CTRL_TRACK_SOLOONLY,
	CTRL_TRACK_SELECT,
	CTRL_TRACK_RECORD_ARM,
	CTRL_TRACK_FADER,
	CTRL_TRACK_FADER_TOUCH,
	CTRL_TRACK_PAN,      // set/get pan value, parameter trackID, pan value
	CTRL_TRACK_EQ_HIGH_GAIN,
	CTRL_TRACK_EQ_MID_GAIN,
	CTRL_TRACK_EQ_LOW_GAIN,
	CTRL_TRACK_EQ_HIGH_FREQ,
	CTRL_TRACK_EQ_MID_FREQ,
	CTRL_TRACK_EQ_LOW_FREQ,
	CTRL_TRACK_EQ_HIGH_BAND,
	CTRL_TRACK_EQ_MID_BAND,
	CTRL_TRACK_EQ_LOW_BAND,
	CTRL_TRACK_OUT,
	CTRL_TRACK_BUS,

	// Bank Switch Controls
	CTRL_BANK_LEFT = 35,
	CTRL_BANK_RIGHT,

	// Misc. Other Switches
	CTRL_AUX = 37,
	CTRL_ASGN,
	CTRL_F,
	CTRL_DISPLAYTYPE,
	CTRL_REGISTRY_CHANGE,

	// Only sent from Cool Edit.  You can not send send these to CONTROLLERMSGPROC
	CTRL_TRACK_REC_LEVEL = 42,
	CTRL_DECK_CLOCK,
	CTRL_TRACK_DEVIN,
	CTRL_TRACK_DEVOUT,
	CTRL_TRACK_TITLE,
	CTRL_TRACK_INPUTGAIN,
	CTRL_BANK_LEFTENABLE,
	CTRL_BANK_RIGHTENABLE
} COOLEDIT_CTRLMSG;


typedef void (*CONTROLLERMSGPROC)(COOLEDIT_CTRLMSG dwMsg, short iValue1, short iValue2, short iValue3);
typedef void (*GETCTRLSTATUS)(COOLEDIT_CTRLMSG dwMsg, void *piValue1, void *piValue2, void *piValue3);

typedef UINT (*GET_COMMAND)(DWORD dwIndex, LPSTR szCommandName, UINT cbName, int *piUID, DWORD * pdwCommandFlags);
typedef BOOL (*LOOKUP_COMMAND_INDEX)(int iUID, DWORD * pdwIndex);
typedef BOOL (*EXECUTE_COMMAND)(DWORD dwIndex, WORD wKeyState);

typedef struct
{
	// Core API
	CONTROLLERMSGPROC pControlProc;
	GETCTRLSTATUS pGetStatus;

	// Programmable API Extension
	GET_COMMAND GetCommand;
	LOOKUP_COMMAND_INDEX LookupCommandIndex;
	EXECUTE_COMMAND ExecuteCommand;
	int iCommandCount;
} ControllerFunctions;

// Message wrappers
#define CEP_STATUSMSG(fn, msg, p1, p2, p3) (fn)((msg), (void *)(p1), (void *)(p2), (void *)(p3))
#define CEP_NOTIFYMSG(fn, msg, p1, p2, p3) (fn)((msg), (short)(p1), (short)(p2), (short)(p3))

// void GET_TRACK_FADER(GETCTRLSTATUS fn, short track, double *pVolume)
#define GET_TRACK_FADER(fn, track, pVolume) \
	CEP_STATUSMSG(fn, CTRL_TRACK_FADER, (short)(track), (double *)(pVolume), 0)

// void GET_TRACK_MUTE(GETCTRLSTATUS fn, short track, BOOL *pBool)
#define GET_TRACK_MUTE(fn, track, pBool) \
	CEP_STATUSMSG(fn, CTRL_TRACK_MUTE, (short)(track), (BOOL *)(pBool), 0)

// void GET_TRACK_SOLO(GETCTRLSTATUS fn, short track, BOOL *pBool)
#define GET_TRACK_SOLO(fn, track, pBool) \
	CEP_STATUSMSG(fn, CTRL_TRACK_SOLO, (short)(track), (BOOL *)(pBool), 0)

// void GET_TRACK_SELECT(GETCTRLSTATUS fn, short *pTrack, short *pNumBlocks)
#define GET_TRACK_SELECT(fn, pTrack, pNumTracks) \
	CEP_STATUSMSG(fn, CTRL_TRACK_SELECT, (short *)(pTrack), (short *)(pNumTracks), 0)

// void GET_TRACK_RECORDARM(GETCTRLSTATUS fn, short track, BOOL *pBool)
#define GET_TRACK_RECORDARM(fn, track, pBool) \
	CEP_STATUSMSG(fn, CTRL_TRACK_RECORD_ARM, (short)(track), (BOOL *)(pBool), 0)

// void GET_TRACK_PAN(GETCTRLSTATUS fn, short track, short *pLeftRight)
#define GET_TRACK_PAN(fn, track, pLeftRight) \
	CEP_STATUSMSG(fn, CTRL_TRACK_PAN, (short)(track), (short *)(pLeftRight), 0)

// void GET_TRACK_TITLE(GETCTRLSTATUS fn, short track, char *name)
#define GET_TRACK_TITLE(fn, track, name) \
	CEP_STATUSMSG(fn, CTRL_TRACK_TITLE, (short)(track), (char *)(name), 0)

// void GET_TRACK_EQHIGHGAIN(GETCTRLSTATUS fn, short track, double *pVal)
#define GET_TRACK_EQHIGHGAIN(fn, track, pVal) \
	CEP_STATUSMSG(fn, CTRL_TRACK_EQ_HIGH_GAIN, (short)(track), (double *)(pVal), 0)
#define GET_TRACK_EQMIDGAIN(fn, track, pVal) \
	CEP_STATUSMSG(fn, CTRL_TRACK_EQ_MID_GAIN, (short)(track), (double *)(pVal), 0)
#define GET_TRACK_EQLOWGAIN(fn, track, pVal) \
	CEP_STATUSMSG(fn, CTRL_TRACK_EQ_LOW_GAIN, (short)(track), (double *)(pVal), 0)

#define GET_TRACK_EQHIGHFREQ(fn, track, pVal) \
	CEP_STATUSMSG(fn, CTRL_TRACK_EQ_HIGH_FREQ, (short)(track), (double *)(pVal), 0)
#define GET_TRACK_EQMIDFREQ(fn, track, pVal) \
	CEP_STATUSMSG(fn, CTRL_TRACK_EQ_MID_FREQ, (short)(track), (double *)(pVal), 0)
#define GET_TRACK_EQLOWFREQ(fn, track, pVal) \
	CEP_STATUSMSG(fn, CTRL_TRACK_EQ_LOW_FREQ, (short)(track), (double *)(pVal), 0)

#define GET_TRACK_EQHIGHBAND(fn, track, pVal) \
	CEP_STATUSMSG(fn, CTRL_TRACK_EQ_HIGH_BAND, (short)(track), (double *)(pVal), 0)
#define GET_TRACK_EQMIDBAND(fn, track, pVal) \
	CEP_STATUSMSG(fn, CTRL_TRACK_EQ_MID_BAND, (short)(track), (double *)(pVal), 0)
#define GET_TRACK_EQLOWBAND(fn, track, pVal) \
	CEP_STATUSMSG(fn, CTRL_TRACK_EQ_LOW_BAND, (short)(track), (double *)(pVal), 0)

// void GET_TRACK_DEVIN(GETCTRLSTATUS fn, short track, char *name)
#define GET_TRACK_DEVIN(fn, track, device_name) \
	CEP_STATUSMSG(fn, CTRL_TRACK_DEVIN, (short)(track), (char *)(device_name), 0)

// void GET_TRACK_DEVOUT(GETCTRLSTATUS fn, short track, char *name)
#define GET_TRACK_DEVOUT(fn, track, device_name) \
	CEP_STATUSMSG(fn, CTRL_TRACK_DEVOUT, (short)(track), (char *)(device_name), 0)

// void GET_TRACK_RECLEVEL(GETCTRLSTATUS fn, short track, double *pLevel)
//#define GET_TRACK_RECLEVEL(fn, track, device_name) \
//	CEP_STATUSMSG(fn, CTRL_TRACK_REC_LEVEL, (short)(track), (double *)pLevel, 0)

// void GET_MASTERFADER(GETCTRLSTATUS fn, double *pVolume)
//#define GET_MASTERFADER(fn, pVolume) \
//	CEP_STATUSMSG(fn, CTRL_DECK_MASTERFADER, double *pVolume, 0, 0)

// void GET_DISPLAYTYPE(GETCTRLSTATUS fn, BOOL *pBoolMultitrack)
#define GET_DISPLAYTYPE(fn, pBoolMultitrack) \
	CEP_STATUSMSG(fn, CTRL_DISPLAYTYPE, (BOOL *)(pBoolMultitrack), 0, 0)


//////////////////////////////////////////////////////////////////////////////
// DECK MESSAGES
//////////////////////////////////////////////////////////////////////////////
// void SET_DECK_REW(CONTROLLERMSGPROC fn, BOOL Pressed)
#define SET_DECK_REW(fn, Pressed) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_REW, (BOOL)(Pressed), 0, 0)
// void SET_DECK_FFW(CONTROLLERMSGPROC fn, BOOL Pressed)
#define SET_DECK_FFW(fn, Pressed) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_FFW, (BOOL)(Pressed), 0, 0)
// void SET_DECK_PLAY(CONTROLLERMSGPROC fn)
#define SET_DECK_PLAY(fn) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_PLAY, TRUE, 0, 0)
// void SET_DECK_PLAYTOEND(CONTROLLERMSGPROC fn)
#define SET_DECK_PLAYTOEND(fn) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_PLAYTOEND, TRUE, 0, 0)
// void SET_DECK_PLAYLOOPED(CONTROLLERMSGPROC fn)
#define SET_DECK_PLAYLOOPED(fn) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_PLAYLOOPED, TRUE, 0, 0)
// void SET_DECK_RECORD(CONTROLLERMSGPROC fn)
#define SET_DECK_RECORD(fn) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_RECORD, TRUE, 0, 0)
// void SET_DECK_TOGGLE_MONITOR(CONTROLLERMSGPROC fn)
#define SET_DECK_TOGGLE_MONITOR(fn) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_MONITOR, TRUE, 0, 0)
// void SET_DECK_TOGGLE_PAUSE(CONTROLLERMSGPROC fn)
#define SET_DECK_TOGGLE_PAUSE(fn) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_PAUSE, TRUE, 0, 0)
// void SET_DECK_STOP(CONTROLLERMSGPROC fn)
#define SET_DECK_STOP(fn) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_STOP, TRUE, 0, 0)
// void SET_DECK_SET_CUE(CONTROLLERMSGPROC fn)
#define SET_DECK_SET_CUE(fn) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_SET, TRUE, 0, 0)
// void SET_DECK_LOCATE_RIGHT(CONTROLLERMSGPROC fn)
#define SET_DECK_LOCATE_RIGHT(fn) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_LOCATE_RIGHT, TRUE, 0, 0)
// void SET_DECK_LOCATE_LEFT(CONTROLLERMSGPROC fn)
#define SET_DECK_LOCATE_LEFT(fn) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_LOCATE_LEFT, TRUE, 0, 0)
// void SET_DECK_DIAL_UP(CONTROLLERMSGPROC fn)
#define SET_DECK_DIAL_UP(fn) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_DIAL, 1, 0, 0)
// void SET_DECK_DIAL_DOWN(CONTROLLERMSGPROC fn)
#define SET_DECK_DIAL_DOWN(fn) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_DIAL, -1, 0, 0)
// void SET_DECK_TOGGLE_METRONOME(CONTROLLERMSGPROC fn)
#define SET_DECK_TOGGLE_METRONOME(fn, Pressed) \
	CEP_NOTIFYMSG(fn, CTRL_DECK_METRONOME, TRUE, 0, 0)

//////////////////////////////////////////////////////////////////////////////
// TRACK MESSAGES
//////////////////////////////////////////////////////////////////////////////
// void SET_TRACK_MUTE(CONTROLLERMSGPROC fn, short track)
#define SET_TRACK_TOGGLE_MUTE(fn, track) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_MUTE, TRUE, (short)(track), 0)

// void SET_TRACK_FADER(CONTROLLERMSGPROC fn, short track, short volume)
#define SET_TRACK_FADER(fn, track, volume) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_FADER, (short)(volume), (short)(track), 0)

// void SET_TRACK_FADER_F(CONTROLLERMSGPROC fn, short track, double volume)
#define SET_TRACK_FADER_F(fn, track, volume) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_FADER, (short)((volume) * 100), (short)(track), 0)

// void SET_TRACK_SOLO(CONTROLLERMSGPROC fn, short track)
#define SET_TRACK_TOGGLE_SOLO(fn, track) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_SOLO, TRUE, (short)(track), 0)

// void SET_TRACK_SELECT(CONTROLLERMSGPROC fn, short track)
#define SET_TRACK_SELECT(fn, track) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_SELECT, 0, (short)(track), 0)

// void SET_TRACK_SELECT(CONTROLLERMSGPROC fn, short track)
#define SET_TRACK_FADER_TOUCH(fn, track, touch) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_FADER_TOUCH, (short)touch, (short)(track), 0)

// void SET_TRACK_RECORDARM(CONTROLLERMSGPROC fn, short track)
#define SET_TRACK_TOGGLE_RECORDARM(fn, track) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_RECORD_ARM, TRUE, (short)(track), 0)

// void SET_TRACK_PAN_SPIN(CONTROLLERMSGPROC fn, short track, short LeftRight)
#define SET_TRACK_PAN_SPIN(fn, track, LeftRight) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_PAN, LeftRight, (short)(track), 0)
// void SET_TRACK_PAN_LEFT(CONTROLLERMSGPROC fn, short track)
#define SET_TRACK_PAN_LEFT(fn, track) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_PAN, -5, (short)(track), 0)
// void SET_TRACK_PAN_RIGHT(CONTROLLERMSGPROC fn, short track)
#define SET_TRACK_PAN_RIGHT(fn, track) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_PAN, 5, (short)(track), 0)

// void SET_TRACK_TITLE(CONTROLLERMSGPROC fn, short track, char *name)
//#define SET_TRACK_TITLE(fn, track, name) \
//	CEP_NOTIFYMSG(fn, CTRL_TRACK_TITLE, (short)(track), (char *)(name), 0)

#define SET_TRACK_EQHIGHGAIN(fn, track, delta) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_EQ_HIGH_GAIN, delta, (short)(track), 0)
#define SET_TRACK_EQMIDGAIN(fn, track, delta) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_EQ_MID_GAIN, delta, (short)(track), 0)
#define SET_TRACK_EQLOWGAIN(fn, track, delta) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_EQ_LOW_GAIN, delta, (short)(track), 0)

#define SET_TRACK_EQHIGHFREQ(fn, track, delta) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_EQ_HIGH_FREQ, delta, (short)(track), 0)
#define SET_TRACK_EQMIDFREQ(fn, track, delta) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_EQ_MID_FREQ, delta, (short)(track), 0)
#define SET_TRACK_EQLOWFREQ(fn, track, delta) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_EQ_LOW_FREQ, delta, (short)(track), 0)

#define SET_TRACK_EQHIGHBAND(fn, track, delta) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_EQ_HIGH_BAND, delta, (short)(track), 0)
#define SET_TRACK_EQMIDBAND(fn, track, delta) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_EQ_MID_BAND, delta, (short)(track), 0)
#define SET_TRACK_EQLOWBAND(fn, track, delta) \
	CEP_NOTIFYMSG(fn, CTRL_TRACK_EQ_LOW_BAND, delta, (short)(track), 0)


//some define for the UniqueID for audition command
#define UID_METRONOME		517
#define UID_ADDCUE			131
#define UID_LOOP_MODE_ON    501 //"Loop Mode On",
#define UID_LOOP_MODE_OFF	502 //"Loop Mode Off",
#define UID_ADJ_CHAN_LEFT	152 //"Adjust Channel Left / Track Up",
#define UID_ADJ_CHAN_RIGHT  153 //"Adjust Channel Right / Track Down",
#define UID_ADJ_L_TO_L      300 //"Adjust Selection Left Side to the Left",
#define UID_ADJ_R_TO_R		303 //"Adjust Selection Right Side to the Right",
#define UID_ZOOM_IN_V		448 //"Zoom In Vertically",
#define UID_ZOOM_OUT_V		449 //"Zoom Out Vertically",
#define UID_ZOOM_OUT_H		440 //"Zoom Out Horizontally",
#define UID_ZOOM_TO_SEL		443 //"Zoom to Selection",
#define UID_PLAY_NORMAL		600 //"Play Normal",
#define UID_PLAY_ALL		605//"Play All/Pause toggle",
#define UID_UNDO			109	 //"Undo (1)";
#define UID_REDO			110	 //"Redo";
#define UID_COPY			100	 //"Copy (1)";
#define UID_CUT				103	 //"Cut (1)";
#define UID_PASTE			105	 //"Paste (1)";
#define UID_TRIM			112	 //"Trim";//"Repeat Last Command (no dialog)";
#define UID_FREQ_TOGGLE		431	 //"Frequency Window toggle";
#define UID_PHASE_TOGGLE	432	 //"Phase Window toggle";
#define UID_SPECTRAL_TOGGLE	406	 //"Spectral/Waveform toggle";
#define UID_INSERT_MULT		118	 //"Insert In Multitrack";
#define UID_MULTI_TOGGLE	400	 //"Multitrack toggle";
#define UID_MONITOR_REC		504	 //"Monitor Record Level toggle";
#define UID_CUE_WIN_TOGGLE	416	 //"Cue List Window toggle";
#define UID_SEL_ALL_CLIPS	326	 //"Select Entire Wave / All Clips";
#define UID_ZOOM_OUT_FULL	442	 //"Zoom Out Full Both Axis";
#define UID_CLOSE			 14	 //"Close (1)";
#define UID_SAVE			 19	 //"Save";
#define UID_MIXER_TOGGLE	426	 //"Mixer Window toggle";
#define UID_SES_PROP_TOGGLE	425	 //"Session Properties Window toggle";
#define UID_TRACK_EQ_TOGGLE	427	 //"Track EQ Window toggle";
#define UID_ORGANIZER		415	 //"Organizer Window toggle";
#define UID_SEL_NEXTCLIP	328	 //"Select Next Clip In Track";
#define UID_CLIP_SPLIT		222	 //"Clips: Split";
#define UID_MIDI_TRIGGER	506	 //"MIDI Trigger Enable toggle";
#define UID_REMOVE_CLIP		231	 //"Clips: Remove Clip";
#define UID_MOVE_CUR		312	 //"Move Cursor To Beginning";
#define UID_SMPTE_SLAVE		512	 //"SMPTE Slave Enable toggle";
#define UID_SMPTE_MASTER	513	 //"SMPTE Master Enable toggle";
#define UID_NUDGE_LEFT		246	 //"Clips: Nudge Left";
#define UID_NUDGE_RIGHT		247	 //"Clips: Nudge Right";
#define UID_PUNCH_IN		212	 //"Clips: Punch In";
#define UID_ANCHOR_LEFT		338	 //"Selection Anchor Left when playing";
#define UID_ANCHOR_RIGHT	339	 //"Selection Anchor Right when playing";

#endif // SYNTRILLIUM_CTRLMSG___