/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2003-2007 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

#include <windows.h>
#include <mmsystem.h>
#include <stdio.h>
#include "..\Filters.h"
#include "resource.h"
#include "..\RiffStuff.h"

// Prototyped as per documentation found here: http://support.microsoft.com/default.aspx?scid=kb;en-us;94248
extern BOOL WINAPI _CRT_INIT(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpReserved);

#define MAXFMTSIZE 256

typedef struct output_tag  // any special vars associated with output file
{
	HANDLE hFile;
	long lSize;
	WORD wChannels;
	long lSamprate;
	WORD wBitsPerSample;
	WORD wBlockAlign;
	long lFact;
	WORD wOrigBitsPerSample;
	long bytesleft;
	DWORD dwFormat;
	DWORD dwBytesWritten;
	long lFactAt;
	DWORD dwDataOffset;
	DWORD dwFmtOffset;
	BOOL bWroteDataChunk;
	WAVEFORMATEX pcmWaveFormat;
	char szExtra[MAXFMTSIZE];
	RIFFSTUFF riff;

}	MYOUTPUT;

typedef struct input_tag // any special vars associated with input file
{
	HANDLE hFile;
	long lSize;
	WORD wChannels;
	long lSamprate;
	WORD wBitsPerSample;
	WORD wBlockAlign;
	long lFact;
	long bytesleft;
	DWORD dwFormat;
	DWORD dwFilePos;
	DWORD dwFullSize;
	DWORD dwDataOffset;
	DWORD dwDataSize;
	WAVEFORMATEX pcmWaveFormat;
	char szExtra[MAXFMTSIZE];
	RIFFSTUFF riff;

}	MYINPUT;

BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved);
__declspec(dllexport) BOOL WINAPI DIALOGMsgProc(HWND hWndDlg, UINT Message, WPARAM wParam, LPARAM lParam);

#ifdef NDEBUG	// RELEASE version optimization

//-------------------------------------------------------------
// Dummy startup routine that does nothing except call DllMain
// This cuts out all of the standard startup code crud that
// bloats the DLL, and makes it take longer to load
//-------------------------------------------------------------
static BOOL __stdcall _DllMainCRTStartup( HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	return DllMain( hinstDLL, fdwReason, lpvReserved );
}

#endif	// NDEBUG

static HINSTANCE g_hInstDLL;
BOOL WINAPI DllMain (HINSTANCE hInstance, DWORD fdwReason, LPVOID lpReserved)
{
	static BOOL fInit = FALSE;

	switch (fdwReason)
	{
		case DLL_PROCESS_ATTACH:
#if NDEBUG
			if (!_CRT_INIT(hInstance, fdwReason, lpReserved))
				return FALSE;
#endif
			DisableThreadLibraryCalls(hInstance);
	
			if (!fInit)
			{
				g_hInstDLL = hInstance;
				if (g_hInstDLL)
				{
					fInit = TRUE;
				}
			}
			break;

		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
			break;

		case DLL_PROCESS_DETACH:
#if NDEBUG
		  	if (!_CRT_INIT(hInstance, fdwReason, lpReserved))
				return FALSE;
#endif
			if (fInit)
			{
				FreeLibrary(g_hInstDLL);
				g_hInstDLL = NULL;
				fInit = FALSE;
			}
			break;
	}

	return TRUE;
}


// Fill COOLQUERY structure with information regarding this file filter

__declspec(dllexport) short WINAPI QueryCoolFilter(COOLQUERY far * cq)
{
	WCHAR sz[MAX_PATH];
	LoadString(g_hInstDLL, IDS_NAME, sz, sizeof(sz)/sizeof(*sz));
	wcscpy(cq->szName,sz); 
	LoadString(g_hInstDLL, IDS_COPYRIGHT, sz, sizeof(sz)/sizeof(*sz));
	wcscpy(cq->szCopyright,sz);
	wcscpy(cq->szExt, L"WAV"); 
	cq->lChunkSize=30000; 
	cq->dwFlags=QF_RATEADJUSTABLE|QF_CANSAVE|QF_CANLOAD|QF_READSPECIALLAST|QF_WRITESPECIALLAST|QF_HASOPTIONSBOX;
	cq->Stereo8=0xFF; // supports all rates of stereo 8
	cq->Stereo16=0xFF;
	cq->Stereo24=0x00;
	cq->Stereo32=0x00;
	cq->Mono8=0xFF; // supports all rates of stereo 8
	cq->Mono16=0xFF;
	cq->Mono24=0x00;
	cq->Mono32=0x00;
	cq->Quad32=0x00;
	cq->Quad16=0x00;
	cq->Quad8=0x00; 

	return C_VALIDLIBRARY;
}

__declspec(dllexport) long WINAPI FilterGetFileSize(HANDLE hInput)
{
	long lSize=0L;

	if (hInput)
	{
		MYINPUT far *mi;
		mi=(MYINPUT far *)GlobalLock(hInput);
		lSize=mi->lSize;
		GlobalUnlock(hInput);
	}
	return lSize;
}

__declspec(dllexport) BOOL WINAPI FilterUnderstandsFormat(LPWSTR filename)
{
	FILE *nFile;
	BOOL bUnderstands=FALSE;
	
	nFile=_wfopen(filename, L"rb");
	if (nFile)
	{
		char m[13];
		
		while (1)
		{
			long biglen;
			if (fread(m,1,4,nFile)!=4)
				break;
			m[4]=0x00;
			if (strcmp(m,"RIFF")!=0)
				break;
			if (fread(&biglen,1,4,nFile)!=4)
				break;
			if (fread(m,1,4,nFile)!=4)
				break;
			
			while (strcmp(m,"WAVE")!=0)
			{	
				long length;
				if (fread(&length,1,4,nFile)!=4)
				{	
					fclose(nFile);
					return FALSE;
				}
				if ((length < 4) || (length > 0x7fff0000))
				{
					fclose(nFile);
					return FALSE;
				}

				fseek(nFile,length,SEEK_CUR);
				
				if (fread(m,1,4,nFile)!=4)
				{
					fclose(nFile);
					return FALSE;
				}
			}
			
			if (fread(m,1,4,nFile)!=4)
				break;
			
			while (strcmp(m,"fmt ")!=0)
			{
				long length;
				if (fread(&length,1,4,nFile)!=4)
				{
					fclose(nFile);
					return FALSE;
				}
				fseek(nFile,length,SEEK_CUR);
			
				if (fread(m,1,4,nFile)!=4)
				{
					fclose(nFile);
					return FALSE;
				}
			}

			{	short type;
				long length;
				fread(&length,4,1,nFile);
				fread(&type,2,1,nFile);
				if ((type==7) || (type==6))
					bUnderstands=TRUE;
			}
			break;
		}
		fclose(nFile);
	}
	return bUnderstands;
}


__declspec(dllexport) DWORD WINAPI FilterGetOptions(HWND hWnd, HINSTANCE hInst, long lSamprate, WORD wChannels, WORD wBitsPerSample, DWORD dwOptions) // return 0 if no options box
{
	long nDialogReturn=0L;
			
	if (dwOptions==0)
		nDialogReturn=7;	// default to mu-law
	else
		nDialogReturn=dwOptions;
	
	return nDialogReturn = (long)DialogBoxParam(g_hInstDLL, MAKEINTRESOURCE(IDD_COMPRESSION), hWnd, DIALOGMsgProc, nDialogReturn);
}

__declspec(dllexport) DWORD WINAPI FilterOptions(HANDLE hInput)
{
	MYINPUT far *mi;
	DWORD options;
	if (!hInput) return 0;
	mi=(MYINPUT far *)GlobalLock(hInput);
	options=(DWORD)mi->dwFormat;
	GlobalUnlock(hInput);
	return options;
}

__declspec(dllexport) DWORD WINAPI FilterOptionsString(HANDLE hInput, LPWSTR szString)
{
	MYINPUT far *mi;
	DWORD options;
	WCHAR sz[MAX_PATH];
	if (!hInput) return 0;
	mi=(MYINPUT far *)GlobalLock(hInput);
	options=(DWORD)mi->dwFormat;
	switch (options)
	{
		case 7:
			LoadString(g_hInstDLL, IDS_OPTION1, sz, sizeof(sz)/sizeof(*sz));
			wcscpy(szString,sz);
			break;
		case 6:
			LoadString(g_hInstDLL, IDS_OPTION2, sz, sizeof(sz)/sizeof(*sz));
			wcscpy(szString,sz);
			break;
	}
	GlobalUnlock(hInput);
	return options;
}


__declspec(dllexport) BOOL WINAPI DIALOGMsgProc(HWND hWndDlg, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message)
	{
		case WM_INITDIALOG:
		{
			long nDialogReturn;
			nDialogReturn=(long)lParam;

			if (nDialogReturn==6)
				CheckDlgButton(hWndDlg,IDC_RADIO1,TRUE);
			else
				CheckDlgButton(hWndDlg,IDC_RADIO2,TRUE);  // so anything other than a-law defaults to mu-law
		}
			break;

		case WM_CLOSE:
			PostMessage(hWndDlg, WM_COMMAND, IDCANCEL, 0L);
			break;

		case WM_COMMAND:
		{
			switch(LOWORD(wParam))
			{
				case IDOK:
				{
					long nDialogReturn;
					if (IsDlgButtonChecked(hWndDlg,IDC_RADIO1))
						nDialogReturn=6;
					else
						nDialogReturn=7;	// so anything else but alaw defaults to mu-law

					EndDialog(hWndDlg, (short)nDialogReturn);
				}
					break;

				case IDCANCEL:
					// Ignore data values entered into the controls
					// and dismiss the dialog window returning FALSE
					EndDialog(hWndDlg, FALSE);
					break;
			}
		}
			break;

		default:
			return FALSE;
	}
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////// SPECIAL DATA STUFF


__declspec(dllexport) DWORD WINAPI FilterWriteSpecialData(HANDLE hOutput,LPCSTR szListType, LPCSTR szType, 
															char * pData, DWORD dwSize)
{
	DWORD dwReturn;
	MYOUTPUT * mo;
	if (!hOutput)
		return 0; // FALSE
	mo=(MYOUTPUT *)GlobalLock(hOutput);

	dwReturn = RiffStuffWriteSpecialData(&mo->riff, szListType, szType, pData, dwSize);
	
	GlobalUnlock(hOutput);
	return dwReturn;
}

// Note, mi->dwFilePos is not updated in this next call
__declspec(dllexport) DWORD WINAPI FilterGetNextSpecialData(HANDLE hInput, SPECIALDATA * psp)
{
	MYINPUT * mi;
	DWORD dwReturn;
	if ((!hInput) || (!psp))
		return 0;
	mi=(MYINPUT *)GlobalLock(hInput);

	dwReturn = RiffStuffGetNextSpecialData(&mi->riff, psp);

	GlobalUnlock(hInput);
	return dwReturn;
}

__declspec(dllexport) DWORD WINAPI FilterGetFirstSpecialData(HANDLE hInput, SPECIALDATA * psp)
{
	DWORD dwReturn;
	MYINPUT * mi;
	if (!hInput)
		return 0;  // return is boolean
	mi=(MYINPUT *)GlobalLock(hInput);

	dwReturn = RiffStuffGetFirstSpecialData(&mi->riff, psp);

	GlobalUnlock(hInput);
	return dwReturn;
}

///////// END OF SPECIAL DATA STUFF
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////


__declspec(dllexport) HANDLE WINAPI OpenFilterOutput(LPWSTR lpstrFilename,long lSamprate,WORD wBitsPerSample,WORD wChannels,long lSize, long far * lChunkSize,DWORD options)
{
	HANDLE hOutput;
	WORD wBlockAlign;
	long lFact;
	DWORD dwZero=0;
	
	wBlockAlign=wBitsPerSample*wChannels/8;
	if (options!=6)
		options=7; // defaults to mu-law

	hOutput=GlobalAlloc(GMEM_MOVEABLE|GMEM_SHARE|GMEM_ZEROINIT,sizeof(MYOUTPUT));

	if (hOutput)
	{
		MYOUTPUT far *mo;
		DWORD dwBytesWritten;
		DWORD dwSize;

		mo=(MYOUTPUT *)GlobalLock(hOutput);
		mo->lSize=lSize/2;
		mo->wOrigBitsPerSample=wBitsPerSample;
		mo->wChannels=wChannels;
		mo->wBitsPerSample=8;
		mo->lSamprate=lSamprate;
		mo->wBlockAlign=wBlockAlign;
		mo->dwBytesWritten=0L;
		mo->dwFormat=options;
		mo->lFactAt=0L;
		mo->pcmWaveFormat.wBitsPerSample=mo->wBitsPerSample;
		mo->pcmWaveFormat.wFormatTag=(short)options;
		mo->pcmWaveFormat.nChannels=wChannels;
		mo->pcmWaveFormat.nSamplesPerSec=lSamprate;
		mo->pcmWaveFormat.nAvgBytesPerSec=(DWORD)wChannels*mo->pcmWaveFormat.nSamplesPerSec*(DWORD)mo->wBitsPerSample/8;
		mo->pcmWaveFormat.cbSize=0;
		mo->pcmWaveFormat.nBlockAlign=wChannels*mo->wBitsPerSample/8;
		*lChunkSize=30000;

		lFact=lSize/wBlockAlign;
		mo->hFile=CreateFile(lpstrFilename, GENERIC_READ|GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if (mo->hFile==INVALID_HANDLE_VALUE)
		{
			GlobalUnlock(hOutput);
			GlobalFree(hOutput);
			return NULL;
		}

		RiffStuffOpenOutput(&mo->riff, mo->hFile, 12);

		WriteFile(mo->hFile,"RIFF",4,&dwBytesWritten,NULL);
		WriteFile(mo->hFile,&dwZero,4,&dwBytesWritten,NULL); // placeholder for file size-8 (RIFF size)
		WriteFile(mo->hFile,"WAVE",4,&dwBytesWritten,NULL);
		// Write out format chunk
		RiffStuffEnter(&mo->riff,"fmt ");
		dwSize=sizeof(WAVEFORMATEX)+mo->pcmWaveFormat.cbSize; // let's not use new format header?
		WriteFile(mo->hFile,&mo->pcmWaveFormat.wFormatTag,dwSize,&dwBytesWritten,NULL);
		RiffStuffExit(&mo->riff);

		// Write fact chunk here
		WriteFile(mo->hFile,"fact",4,&dwBytesWritten,NULL);
		dwSize=4;
		WriteFile(mo->hFile,&dwSize,4,&dwBytesWritten,NULL);
		mo->lFactAt=SetFilePointer(mo->hFile,0,NULL,FILE_CURRENT);
		WriteFile(mo->hFile,&lFact,4,&dwBytesWritten,NULL);

		mo->bWroteDataChunk = FALSE;

		mo->lFact=lFact;
		GlobalUnlock(hOutput);
	}
	return hOutput;
}


__declspec(dllexport) void WINAPI CloseFilterOutput(HANDLE hOutput)
{
	if (hOutput)
	{
		MYOUTPUT far *mo;
		DWORD dwFileSize;
		DWORD dwBytesWritten;
		long lFact;
		mo=(MYOUTPUT far *)GlobalLock(hOutput);

		lFact=(long)(mo->dwBytesWritten/(DWORD)mo->wBlockAlign);

		RiffStuffCloseOutput(&mo->riff);

		// Update total file size thingy
		dwFileSize=GetFileSize(mo->hFile,NULL);
		dwFileSize-=8;
		SetFilePointer(mo->hFile,4,NULL,FILE_BEGIN);
		WriteFile(mo->hFile,&dwFileSize,4,&dwBytesWritten,NULL);
		// Update fact data
		SetFilePointer(mo->hFile,mo->lFactAt,NULL,FILE_BEGIN);
		WriteFile(mo->hFile,&lFact,4,&dwBytesWritten,NULL);
		// Just to be safe, go to end before closing
		SetFilePointer(mo->hFile,0,NULL,FILE_END);
		FlushFileBuffers(mo->hFile);
		CloseHandle(mo->hFile);

		GlobalUnlock(hOutput);
		GlobalFree(hOutput);
	}
}


// return handle that will be passed in to close, and write routines
__declspec(dllexport) HANDLE WINAPI OpenFilterInput(LPWSTR lpstrFilename,long far *lSR,WORD far *wBPS,WORD far *wC,HWND hWnd, long far *lChunkSize)
{
	HANDLE hInput;
	MYINPUT *mi;
	long lFact=0;
	DWORD dwDataOffset=0;
	DWORD dwDataSize=0;
	HANDLE hFile=INVALID_HANDLE_VALUE;
	DWORD dwBytesRead;
	char szType[5]={"xxxx"};
	DWORD dwFullSize=0;
	DWORD dwFileSize=0;

	hFile=CreateFile(lpstrFilename,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,NULL);
	if (hFile==INVALID_HANDLE_VALUE)
		return NULL;

	// Find data start and lFact value
	ReadFile(hFile,szType,4,&dwBytesRead,NULL);
	if (strcmp(szType,"RIFF")!=0)
	{
		CloseHandle(hFile);
		return NULL;
	}
	ReadFile(hFile,&dwFullSize,4,&dwBytesRead,NULL);
	ReadFile(hFile,szType,4,&dwBytesRead,NULL);
	if (strcmp(szType,"WAVE")!=0)
	{
		CloseHandle(hFile);
		return NULL;
	}
	hInput=GlobalAlloc(GMEM_MOVEABLE|GMEM_SHARE|GMEM_ZEROINIT,sizeof(MYINPUT));
	if (!hInput)
	{
		CloseHandle(hFile);
		return NULL;
	}
	dwFileSize=GetFileSize(hFile,NULL);
	mi=(MYINPUT *)GlobalLock(hInput);
	mi->dwFullSize=dwFullSize;
	mi->hFile=hFile;
	mi->dwFilePos=SetFilePointer(hFile,0,NULL,FILE_CURRENT); // should be 12

	RiffStuffOpenInput(&mi->riff, hFile, dwFullSize, mi->dwFilePos);
	while (1) // good, now read as we need
	{
		DWORD dwChunkSize;
		ReadFile(hFile,szType,4,&dwBytesRead,NULL);
		if (!dwBytesRead)
			break;
		ReadFile(hFile,&dwChunkSize,4,&dwBytesRead,NULL);
		if (!dwBytesRead)
			break;
		mi->dwFilePos+=8;
		if (strcmp(szType,"fmt ")==0)
		{
			DWORD dwSizeToRead=min(dwChunkSize,sizeof(WAVEFORMATEX));
			mi->pcmWaveFormat.cbSize=0;
			ReadFile(hFile,&mi->pcmWaveFormat,dwSizeToRead,&dwBytesRead,NULL);
			if (dwSizeToRead=min(mi->pcmWaveFormat.cbSize,MAXFMTSIZE)) // !0
				ReadFile(hFile,&mi->szExtra,dwSizeToRead,&dwBytesRead,NULL);
			mi->dwFilePos+=((dwChunkSize+1)/2)*2;
			SetFilePointer(hFile,mi->dwFilePos,NULL,FILE_BEGIN);
			if ((mi->pcmWaveFormat.wFormatTag!=6)  &&
				(mi->pcmWaveFormat.wFormatTag!=7)) // THIS ONE ONLY HANDLES TYPES 6 & 7 ALAW/muLAW DATA
			{
				CloseHandle(hFile);
				GlobalUnlock(hInput);
				GlobalFree(hInput);
				return NULL;
			}
		}
		else if (strcmp(szType,"data")==0)
		{	
			if (!mi->dwDataOffset)
			{
				mi->dwDataOffset=SetFilePointer(hFile,0,NULL,FILE_CURRENT);
				mi->dwDataSize=dwChunkSize; // may be greater than true data size
				SetFilePointer(hFile,mi->dwDataOffset+dwChunkSize,NULL,FILE_BEGIN);
				mi->dwFilePos+=((dwChunkSize+1)/2)*2;
			}
		}
		else if (strcmp(szType,"fact")==0)
		{
			if (dwChunkSize)
				ReadFile(hFile,&lFact,min(4,dwChunkSize),&dwBytesRead,NULL);
			mi->dwFilePos+=((dwChunkSize+1)/2)*2;
			if (dwChunkSize!=4)
				SetFilePointer(hFile,mi->dwFilePos,NULL,FILE_BEGIN);
		}
		else // we don't care, just skip it
		{
			mi->dwFilePos+=((dwChunkSize+1)/2)*2;
			SetFilePointer(hFile,mi->dwFilePos,NULL,FILE_BEGIN);
			if ((mi->dwFilePos>=mi->dwFullSize+8) || (mi->dwFilePos>dwFileSize))
				break; // done
		}
	}
	if (mi->dwDataOffset>dwFileSize)
	{
		// Error in RIFF
		CloseHandle(hFile);
		GlobalUnlock(hInput);
		GlobalFree(hInput);
		return NULL;
	}

	if ((mi->pcmWaveFormat.wBitsPerSample!=8) && (mi->pcmWaveFormat.wBitsPerSample!=16))  // take this second part out for final release
	{
		CloseHandle(hFile);
		GlobalUnlock(hInput);
		GlobalFree(hInput);
		return NULL; // can only do 8-bit mu-law and a-law for now
	}

	mi->wChannels=mi->pcmWaveFormat.nChannels;
	mi->lSamprate=mi->pcmWaveFormat.nSamplesPerSec;
	mi->wBitsPerSample=16;
	mi->wBlockAlign=(mi->wChannels*mi->wBitsPerSample/8);
	mi->lFact=lFact;
	mi->lSize=mi->dwDataSize*2;  // account for expansion of alaw to pcm

	mi->wBitsPerSample=16; // maybe 16 eh?
	mi->wBlockAlign=(mi->wChannels*mi->wBitsPerSample/8);
	mi->bytesleft=mi->lSize;
	// ADPCM	
	mi->wBitsPerSample=16; // maybe 16 eh?
	mi->wBlockAlign=(mi->wChannels*mi->wBitsPerSample/8);
	mi->bytesleft=mi->lSize;
	mi->dwFormat=(DWORD)mi->pcmWaveFormat.wFormatTag;

	*lChunkSize=30000;

	*lSR=mi->lSamprate;  
	*wBPS=mi->wBitsPerSample;
	*wC=mi->wChannels;

	SetFilePointer(hFile,mi->dwDataOffset,NULL,FILE_BEGIN);    
	mi->dwFilePos=mi->dwDataOffset;
	GlobalUnlock(hInput);

	return hInput;
}

__declspec(dllexport) void WINAPI CloseFilterInput(HANDLE hInput)
{
	if (hInput)
	{
		MYINPUT far *mi;
		mi=(MYINPUT *)GlobalLock(hInput);
		if (mi->hFile!=INVALID_HANDLE_VALUE)
		{
			CloseHandle(mi->hFile);
			mi->hFile=INVALID_HANDLE_VALUE;
		}
		GlobalUnlock(hInput);
		GlobalFree(hInput);
	}
}


short alaw[256]=
{-688,-656,-752,-720,-560,-528,-624,-592,-944,-912,-1008,-976,
-816,-784,-880,-848,-344,-328,-376,-360,-280,-264,-312,-296,
-472,-456,-504,-488,-408,-392,-440,-424,-2752,-2624,-3008,-2880,
-2240,-2112,-2496,-2368,-3776,-3648,-4032,-3904,-3264,-3136,
-3520,-3392,-1376,-1312,-1504,-1440,-1120,-1056,-1248,-1184,
-1888,-1824,-2016,-1952,-1632,-1568,-1760,-1696,-43,-41,-47,-45,
-35,-33,-39,-37,-59,-57,-63,-61,-51,-49,-55,-53,-11,-9,-15,
-13,-3,-1,-7,-5,-27,-25,-31,-29,-19,-17,-23,-21,-172,-164,
-188,-180,-140,-132,-156,-148,-236,-228,-252,-244,-204,-196,
-220,-212,-86,-82,-94,-90,-70,-66,-78,-74,-118,-114,-126,-122,
-102,-98,-110,-106,688,656,752,720,560,528,624,592,944,912,
1008,976,816,784,880,848,344,328,376,360,280,264,312,296,472,
456,504,488,408,392,440,424,2752,2624,3008,2880,2240,2112,
2496,2368,3776,3648,4032,3904,3264,3136,3520,3392,1376,1312,
1504,1440,1120,1056,1248,1184,1888,1824,2016,1952,1632,1568,
1760,1696,43,41,47,45,35,33,39,37,59,57,63,61,51,49,55,53,
11,9,15,13,3,1,7,5,27,25,31,29,19,17,23,21,172,164,188,
180,140,132,156,148,236,228,252,244,204,196,220,212,
86,82,94,90,70,66,78,74,118,114,126,122,102,98,110,106};

unsigned char alawfxn(short wSample)
{
	BYTE	alaw;

	// We'll init our A-law value per the sign of the PCM sample.  A-law
	// characters have the MSB=1 for positive PCM data.  Also, we'll
	// convert our signed 16-bit PCM value to it's absolute value and
	// then work on that to get the rest of the A-law character bits.
	if (wSample < 0)
	{
		alaw = 0x00;
		wSample = -wSample;
		if (wSample < 0) 
			wSample = 0x7FFF;
	}
	else
	{
		alaw = 0x80;
	}

	// Now we test the PCM sample amplitude and create the A-law character.
	// Study the CCITT A-law for more detail.

	if (wSample >= 2048)	// 2048 <= wSample < 32768
	{
		if (wSample >= 8192)	// 8192 <= wSample < 32768
		{
			if (wSample >= 16384)	// 16384 <= wSample < 32768
			{	alaw |= 0x70 | ((wSample >> 10) & 0x0F);
			}
			else	// 8192 <= wSample < 16384
			{	alaw |= 0x60 | ((wSample >> 9) & 0x0F);
			}
		}
		else	// 2048 <= wSample < 8192	    
		{	if (wSample >= 4096)	// 4096 <= wSample < 8192
			{	alaw |= 0x50 | ((wSample >> 8) & 0x0F);
			}
			else	// 2048 <= wSample < 4096
			{	alaw |= 0x40 | ((wSample >> 7) & 0x0F);
			}
		}
	}
	else	// 0 <= wSample < 2048
	{	if (wSample >= 512)		// 512 <= wSample < 2048
		{	if (wSample >= 1024)	// 1024 <= wSample < 2048
			{	alaw |= 0x30 | ((wSample >> 6) & 0x0F);
			}
			else	// 512 <= wSample < 1024
			{	alaw |= 0x20 | ((wSample >> 5) & 0x0F);
			}
		}
		else	// 0 <= wSample < 512
		{	alaw |= 0x00 | ((wSample >> 4) & 0x1F);
		}
	}

	return alaw ^ 0x55;		// Invert even bits
}

short mulaw[256]=
{ -8031,-7775,-7519,-7263,-7007,-6751,-6495,-6239,-5983,-5727,
  -5471,-5215,-4959,-4703,-4447,-4191,-3999,-3871,-3743,-3615,
  -3487,-3359,-3231,-3103,-2975,-2847,-2719,-2591,-2463,-2335,
  -2207,-2079,-1983,-1919,-1855,-1791,-1727,-1663,-1599,-1535,
  -1471,-1407,-1343,-1279,-1215,-1151,-1087,-1023,-975, -943,
  -911, -879, -847, -815, -783, -751, -719, -687, -655, -623,
  -591, -559, -527, -495, -471, -455, -439, -423, -407, -391,
  -375, -359, -343, -327, -311, -295, -279, -263, -247, -231,
  -219, -211, -203, -195, -187, -179, -171, -163, -155, -147,
  -139, -131, -123, -115, -107, -99,  -93,  -89,  -85,  -81,
  -77,  -73,  -69,  -65,  -61,  -57,  -53,  -49,  -45,  -41,
  -37,  -33,  -30,  -28,  -26,  -24,  -22,  -20,  -18,  -16,
  -14,  -12,  -10,  -8,   -6,   -4,   -2,   0,   8031,  7775,
  7519, 7263, 7007, 6751, 6495, 6239, 5983, 5727, 5471, 5215,
  4959, 4703, 4447, 4191, 3999, 3871, 3743, 3615, 3487, 3359,
  3231, 3103, 2975, 2847, 2719, 2591, 2463, 2335, 2207, 2079,
  1983, 1919, 1855, 1791, 1727, 1663, 1599, 1535, 1471, 1407, 1343, 1279, 1215, 1151, 1087, 1023, 
  975,  943,  911,  879,  847,  815,  783,  751,  719,  687,  655,  623,  591,  559,  527,  495,  
  471,  455,  439,  423,  407,  391,  375,  359,  343,  327,  311,  295,  279,  263,  247,  231,   // <480, =207- (sig-224)/16  
  219,  211,  203,  195,  187,  179,  171,  163,  155,  147,  139,  131,  123,  115,  107,  99,    // <224, =223- (sig-96)/8
  93,   89,   85,   81,   77,   73,   69,   65,   61,   57,   53,   49,   45,   41,   37,   33,    // <96,  =239- (sig-32)/4    for 224-239
  30,   28,   26,   24,   22,   20,   18,   16,   14,   12,   10,   8,    6,    4,    2,    0};    // <32,  =255-sig/2          for 240-255


unsigned char mulawfxn(short sig)
{
	static short error=0;
	short maxdif=20000;
	short minat=0;
	short lo=0;
	short hi=255;
	short addin=0;

	if (sig<0)
	{	sig=-sig;
		addin=128;
	}
	if (sig<32)
	{	minat=255-(sig>>1);
	}
	else if (sig<96)
	{	minat=239-((sig-32)>>2);
	}
	else if (sig<224)
	{	minat=223-((sig-96)>>3);
	}
	else if (sig<480)
	{	minat=207-((sig-224)>>4);
	}
	else if (sig<992)
	{	minat=191-((sig-480)>>5);
	}
	else if (sig<2016)
	{	minat=175-((sig-992)>>6);
	}
	else if (sig<4064)
	{	minat=159-((sig-2016)>>7);
	}
	else if (sig<8160)
	{	minat=143-((sig-4064)>>8);
	}
	else
	{	minat=128;
	}
	return (unsigned char)(minat-addin);
}


long vocWaveToTemp(MYINPUT * mi, unsigned char far *data, long readthismany, long far * bytesleft, WORD wChannels, DWORD dwFormat)
{
	DWORD read=0;
	DWORD actual=0;
	short far *ibuf;
	unsigned char far *cbuf;
	long readlength;
	DWORD offset;
	DWORD dwBytesRead;
	
	ibuf=(short far *)data;
	readlength=readthismany/2L;
	cbuf=data+readlength;
	
	ReadFile(mi->hFile,cbuf,readlength,&dwBytesRead,NULL);
	
	if (dwFormat==6)
	{	for(offset=0;offset<dwBytesRead;++offset)
			ibuf[offset]=alaw[cbuf[offset]]*8;
	}
	else if (dwFormat==7)
	{	for(offset=0;offset<dwBytesRead;++offset)
			ibuf[offset]=mulaw[cbuf[offset]]*4;
	}
	return dwBytesRead*2;
}

long vocTempToWave(MYOUTPUT * mo, unsigned char far *data, long writethismany, WORD wOrigBitsPerSample, DWORD dwFormat)
{
	DWORD written;
	long length;
	short offset;
	
	if (dwFormat==6) // ALaw
	{
		if (wOrigBitsPerSample==8)
		{
			length=writethismany;		// length in samples
			for(offset=0;offset<length;++offset)
				data[offset]=alawfxn(((short)(data[offset]-128)));
			WriteFile(mo->hFile,data,length,&written,NULL);
		}
		else if (wOrigBitsPerSample==16)
		{
			short far *ibuf;
			ibuf=(short far *)data;
		 	length=writethismany/2;		// length in samples
			for(offset=0;offset<length;++offset)
			    data[offset]=alawfxn(ibuf[offset]);
			WriteFile(mo->hFile,data,length,&written,NULL);
			written*=2;
		}		  
	}
	else if (dwFormat==7) // mu-Law
	{
		if (wOrigBitsPerSample==8)
		{
			length=writethismany;		// length in samples
			for(offset=0;offset<length;++offset)
				data[offset]=mulawfxn((short)((data[offset]-128)*64));
			WriteFile(mo->hFile,data,length,&written,NULL);
		}
		else if (wOrigBitsPerSample==16)
		{
			short far *ibuf;
			unsigned char cCoded;
			short dither=0;
			short dithrnd=0;
			long lVal;
			ibuf=(short far *)data;
			length=writethismany/2;		// length in samples
			for(offset=0;offset<length;++offset)
			{
				lVal=(long)ibuf[offset];
				
				// put bounds checking here for lVal
				cCoded=mulawfxn((short)(lVal/4));
				data[offset]=cCoded;
			}
			WriteFile(mo->hFile,data,length,&written,NULL);
			written*=2;
		}
	
	}
	return (long)written;
}


__declspec(dllexport) DWORD WINAPI ReadFilterInput(HANDLE hInput, unsigned char far *buf, long lBytes)
{
	DWORD read=0L;
	if (hInput)
	{
		MYINPUT far *mi;
		mi=(MYINPUT far *)GlobalLock(hInput);
		
		read=vocWaveToTemp(mi,buf,lBytes,(long far *)&mi->bytesleft,mi->wChannels,mi->dwFormat);

		GlobalUnlock(hInput);
	}
	return read;	
}


__declspec(dllexport) DWORD WINAPI WriteFilterOutput(HANDLE hOutput, unsigned char far *buf, long lBytes)
{
	DWORD written=0L;

	if (hOutput)
	{
		MYOUTPUT far *mo;
		mo=(MYOUTPUT far *)GlobalLock(hOutput);

		// since special data can be first or last, we will start data chunk at first Write call
		if (!mo->bWroteDataChunk)
		{
			RIFFSTUFF * pRiff = &mo->riff;
			// If we were in a previous list, must update size and such		
			if ((pRiff->szCurrentList[0]) && (strcmp(pRiff->szCurrentList,"WAVE")!=0)) 
			{
				DWORD dwBytesWritten;
				DWORD dwLength=(DWORD)(SetFilePointer64(pRiff->hFile,0,FILE_CURRENT)-pRiff->dwListStart+4);
				SetFilePointer64(pRiff->hFile,pRiff->dwListStart-8,FILE_BEGIN);
				WriteFile(pRiff->hFile,&dwLength,4,&dwBytesWritten,NULL);
				SetFilePointer64(pRiff->hFile,pRiff->dwListStart+dwLength-4,FILE_BEGIN);
				strcpy(pRiff->szCurrentList,"");
			}

			RiffStuffEnter(&mo->riff, "data");
			mo->bWroteDataChunk = TRUE;
		}

		if (strcmp(mo->riff.szCurrentChunk,"data")==0)
		{
			written=(DWORD)vocTempToWave(mo,buf,lBytes,mo->wOrigBitsPerSample,mo->dwFormat);
			if ((long)written<0)
				written=(DWORD)-1;
			else
				mo->dwBytesWritten+=written;
		}

		GlobalUnlock(hOutput);
	}
	return written;
}

