/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2003-2007 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

//////////////////////////////////////////////////////
// GENERAL PURPOSE DEFS FOR CREATING CUSTOM FILTERS //
//////////////////////////////////////////////////////
#ifndef __FILT_FILTERS_H__
#define __FILT_FILTERS_H__

typedef struct 
{
	HANDLE hSpecialData;	
	HANDLE hData;			// Actual data handle (not used anymore)
	DWORD  dwSize;			// size of data in handle (now size returned in szDataIn)
	DWORD  dwExtra;			// optional extra data (usually a count)
	char  szListType[8];	// Parent list type (usually "WAVE" or "INFO", or "adtl")
	char  szType[8];		// Usually a four character code for data, but can be up to 7 chars
	LPSTR  szDataIn;	    // User provided data handle (null to have DLL allocate it - don't do this!)
	DWORD  dwSizeIn;		// Size of handle passed in
} SPECIALDATA;

// "CUE " dwExtra=number of cues, each cue is 8 bytes		([4] name [4] sample offset)
// "LTXT" dwExtra=number of items, each one is 8 bytes		([4] ltxt len [4] name [4] cue length [4] purpose [n] data)
// "NOTE" dwExtra=number of strings, each one is n bytes	([4] name [n-4] length zero term)
// "LABL" dwExtra=number of strings, each one is n bytes	([4] name [n-4] length zero term)
// "PLST" dwExtra=number if items, each one is 16 bytes		([4] name [4] dwLen [4] dwLoops [4] dwMode)
                                           
#define COOLQUERY_RATE_NONE     (0x0000)     // nothing
#define COOLQUERY_RATE_ALL      (0xFFFF)     // everything
#define COOLQUERY_RATE_005500   (0x0001)     // 5500 (5512.5)                            
#define COOLQUERY_RATE_011025   (0x0002)     // 11025 (11K)                              
#define COOLQUERY_RATE_022050   (0x0004)     // 22050 (22K)                              
#define COOLQUERY_RATE_032075   (0x0008)     // 32075 (32K, or 32000)                    
#define COOLQUERY_RATE_044100   (0x0010)     // 44100 (44K)                              
#define COOLQUERY_RATE_048000   (0x0020)     // 48000 (48K)                              
#define COOLQUERY_RATE_088200   (0x0040)     // 88200 (88K)   (future ultra-sonic rates?)
#define COOLQUERY_RATE_096000   (0x0080)     // 96000 (96K)                              
#define COOLQUERY_RATE_132300   (0x0100)     // 132300 (132K)                            
#define COOLQUERY_RATE_176400   (0x0200)     // 176400 (176K)                            

typedef struct
{
    WCHAR szName[24];
    WCHAR szCopyright[80];

    // rate table, bits are set for modes that can be handled
    WORD Quad32;  // Quads are 3-D encoded
    WORD Quad16;
    WORD Quad8;
    WORD Stereo8;    		
    WORD Stereo12;   		
    WORD Stereo16;   		
    WORD Stereo24;   		
    WORD Stereo32;   		
    WORD Mono8;      		
    WORD Mono12;	  		
    WORD Mono16;	  		
    WORD Mono24;	  		
    WORD Mono32;     		
	 				    
    DWORD dwFlags;
    WCHAR szExt[4];
    long lChunkSize;
    WCHAR szExt2[4];
    WCHAR szExt3[4];
    WCHAR szExt4[4];
} COOLQUERY;

#ifndef COPYRIGHTYEAR
#define COPYRIGHTYEAR	L"2007"
#endif

#define R_5500   1
#define R_11025  2
#define R_22050  4
#define R_32075  8
#define R_44100  16
#define R_48000  32
#define R_88200  64
#define R_96000  128
#define R_132300 256
#define R_176400 512

#define C_VALIDLIBRARY 2000

// The following table lists prior relevant versions of Audition and Cool Edit,
// the C_VALIDLIBRARY value for each, and some comments about each:

// Cool Edit				1152	oldest version supported by Audition
// Audition 1.0 & 1.5		1155	ascii API support
// Audition 2.0 & 3.0		2000	unicode API support

// For more details regarding these C_VALIDLIBRARY values, please see the File Filter SDK documentation.

// The FilterReadSpecial, FilterWriteSpecial, FilterCanReadSpecial and FilterCanWriteSpecial
// APIs are no longer supported by Audition.  If you are reading or writing extra data, you
// must implement the relevant new APIs, which are:
// FilterGetFirstSpecialData, FilterGetNextSpecialData, and FilterWriteSpecialData.

#define QF_RATEADJUSTABLE		0x00001	// if can handle non-standard sample rates
										// if not, only rates in bit rate table understood
#define QF_CANSAVE				0x00002
#define QF_CANLOAD				0x00004
#define QF_UNDERSTANDSALL		0x00008	// will read ANYTHING, so it is the last resort if no other formats match
#define QF_READSPECIALFIRST		0x00010	// read special info before trying to read data
#define QF_READSPECIALLAST		0x00020	// read special info after reading data
#define QF_WRITESPECIALFIRST	0x00040	// when writing a file, special info is sent to DLL before data
#define QF_WRITESPECIALLAST		0x00080	// when writing, special info is sent to DLL after data
#define QF_HASOPTIONSBOX		0x00100	// set if options box implemented
#define QF_NOASKFORCONVERT		0x00200	// set to bypass asking for conversion if original in different rate, auto convert
#define QF_NOHEADER				0x00400	// set if this is a raw data format with no header
#define QF_CANDO32BITFLOATS		0x00800	// set if file format can handle 32-bit sample data for input
#define QF_CANOPENVIRTUAL		0x01000	// Set if data is in Intel 8-bit or 16-bit sample format, or floats
										// and the GetDataOffset() function is implemented

#define QF_IGNOREMRU			0x04000	// Do not add this to MRU list if opening a file of this format.
#define QF_NONLOSSY				0x10000	// Set this if the format is not a lossy compression
#define QF_FLOATDATANORMALIZED  0x20000	// Set this if the format's floating point data is +/-1.0 range. (CEP2.0)
#define QF_PRESERVEWORKINGDIR	0x40000
#define QF_TRYREADPASTEND		0x80000	// Set this if you can not give a good estimate of file size at file open time


// special types are read from and written to DLL in the order below
// special types (particular to Windows waveforms)

#define SP_IART  20
#define SP_ICMT  21
#define SP_ICOP  22
#define SP_ICRD  23
#define SP_IENG  24
#define SP_IGNR  25
#define SP_IKEY  26
#define SP_IMED  27
#define SP_INAM  28
#define SP_ISFT  29
#define SP_ISRC  30
#define SP_ITCH  31
#define SP_ISBJ  32
#define SP_ISRF  33
#define SP_DISP  34
#define SP_CUE   40 // returns number of cues of size cue_type 
#define SP_LTXT  41 // returns number of adtl texts of size 8 (4,id and 4,len)
#define SP_NOTE  42 // returns LO=size, HI=number of strings (sz sz sz...)
#define SP_LABL	 43 // returns LO=size, HI=number of strings (sz sz sz...)
#define SP_PLST  44 // returns number of playlist entries size play_type 

// For special data, .FLT must implement FilterGetFirstSpecialData and FilterGetNextSpecialData
typedef DWORD           FOURCC;         // a four character code

struct cue_type  { DWORD dwName;
				   DWORD dwPosition;
				   FOURCC fccChunk;
				   DWORD dwChunkStart;
				   DWORD dwBlockStart;
				   DWORD dwSampleOffset;
				 };

struct play_type { DWORD dwName;
				   DWORD dwLength;
				   DWORD dwLoops;
				 };

#define FILTERGETOPTIONS2F_OUTPUT	1
#define FILTERGETOPTIONS2F_INPUT	2

#endif // __FILT_FILTERS_H__
