/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2003-2007 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

#include <windows.h>
#include <assert.h>
#include "..\..\AuditionSDK\FLT_File_Filter\Filters.h"
#include "riffstuff.h"

// This does an unsigned move to get us over the 2GB seek boundary
DWORD SetFilePointer2(HANDLE hFile, DWORD dwLow, DWORD dwHigh, DWORD dwMethod)
{
	return SetFilePointer(hFile, dwLow, &dwHigh, dwMethod);
}

int64_t SetFilePointer64(HANDLE hFile, int64_t qiMoveTo, DWORD dwMethod)
{
	LONG lMoveHigh = (LONG)(qiMoveTo >> 32);
	LONG lMoveLow = (LONG)(qiMoveTo & 0xffffffffLL);
	DWORD dwLowMoved = SetFilePointer(hFile, lMoveLow, &lMoveHigh, dwMethod);
	return dwLowMoved | (((int64_t)lMoveHigh) << 32);
}

void RiffStuffEnter(RIFFSTUFF * pRiff, LPCSTR szType)
{	DWORD dwBytesWritten;
	DWORD dwZero=0;
	lstrcpyA(pRiff->szCurrentChunk, szType);
	WriteFile(pRiff->hFile,szType,4,&dwBytesWritten,NULL);
	WriteFile(pRiff->hFile,&dwZero,4,&dwBytesWritten,NULL);
	pRiff->dwChunkStart=SetFilePointer64(pRiff->hFile,0,FILE_CURRENT);
}

void RiffStuffExit(RIFFSTUFF * pRiff)
{	DWORD dwBytesWritten;
	int iPad=0;
	DWORD dwLength=(DWORD)(SetFilePointer64(pRiff->hFile,0,FILE_CURRENT) - pRiff->dwChunkStart);
	DWORD dwZero=0;
	if (dwLength%2) // pad to a word while still giving previous length
	{	WriteFile(pRiff->hFile,&dwZero,1,&dwBytesWritten,NULL);		
		iPad=1;
	}
	SetFilePointer64(pRiff->hFile,pRiff->dwChunkStart-4,FILE_BEGIN);
	WriteFile(pRiff->hFile,&dwLength,4,&dwBytesWritten,NULL);
	SetFilePointer64(pRiff->hFile,pRiff->dwChunkStart+dwLength+iPad,FILE_BEGIN);
	*pRiff->szCurrentChunk = 0;
}

BOOL RiffStuffOpenInput(RIFFSTUFF * pRiff, HANDLE hFile, int64_t dwFullSize, int64_t dwChunkStart)
{
	pRiff->hFile = hFile;
	pRiff->dwFullSize = dwFullSize;
	pRiff->dwChunksStart = dwChunkStart;
	pRiff->iNumSpecialPending = 0;
	return TRUE;
}
	
BOOL RiffStuffOpenOutput(RIFFSTUFF * pRiff, HANDLE hFile, DWORD dwListStart)
{
	pRiff->hFile = hFile;
	pRiff->dwListStart=dwListStart;
	*pRiff->szCurrentList = 0; // top level "WAVE"		
	return TRUE;
}
	

BOOL RiffStuffCloseOutput(RIFFSTUFF * pRiff)
{		
	// If we were just writing data before, exit that data chunk now
	if (lstrcmpA(pRiff->szCurrentChunk, "data")==0)
	{
		RiffStuffExit(pRiff);
	}
	else	// If we were in a previous list, must update size and such		
		 if ((pRiff->szCurrentList[0]) && (lstrcmpA(pRiff->szCurrentList,"WAVE")!=0)) 
	{	
		DWORD dwBytesWritten;
		DWORD dwLength=(DWORD)(SetFilePointer64(pRiff->hFile,0,FILE_CURRENT)-pRiff->dwListStart+4);
		SetFilePointer64(pRiff->hFile,pRiff->dwListStart-8,FILE_BEGIN);
		WriteFile(pRiff->hFile,&dwLength,4,&dwBytesWritten,NULL);
		SetFilePointer64(pRiff->hFile,pRiff->dwListStart+dwLength-4,FILE_BEGIN);
		*pRiff->szCurrentList = 0;
	}

	return TRUE;
}
		

DWORD RiffStuffWriteSpecialData(RIFFSTUFF * pRiff, LPCSTR szListType, LPCSTR szType, 
												   char * pData,DWORD dwSize)
{
	DWORD dwBytesWritten;
	DWORD dwZero=0; // placeholder
	DWORD dwLength;

	if (!pRiff)
		return 0;

	// Exit data since we must be done writing data
	if (lstrcmpA(pRiff->szCurrentChunk, "data")==0)
	{
		RiffStuffExit(pRiff);
	}
		
	if (lstrcmpA(pRiff->szCurrentList,szListType)!=0) // changing lists
	{	// if we were in a previous list, must update length in id
		if ((pRiff->szCurrentList[0]) && (lstrcmpA(pRiff->szCurrentList,"WAVE")!=0)) 
		{	dwLength=(DWORD)(SetFilePointer64(pRiff->hFile,0,FILE_CURRENT)-pRiff->dwListStart+4);
			SetFilePointer64(pRiff->hFile,pRiff->dwListStart-8,FILE_BEGIN);
			WriteFile(pRiff->hFile,&dwLength,4,&dwBytesWritten,NULL);
			SetFilePointer64(pRiff->hFile,pRiff->dwListStart+dwLength-4,FILE_BEGIN);
			*pRiff->szCurrentList = 0;
		}
		// not in any list now, enter the new one we are asking for, or stay at this level if "WAVE"
		lstrcpyA(pRiff->szCurrentList,szListType);
		if ((*szListType) && (lstrcmpA(szListType,"WAVE")!=0))
		{	WriteFile(pRiff->hFile,"LIST",4,&dwBytesWritten,NULL);
			WriteFile(pRiff->hFile,&dwZero,4,&dwBytesWritten,NULL);
			WriteFile(pRiff->hFile,szListType,4,&dwBytesWritten,NULL);
			pRiff->dwListStart=SetFilePointer64(pRiff->hFile,0,FILE_CURRENT);
		}
	}
	
	if (lstrcmpA(szListType,"INFO")==0)
	{	RiffStuffEnter(pRiff, szType);
		WriteFile(pRiff->hFile,pData,dwSize,&dwBytesWritten,NULL);
		//if (dwSize%2) // pad to a word
		//	WriteFile(pRiff->hFile,&dwZero,1,&dwBytesWritten,NULL);
		RiffStuffExit(pRiff);
	}
	else if (lstrcmpA(szListType,"adtl")==0)
	{	// ltxt, note, and labl chunks
		if (lstrcmpA(szType,"ltxt")==0)
		{	// "LTXT" dwExtra=number of items, each one is 8 bytes		
			// ([4] ltxt len [4] name [4] cue length [4] purpose [n] data)
			DWORD dwLtxtSize;
			DWORD * pdwData=(DWORD *)pData;
			DWORD dwTextLen;
			// use Zero for country,language,dialect, codepage
			while (1)
			{	RiffStuffEnter(pRiff, szType);
				dwLtxtSize=*pdwData++;
				// writing name,length,purpose
				WriteFile(pRiff->hFile,pdwData,3*sizeof(DWORD),&dwBytesWritten,NULL);
				pdwData+=3;
				dwTextLen=dwLtxtSize-16;
				// write country,language,dialect,codepage
				WriteFile(pRiff->hFile,&dwZero,4,&dwBytesWritten,NULL);
				WriteFile(pRiff->hFile,&dwZero,4,&dwBytesWritten,NULL);
				if (dwTextLen) // extra data too!
				{	WriteFile(pRiff->hFile,pdwData,dwTextLen,&dwBytesWritten,NULL);
					((char *)pdwData)+=dwTextLen;
				}
				//if (dwTextLen%2) // pad to 2 bytes
				//	WriteFile(pRiff->hFile,&dwZero,1,&dwBytesWritten,NULL);
				dwSize-=dwLtxtSize;
				RiffStuffExit(pRiff);
				if ((long)dwSize<=0)
					break;
			}
		}
		else if ((lstrcmpA(szType,"labl")==0) || (lstrcmpA(szType,"note")==0))
		{	// "LABL"/"NOTE" dwExtra=number of strings, each one is n bytes	
			// ([4] name [n-4] length zero term)
			while (1)
			{	DWORD dwLength=4+lstrlenA(pData+4)+1;
				RiffStuffEnter(pRiff, szType);				
				WriteFile(pRiff->hFile,pData,dwLength,&dwBytesWritten,NULL);
				dwSize-=dwLength;
				pData+=dwLength;
				//if (dwLength%2) // pad
				//	WriteFile(pRiff->hFile,&dwZero,1,&dwBytesWritten,NULL);
				RiffStuffExit(pRiff);
				if ((long)dwSize<=0)
					break;
			}
		}
		else if ((lstrcmpA(szType,"file")==0))
		{
			while (1)
			{
				DWORD dwFileSize = *((DWORD *)pData);
				pData += 4;
				dwSize -= 4;
				if (dwSize < dwFileSize)
					break;
				RiffStuffEnter(pRiff, szType);
				WriteFile(pRiff->hFile, pData, dwFileSize, &dwBytesWritten, NULL);
				dwFileSize=((dwFileSize+1)/2)*2;
				dwSize-=dwFileSize;
				pData+=dwFileSize;
				RiffStuffExit(pRiff);
				if ((long)dwSize<=0)
					break;
			}
		}
		// else we IGNORE IT!!!!
	}
	else if ((lstrcmpA(szListType,"WAVE")==0) && (lstrcmpA(szType,"cue ")==0))
	{	// "CUE " dwExtra=number of cues, each cue is 8 bytes		
		// ([4] name [4] sample offset)
		DWORD * pdwData=(DWORD *)pData;			
		DWORD dwCt=dwSize/(2*sizeof(DWORD));
		char szChunk[5]={"data"};
		RiffStuffEnter(pRiff, szType);
		WriteFile(pRiff->hFile,&dwCt,4,&dwBytesWritten,NULL);			
		while (1)
		{	DWORD dwName = *pdwData++;
			DWORD dwOffset = *pdwData++;
			WriteFile(pRiff->hFile,&dwName,4,&dwBytesWritten,NULL);
			WriteFile(pRiff->hFile,&dwOffset,4,&dwBytesWritten,NULL);
			WriteFile(pRiff->hFile,szChunk,4,&dwBytesWritten,NULL);
			WriteFile(pRiff->hFile,&dwZero,4,&dwBytesWritten,NULL);
			WriteFile(pRiff->hFile,&dwZero,4,&dwBytesWritten,NULL);
			WriteFile(pRiff->hFile,&dwOffset,4,&dwBytesWritten,NULL);			
			dwSize-=2*sizeof(DWORD);
			if ((long)dwSize<=0)
				break;
		}
		RiffStuffExit(pRiff);
	}
	else if ((lstrcmpA(szListType,"WAVE")==0) && (lstrcmpA(szType,"plst")==0))
	{	// "PLST" dwExtra=number if items, each one is 16 bytes		
		// ([4] name [4] dwLen [4] dwLoops [4] dwMode)
		DWORD * pdwData=(DWORD *)pData;			
		DWORD dwCt=dwSize/(4*sizeof(DWORD));
		char szChunk[5]={"data"};
		RiffStuffEnter(pRiff, szType);
		WriteFile(pRiff->hFile,&dwCt,4,&dwBytesWritten,NULL);			
		while (1)
		{	DWORD dwName = *pdwData++;
			DWORD dwLength = *pdwData++;
			DWORD dwLoops = *pdwData++;
			DWORD dwMode = *pdwData++;
			WriteFile(pRiff->hFile,&dwName,4,&dwBytesWritten,NULL);
			WriteFile(pRiff->hFile,&dwLength,4,&dwBytesWritten,NULL);
			WriteFile(pRiff->hFile,&dwLoops,4,&dwBytesWritten,NULL);
			dwSize-=4*sizeof(DWORD);
			if ((long)dwSize<=0)
				break;
		}
		RiffStuffExit(pRiff);
	}
	else // unknown chunk (doesn't matter if it is in a list or not)
	{	RiffStuffEnter(pRiff, szType);
		WriteFile(pRiff->hFile,pData,dwSize,&dwBytesWritten,NULL);
		//if (dwSize%2) // pad to a word
		//	WriteFile(pRiff->hFile,&dwZero,1,&dwBytesWritten,NULL);
		RiffStuffExit(pRiff);
	}	
	return 1; // OK
}

// Note, mi->dwFilePos is not updated in this next call
DWORD RiffStuffGetNextSpecialData(RIFFSTUFF * pRiff, SPECIALDATA * psp)
{	
	DWORD dwSuccess=1; // 1=success
	BOOL bNoMore=FALSE;
	RIFFWAVEDATA * pwd=NULL;
	BOOL bOk=FALSE;			
	DWORD dwReadBytes;
		
	// First send back stuff we already have accumulated
	if (pRiff->iNumSpecialPending) 
	{
		psp->dwSize=pRiff->spPending[(pRiff->iNumSpecialPending)-1].dwSize;
		if (psp->dwSize > psp->dwSizeIn)
		{
			// This is a failure, we need to get more memory from the caller
			return -1;
		}

		pRiff->iNumSpecialPending--;
		psp->hData=pRiff->spPending[pRiff->iNumSpecialPending].hData;
		psp->dwExtra=pRiff->spPending[pRiff->iNumSpecialPending].dwExtra;
		lstrcpyA(psp->szType,pRiff->spPending[pRiff->iNumSpecialPending].szType);		
		lstrcpyA(psp->szListType,pRiff->spPending[pRiff->iNumSpecialPending].szListType);			
		{
			// Now free our memory since we allocated it within _THIS_ DLL
			LPSTR szFrom=GlobalLock(psp->hData);
			LPSTR szTo=psp->szDataIn; //GlobalLock(psp->hDataIn);
			assert(psp->dwSize <= psp->dwSizeIn);
			psp->dwSize=min(psp->dwSize,psp->dwSizeIn);
			CopyMemory(szTo,szFrom,psp->dwSize);
			GlobalUnlock(psp->hData);
			//GlobalUnlock(psp->hDataIn);
			GlobalFree(psp->hData);
			psp->hData=NULL;
		}

		return dwSuccess;
	}

	pwd=(RIFFWAVEDATA *)GlobalLock(psp->hSpecialData);

	if (pwd->dwListEnd) // reading inside an INFO LIST chunk
	{	lstrcpyA(psp->szType,"LIST");
	}

	while (!bOk)
	{	BOOL bSkipTopLevelAdvance=FALSE;
		if (pwd->dwOffset>=8+pwd->dwRiffLength)
		{	bNoMore=TRUE;
			break;
		}
		
		bOk=TRUE;

		if (!pwd->dwListEnd)
		{	// work with both padded and non-padded formats
			if (pwd->dwOffset%2) // if odd, advance if zero-byte
			{	ReadFile(pRiff->hFile,psp->szType,1,&dwReadBytes,NULL);
				if (!(*psp->szType)) // padded format
				{	pwd->dwOffset++;
					ReadFile(pRiff->hFile,psp->szType,4,&dwReadBytes,NULL);
				}
				else // non-padded format
				{	ReadFile(pRiff->hFile,psp->szType+1,3,&dwReadBytes,NULL);
					dwReadBytes++;
				}
			}
			else
				ReadFile(pRiff->hFile,psp->szType,4,&dwReadBytes,NULL);
			psp->szType[4]=0x00;
			lstrcpyA(psp->szListType,"WAVE");
			psp->hData=NULL;
			psp->dwExtra=1;
			ReadFile(pRiff->hFile,&pwd->dwChunkSize,4,&dwReadBytes,NULL);
		}
		
		if ((lstrcmpA(psp->szType,"fmt ")==0) || 
			(lstrcmpA(psp->szType,"data")==0) ||
			(lstrcmpA(psp->szType,"fact")==0))
		{
			bOk=FALSE;	// loop again while advancing
			//bSkipTopLevelAdvance=FALSE;
		}
		else if (lstrcmpA(psp->szType,"JUNQ")==0) // Padding, let's get rid of it
		{
			bOk=FALSE;
		}
		else if (lstrcmpA(psp->szType,"cue ")==0) // entire cue list
		{	DWORD dwCueData[6];
			DWORD t;
			DWORD * pdwOut;
			ReadFile(pRiff->hFile,&psp->dwExtra,4,&dwReadBytes,NULL);
			if (dwReadBytes!=4)
			{	bNoMore=TRUE;
				break;
			}
			psp->dwSize=2*sizeof(DWORD)*psp->dwExtra;
			//psp->hData=GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT,psp->dwSize);
			pdwOut=(DWORD *)psp->szDataIn; //GlobalLock(psp->hData);
			lstrcpyA(psp->szListType,"WAVE");
			// dwName
			// dwPosition	(sample offset copy)
			// fccChunk		('data')
			// dwChunkStart	0
			// dwBlockStart 0
			// dwSampleIffset actual sample offset
			for (t=0;t<psp->dwExtra;t++)
			{	ReadFile(pRiff->hFile,dwCueData,6*sizeof(DWORD),&dwReadBytes,NULL);
				if (dwReadBytes!=6*sizeof(DWORD))
				{	bNoMore=TRUE;
					break;
				}				
				*pdwOut++=dwCueData[0];
				*pdwOut++=dwCueData[5];
			}
			//GlobalUnlock(psp->hData);
			psp->hData=NULL;
			bOk=TRUE;
			//bSkipTopLevelAdvance=FALSE;
		}
		else if (lstrcmpA(psp->szType,"plst")==0) // entire play list
		{	DWORD dwPlstData[3];
			DWORD t;
			DWORD * pdwOut;
			ReadFile(pRiff->hFile,&psp->dwExtra,4,&dwReadBytes,NULL); // count
			if (dwReadBytes!=4)
			{	bNoMore=TRUE;
				break;
			}			
			psp->dwSize=4*sizeof(DWORD)*psp->dwExtra;
			//psp->hData=GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT,psp->dwSize);
			lstrcpyA(psp->szListType,"WAVE");			
			pdwOut=(DWORD *)psp->szDataIn; //GlobalLock(psp->hData);
			// dwName
			// dwLength	(in samples)
			// dwLoops		
			for (t=0;t<psp->dwExtra;t++)
			{	ReadFile(pRiff->hFile,dwPlstData,3*sizeof(DWORD),&dwReadBytes,NULL);
				if (dwReadBytes!=3*sizeof(DWORD))
				{	bNoMore=TRUE;
					break;
				}							
				*pdwOut++=dwPlstData[0];
				*pdwOut++=dwPlstData[1];
				*pdwOut++=dwPlstData[2];
				*pdwOut++=0; // should be 1 for backwards, 2 for forw-back
			}
			//GlobalUnlock(psp->hData);
			psp->hData=NULL;
			bOk=TRUE;
			//bSkipTopLevelAdvance=FALSE;
		}
		else if (lstrcmpA(psp->szType,"LIST")==0)	// LISTS like 'adtl' list and 'INFO' list
		{	int64_t dwFilePos;
			char szListType[5]={"xxxx"};
			if (!pwd->dwListEnd)
			{	pwd->dwListEnd=pwd->dwOffset+8+pwd->dwChunkSize;
				ReadFile(pRiff->hFile,szListType,4,&dwReadBytes,NULL);
				if (dwReadBytes!=4)
				{	bNoMore=TRUE;
					break;
				}
				if (pwd->dwChunkSize <= 4)
				{
					bSkipTopLevelAdvance=FALSE;
					strcpy(szListType,"xxxx");
				}
			}
			else
			{	lstrcpyA(szListType,psp->szListType);
			}
				
			if (lstrcmpA(szListType,"INFO")==0)
			{	// just read one at a time, mark that we are in a list, and save end of list
				char szEntryType[5]={"xxxx"};
				char * pOut;
				DWORD dwTempSize;
				DWORD dwEvenSize;

				bSkipTopLevelAdvance=TRUE;
				ReadFile(pRiff->hFile,szEntryType,4,&dwReadBytes,NULL);
				if (dwReadBytes!=4)
				{	bNoMore=TRUE;
					break;
				}											

				ReadFile(pRiff->hFile,&dwTempSize,4,&dwReadBytes,NULL); // special size is size of text
				if (dwReadBytes!=4)
				{	bNoMore=TRUE;
					break;
				}							

				// JUNQ entry in the list, let's ignore
				if (lstrcmpA(szEntryType,"JUNQ")==0)
				{
					dwEvenSize=((dwTempSize+1)/2)*2;
					SetFilePointer64(pRiff->hFile,dwEvenSize,FILE_CURRENT);
					bOk = FALSE;
					continue;
				}

				psp->dwSize = dwTempSize;
				lstrcpyA(psp->szListType,szListType);
				lstrcpyA(psp->szType,szEntryType);
				//psp->hData=GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT,psp->dwSize);
				pOut=(char *)psp->szDataIn; //GlobalLock(psp->hData);
				dwEvenSize=((psp->dwSize+1)/2)*2;									
				ReadFile(pRiff->hFile,pOut,dwEvenSize,&dwReadBytes,NULL);
				if (dwReadBytes!=dwEvenSize)
				{	bNoMore=TRUE;
					break;
				}							
				
				//GlobalUnlock(psp->hData);
				psp->hData=NULL;

				// if this was last item in list, get out of list
				if (SetFilePointer64(pRiff->hFile,0,FILE_CURRENT)>=pwd->dwListEnd)
				{	bSkipTopLevelAdvance=FALSE;
					pwd->dwListEnd=0;
				}
				bOk=TRUE;
			}
			else if (lstrcmpA(szListType,"adtl")==0) // read this one all at once
			{	// Get all ...
				char szEntryType[5]={"xxxx"};
				DWORD dwSizeLtxt=0;
				DWORD dwSizeLabl=0;
				DWORD dwSizeNote=0;
				DWORD dwSizeFile=0;
				DWORD dwCtLtxt=0;
				DWORD dwCtLabl=0;
				DWORD dwCtNote=0;
				DWORD dwCtFile=0;
				HANDLE hLtxt=NULL;
				HANDLE hLabl=NULL;
				HANDLE hNote=NULL;
				HANDLE hFile=NULL;
				char * pLtxt=NULL;
				char * pLabl=NULL;
				char * pNote=NULL;
				char * pFile=NULL;
				//DWORD dwFP;
				dwFilePos=pwd->dwOffset+12;  // already seeked here
				// first count up sizes of all sections
				lstrcpyA(psp->szListType,szListType);				
				while (SetFilePointer64(pRiff->hFile,0,FILE_CURRENT)<pwd->dwListEnd)
				{	DWORD dwSiz;
					ReadFile(pRiff->hFile,szEntryType,4,&dwReadBytes,NULL);
					if (dwReadBytes!=4)
					{	bNoMore=TRUE;
						break;
					}							
				
					ReadFile(pRiff->hFile,&dwSiz,4,&dwReadBytes,NULL);
					if (dwReadBytes!=4)
					{	bNoMore=TRUE;
						break;
					}							
					
					if (lstrcmpA(szEntryType,"ltxt")==0)
					{	dwSizeLtxt+=dwSiz;
						dwCtLtxt++;
					}
					else if (lstrcmpA(szEntryType,"labl")==0)
					{	dwSizeLabl+=dwSiz;
						dwCtLabl++;
					}
					else if (lstrcmpA(szEntryType,"note")==0)
					{	dwSizeNote+=dwSiz;
						dwCtNote++;
					}
					else if (lstrcmpA(szEntryType,"file")==0)
					{
						dwSizeFile+=dwSiz;
						dwCtFile++;
					}
					dwSiz=((dwSiz+1)/2)*2;
					SetFilePointer64(pRiff->hFile,dwSiz,FILE_CURRENT);
				}
					
				SetFilePointer64(pRiff->hFile,dwFilePos,FILE_BEGIN);
				if (dwSizeLtxt)
				{	hLtxt=GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT,dwSizeLtxt+dwCtLtxt*2+16);
					pLtxt=(char *)GlobalLock(hLtxt);
				}
				if (dwSizeLabl)
				{	hLabl=GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT,dwSizeLabl+16);
					pLabl=(char *)GlobalLock(hLabl);
				}
				if (dwSizeNote)
				{	hNote=GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT,dwSizeNote+16);
					pNote=(char *)GlobalLock(hNote);
				}
				if (dwSizeFile)
				{
					hFile=GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT,dwSizeFile+dwCtFile*4+16);
					pFile=(char *)GlobalLock(hFile);
				}
				// Fill with data
				dwSizeLtxt=dwSizeLabl=dwSizeNote=dwSizeFile=0;
				while (SetFilePointer64(pRiff->hFile,0,FILE_CURRENT)<pwd->dwListEnd)
				{
					DWORD dwSiz;
					DWORD dwData[5];					
					
					ReadFile(pRiff->hFile,szEntryType,4,&dwReadBytes,NULL);
					if (dwReadBytes!=4)
					{	bNoMore=TRUE;
						break;
					}							
					
					ReadFile(pRiff->hFile,&dwSiz,4,&dwReadBytes,NULL);
					if (dwReadBytes!=4)
					{	bNoMore=TRUE;
						break;
					}							
					
					if (lstrcmpA(szEntryType,"ltxt")==0)
					{	ReadFile(pRiff->hFile,dwData,20,&dwReadBytes,NULL);
						if (dwReadBytes!=20)
						{	bNoMore=TRUE;
							break;
						}							
					
						*((DWORD *)pLtxt)=dwSiz-4; // overall length
						pLtxt+=4;						
						*((DWORD *)pLtxt)=dwData[0]; // dwName
						pLtxt+=4;
						*((DWORD *)pLtxt)=dwData[1]; // dwSampleLength
						pLtxt+=4;
						*((DWORD *)pLtxt)=dwData[2]; // dwPurpose
						pLtxt+=4;					// purpose='mark' in CE95, 'rgn ' in sf. 'capt' CC, 'scpt' script
						dwSiz=((dwSiz+1)/2)*2;
						if (dwSiz-20)
							ReadFile(pRiff->hFile,pLtxt,dwSiz-20,&dwReadBytes,NULL); // any other text

						pLtxt+=(dwSiz-20); // make sure we have some zero's at end for null term..
						dwSizeLtxt+=16+(20-dwSiz);
					}
					else if (lstrcmpA(szEntryType,"labl")==0)
					{	ReadFile(pRiff->hFile,dwData,4,&dwReadBytes,NULL);
						if (dwReadBytes!=4)
						{	bNoMore=TRUE;
							break;
						}							
						
						*((DWORD *)pLabl)=dwData[0]; // dwName
						pLabl+=4;
						dwSiz=((dwSiz+1)/2)*2;
						if (dwSiz-4)
							ReadFile(pRiff->hFile,pLabl,dwSiz-4,&dwReadBytes,NULL);
						else
							*pLabl=0x00;
						dwSizeLabl+=4+lstrlenA(pLabl)+1;
						pLabl+=lstrlenA(pLabl)+1;
						
					}
					else if (lstrcmpA(szEntryType,"note")==0)
					{	ReadFile(pRiff->hFile,dwData,4,&dwReadBytes,NULL);
						if (dwReadBytes!=4)
						{	bNoMore=TRUE;
							break;
						}							
						
						*((DWORD *)pNote)=dwData[0]; // dwName
						pNote+=4;
						dwSiz=((dwSiz+1)/2)*2;
						if (dwSiz-4)
							ReadFile(pRiff->hFile,pNote,dwSiz-4,&dwReadBytes,NULL);
						else
							*pNote=0x00;
						dwSizeNote+=4+lstrlenA(pNote)+1;
						pNote+=lstrlenA(pNote)+1; // string already null terminated													
					}
					else if (lstrcmpA(szEntryType,"file")==0)
					{
						ReadFile(pRiff->hFile,dwData,4,&dwReadBytes,NULL);
						if(dwReadBytes!=4)
						{
							bNoMore=TRUE;
							break;
						}							

						*((DWORD *)pFile)=dwSiz; // Size of the buffer
						pFile+=4;

						dwSiz=((dwSiz+1)/2)*2;

						*((DWORD *)pFile)=dwData[0]; // dwName
						pFile+=4;
						if (dwSiz-4)
							ReadFile(pRiff->hFile,pFile,dwSiz-4,&dwReadBytes,NULL);
						else
							*pFile=0x00;
						dwSizeFile+=dwSiz+4;
						pFile+=dwSiz-4;
					}
				}
					
				if (hLtxt)
					GlobalUnlock(hLtxt);
				if (hLabl)
					GlobalUnlock(hLabl);
				if (hNote)
					GlobalUnlock(hNote);
				if (hFile)
					GlobalUnlock(hFile);
				// Fill and send back
				psp->hData=NULL;
				psp->dwSize=0;
				psp->dwExtra=0;
				
				lstrcpyA(psp->szListType,szListType);				
				if (hLtxt)
				{	if (!psp->hData) // return this one
					{	psp->hData=hLtxt;
						psp->dwSize=dwSizeLtxt;
						psp->dwExtra=dwCtLtxt;
						lstrcpyA(psp->szType,"ltxt");						
					}
					else // add to pending
					{	pRiff->spPending[pRiff->iNumSpecialPending].hSpecialData=NULL; // not used
						pRiff->spPending[pRiff->iNumSpecialPending].hData=hLtxt;
						pRiff->spPending[pRiff->iNumSpecialPending].dwSize=dwSizeLtxt;
						pRiff->spPending[pRiff->iNumSpecialPending].dwExtra=dwCtLtxt; 
						lstrcpyA(pRiff->spPending[pRiff->iNumSpecialPending].szType,"ltxt");
						lstrcpyA(pRiff->spPending[pRiff->iNumSpecialPending].szListType,szListType);
						pRiff->iNumSpecialPending++;
					}
				}

				if (hLabl)
				{	if (!psp->hData) // return this one
					{	psp->hData=hLabl;
						psp->dwSize=dwSizeLabl;
						psp->dwExtra=dwCtLabl;
						lstrcpyA(psp->szType,"labl");
					}
					else // add to pending
					{	pRiff->spPending[pRiff->iNumSpecialPending].hSpecialData=NULL; // not used
						pRiff->spPending[pRiff->iNumSpecialPending].hData=hLabl;
						pRiff->spPending[pRiff->iNumSpecialPending].dwSize=dwSizeLabl;
						pRiff->spPending[pRiff->iNumSpecialPending].dwExtra=dwCtLabl;
						lstrcpyA(pRiff->spPending[pRiff->iNumSpecialPending].szType,"labl"); 					
						lstrcpyA(pRiff->spPending[pRiff->iNumSpecialPending].szListType,szListType);
						pRiff->iNumSpecialPending++;
					}
				}

				if (hNote)
				{	if (!psp->hData) // return this one
					{	psp->hData=hNote;
						psp->dwSize=dwSizeNote;
						psp->dwExtra=dwCtNote;
						lstrcpyA(psp->szType,"note");
					}
					else // add to pending
					{	pRiff->spPending[pRiff->iNumSpecialPending].hSpecialData=NULL; // not used
						pRiff->spPending[pRiff->iNumSpecialPending].hData=hNote;
						pRiff->spPending[pRiff->iNumSpecialPending].dwSize=dwSizeNote;
						pRiff->spPending[pRiff->iNumSpecialPending].dwExtra=dwCtNote; 
						lstrcpyA(pRiff->spPending[pRiff->iNumSpecialPending].szType,"note"); 					
						lstrcpyA(pRiff->spPending[pRiff->iNumSpecialPending].szListType,szListType);
						pRiff->iNumSpecialPending++;
					}
				}

				if (hFile)
				{
					if (!psp->hData) // return this one
					{	psp->hData=hFile;
						psp->dwSize=dwSizeFile;
						psp->dwExtra=dwCtFile;
						lstrcpyA(psp->szType,"file");
					}
					else // add to pending
					{
						pRiff->spPending[pRiff->iNumSpecialPending].hSpecialData=NULL; // not used
						pRiff->spPending[pRiff->iNumSpecialPending].hData=hFile;
						pRiff->spPending[pRiff->iNumSpecialPending].dwSize=dwSizeFile;
						pRiff->spPending[pRiff->iNumSpecialPending].dwExtra=dwCtFile; 
						lstrcpyA(pRiff->spPending[pRiff->iNumSpecialPending].szType,"file");
						lstrcpyA(pRiff->spPending[pRiff->iNumSpecialPending].szListType,szListType);
						pRiff->iNumSpecialPending++;
					}
				}

				//bSkipTopLevelAdvance=FALSE;
				pwd->dwListEnd=0; // we're doing these all at once. so we're out of the list now
				bOk=TRUE;
			}
			else if (lstrcmpA(szListType,"xxxx")==0)
			{
				pwd->dwOffset+=8+pwd->dwChunkSize;
				bSkipTopLevelAdvance=TRUE;
				bOk=FALSE;
				SetFilePointer64(pRiff->hFile,pwd->dwListEnd,FILE_BEGIN);
				pwd->dwListEnd=0;
			}
			else
			{	char * pData; // unknown list
				DWORD dwEvenSize;
				psp->dwSize=pwd->dwChunkSize; // includes the heading 'LIST'
				ReadFile(pRiff->hFile, psp->szType, 4, &dwReadBytes, NULL);
				psp->szType[4]=0x00;
				ReadFile(pRiff->hFile,&psp->dwSize,4,&dwReadBytes,NULL); // special size is size of text
				if (psp->dwSize > pwd->dwChunkSize - 8) // invalid
				{
					SetFilePointer64(pRiff->hFile,pwd->dwOffset+8+pwd->dwChunkSize,FILE_BEGIN);
					pwd->dwOffset+=8+pwd->dwChunkSize;
					pwd->dwListEnd=0;
					bOk = FALSE;
					continue;
				}

				if (dwReadBytes!=4)
				{	bNoMore=TRUE;
					break;
				}

				psp->hData=GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT,psp->dwSize+8);
				pData=(char *)GlobalLock(psp->hData);

				dwEvenSize=((psp->dwSize+1)/2)*2;					
				ReadFile(pRiff->hFile,pData,dwEvenSize,&dwReadBytes,NULL);
				if (dwReadBytes!=dwEvenSize)
				{	bNoMore=TRUE;
					break;
				}							
				
				GlobalUnlock(psp->hData);
				bSkipTopLevelAdvance=TRUE;
				lstrcpyA(psp->szListType, szListType);				// "WAVE"

				// if this was last item in list, get out of list
				if (SetFilePointer64(pRiff->hFile,0,FILE_CURRENT)>=pwd->dwListEnd)
				{	
					bSkipTopLevelAdvance=FALSE;
					pwd->dwListEnd=0;
				}
				bOk=TRUE;
			}			
		}
		else // any other data (misc and unknown data types)
		{	char * pData;
			DWORD dwEvenSize;
			psp->dwSize=pwd->dwChunkSize;
			psp->hData=GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT,psp->dwSize+8);
			pData=(char *)GlobalLock(psp->hData);
			dwEvenSize=((psp->dwSize+1)/2)*2;									
			ReadFile(pRiff->hFile,pData,dwEvenSize,&dwReadBytes,NULL);
			if (dwReadBytes!=dwEvenSize)
			{	bNoMore=TRUE;
				break;
			}							
			
			lstrcpyA(psp->szListType,"WAVE");
			GlobalUnlock(psp->hData);
			bOk=TRUE;
			//bSkipTopLevelAdvance=FALSE;
		}
		// advance to next top level item
		if (!bSkipTopLevelAdvance)
		{
			SetFilePointer64(pRiff->hFile,pwd->dwOffset+8+pwd->dwChunkSize,FILE_BEGIN);
			pwd->dwOffset+=8+pwd->dwChunkSize;
		}
	}

	if (psp->hData)
	{	// Now free our memory since we allocated it within _THIS_ DLL
		LPSTR szFrom=GlobalLock(psp->hData);
		LPSTR szTo=psp->szDataIn; //GlobalLock(psp->hDataIn);
		psp->dwSize=min(psp->dwSize,psp->dwSizeIn);
		CopyMemory(szTo,szFrom,psp->dwSize);
		GlobalUnlock(psp->hData);
		//GlobalUnlock(psp->hDataIn);
		GlobalFree(psp->hData);
		psp->hData=NULL;
	}


	GlobalUnlock(psp->hSpecialData);

	if (bNoMore)
	{
		SetFilePointer64(pRiff->hFile,pwd->dwSaveCursor,FILE_BEGIN);
		GlobalFree(psp->hSpecialData);
		psp->hSpecialData=NULL;
		dwSuccess=0;
	}
	return dwSuccess;
}

DWORD RiffStuffGetFirstSpecialData(RIFFSTUFF * pRiff, SPECIALDATA * psp)
{	
	DWORD dwSuccess=0; // 1=success
	RIFFWAVEDATA * pwd=NULL;
	char szType[5]={"xxxx"};

	if (!pRiff)
		return 0;  // return is boolean

	psp->hSpecialData=GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT,sizeof(RIFFWAVEDATA));
	pwd=(RIFFWAVEDATA *)GlobalLock(psp->hSpecialData);
	pwd->dwSaveCursor=SetFilePointer64(pRiff->hFile,0,FILE_CURRENT);
	pwd->dwOffset=0;
	SetFilePointer64(pRiff->hFile,pRiff->dwChunksStart,FILE_BEGIN);
	//pRiff->dwFilePos=pRiff->dwChunksStart;
	pwd->dwRiffLength=pRiff->dwFullSize;
	pwd->dwOffset=pRiff->dwChunksStart;
	pRiff->iNumSpecialPending=0;
	GlobalUnlock(psp->hSpecialData);

	return RiffStuffGetNextSpecialData(pRiff, psp);
}

