/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2006-2007 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

//
// SessionSdkIterpretter.h
//

#pragma once

#include "SessionSdkStructs.h"
#include "SessionSerializerStructs.h"

namespace AuditionSessionSDK
{

class SessionIterpretter_FromSDK
{
public:
	
	SessionIterpretter_FromSDK()  {}
	~SessionIterpretter_FromSDK()  {}

	COMPLETE_SESSION_INFO_NEW& GetSessionDataFromSdk(FULL_SDK_SESSION_INFO* inSdkSessionInfo);

	COMPLETE_SESSION_INFO_NEW& GetSessionInfo()
	{
		return m_sessionInfo;
	}

	static void test();

protected:

	bool	MainSessionInfoFromSdk(SDK_SESSION_MAIN_INFO* inSdkMainSessionInfo, SESSION_MAIN_INFO_NEW& outMainSessionInfo);
	bool	VersionInfoFromSdk(SDK_SESSION_VERSION_INFO* inSdkVersionInfo, VERSION_INFO_NEW& outVersionInfo);
	bool	NotesInfoFromSdk(SDK_SESSION_NOTES_INFO* inSdkNotesInfo, std::wstring& outSessionNotesInfo);
	bool	StateInfoFromSdk(SDK_SESSION_STATE_INFO* inSdkStateInfo, SESSION_STATE_INFO_NEW& outStateInfo);
	bool	TempoInfoFromSdk(SDK_SESSION_TEMPO_INFO* inSdkTempoInfo, SESSION_TEMPO_INFO_NEW& outTempoInfo);
	bool	TracksInfoFromSdk(SDK_SESSION_ALL_TRACKS_INFO* inSdkTracksInfo, SESSION_ALL_TRACKS_INFO_NEW& outTracksInfo);
	bool	WaveEntityInfoFromSdk(SDK_SESSION_ENTITY_INFO** inSdkWaveEntityInfo, unsigned int inNumEntities, std::vector<SESSION_ENTITY_INFO_NEW>& outWaveEntityInfo);
	bool	MidiEntityInfoFromSdk(SDK_SESSION_ENTITY_INFO** inSdkMidiEntityInfo, unsigned int inNumEntities, std::vector<SESSION_ENTITY_INFO_NEW>& outMidiEntityInfo);
	bool	VideoEntityInfoFromSdk(SDK_SESSION_ENTITY_INFO** inSdkVideoEntityInfo, unsigned int inNumEntities, std::vector<SESSION_ENTITY_INFO_NEW>& outVideoEntityInfo);
	bool	GroupInfoFromSdk(SDK_SESSION_GROUP_INFO** inSdkGroupInfo, unsigned int inNumGroups, std::vector<SESSION_GROUP_INFO_NEW>& outGroupInfo);
	bool	BlockInfoFromSdk(SDK_SESSION_BLOCK_INFO* inSdkBlockInfo, SESSION_BLOCK_INFO_NEW& outBlockInfo);
	bool	EnvelopeInfoFromSdk(SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO** inSdkEnvelopeInfo, unsigned int inNumEnvelopeGroups, std::vector<SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW>& outEnvelopeInfo);
	bool	LoopInfoFromSdk(SDK_SESSION_LOOP_INFO** inSdkLoopInfo, unsigned int inNumLoops, std::vector<SESSION_LOOP_INFO_NEW>& outLoopInfo);
	bool	CueInfoFromSdk(SDK_SESSION_CUE_INFO** inSdkCueInfo, unsigned int inNumCues, std::vector<SESSION_CUE_INFO_NEW>& outCueInfo);
	bool	MetronomeInfoFromSdk(SDK_SESSION_METRONOME_INFO* inSdkMetronomeInfo, SESSION_METRONOME_INFO_NEW& outMetronomeInfo);
	bool	MultiChannelEncoderInfoFromSdk(SDK_SESSION_MULTICHANNELENCODER_INFO* inSdkMultiChannelEncoderInfo, SESSION_MULTICHANNELENCODER_INFO_NEW& outMultiChannelEncoderInfo);

private:

	COMPLETE_SESSION_INFO_NEW	m_sessionInfo;

};


// Forward declaration
class SdkAllocatedObject_Abstract;

class MemoryDeallocator
{
public:
	MemoryDeallocator(SdkAllocatedObject_Abstract* inAllocatedObject);
	MemoryDeallocator(const MemoryDeallocator& inMemoryDeallocator);
	~MemoryDeallocator();

	MemoryDeallocator& operator=(const MemoryDeallocator& inRhs);

	void	AddRef();
	void	Release();

private:
	SdkAllocatedObject_Abstract* mAllocatedObject;
	u32* mRefCount;

};

typedef std::vector<MemoryDeallocator> AllocationList;


class SessionIterpretter_ToSDK
{
public:

	SessionIterpretter_ToSDK()  { ClearAllData(); }
	~SessionIterpretter_ToSDK()  {}

	FULL_SDK_SESSION_INFO* FormatSessionDataForSdk(const COMPLETE_SESSION_INFO_NEW& inSessionInfo);

	FULL_SDK_SESSION_INFO* GetSdkSessionInfo()
	{
		return &m_sdkSessionInfo;
	}

	void ClearAllData();

protected:

	SDK_SESSION_MAIN_INFO*						MainSessionInfoForSdk(const SESSION_MAIN_INFO_NEW& inMainSessionInfo);
	SDK_SESSION_VERSION_INFO*					VersionInfoForSdk(const VERSION_INFO_NEW& inVersionInfo);
	SDK_SESSION_NOTES_INFO*						NotesInfoForSdk(const std::wstring& inSessionNotesInfo);
	SDK_SESSION_STATE_INFO*						StateInfoForSdk(const SESSION_STATE_INFO_NEW& inStateInfo);
	SDK_SESSION_TEMPO_INFO*						TempoInfoForSdk(const SESSION_TEMPO_INFO_NEW& inTempoInfo);
	SDK_SESSION_ALL_TRACKS_INFO*				TracksInfoForSdk(const SESSION_ALL_TRACKS_INFO_NEW& inTracksInfo);
	SDK_SESSION_ENTITY_INFO**					WaveEntityInfoForSdk(const std::vector<SESSION_ENTITY_INFO_NEW>& inWaveEntityInfo, unsigned int& outNumEntities);
	SDK_SESSION_ENTITY_INFO**					MidiEntityInfoForSdk(const std::vector<SESSION_ENTITY_INFO_NEW>& inMidiEntityInfo, unsigned int& outNumEntities);
	SDK_SESSION_ENTITY_INFO**					VideoEntityInfoForSdk(const std::vector<SESSION_ENTITY_INFO_NEW>& inVideoEntityInfo, unsigned int& outNumEntities);
	SDK_SESSION_GROUP_INFO**					GroupInfoForSdk(const std::vector<SESSION_GROUP_INFO_NEW>& inGroupInfo, unsigned int& outNumGroups);
	SDK_SESSION_BLOCK_INFO*						BlockInfoForSdk(const SESSION_BLOCK_INFO_NEW& inBlockInfo);
	SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO**		EnvelopeInfoForSdk(const std::vector<SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW>& inEnvelopeInfo, unsigned int& outNumEnvelopeGroups);
	SDK_SESSION_LOOP_INFO**						LoopInfoForSdk(const std::vector<SESSION_LOOP_INFO_NEW>& inLoopInfo, unsigned int& outNumLoops);
	SDK_SESSION_CUE_INFO**						CueInfoForSdk(const std::vector<SESSION_CUE_INFO_NEW>& inCueInfo, unsigned int& outNumCues);
	SDK_SESSION_METRONOME_INFO*					MetronomeInfoForSdk(const SESSION_METRONOME_INFO_NEW& inMetronomeInfo);
	SDK_SESSION_MULTICHANNELENCODER_INFO*		MultiChannelEncoderInfoForSdk(const SESSION_MULTICHANNELENCODER_INFO_NEW& inMultiChannelEncoderInfo);

private:

	FULL_SDK_SESSION_INFO	m_sdkSessionInfo;
	AllocationList mAllocationList;

};


}		// namespace AuditionSessionSDK


