/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2006-2007 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

//
// SessionSdkStructs.h
//

#pragma once


#pragma pack(1)

typedef __int32				i32;
typedef unsigned __int8		u8;
typedef unsigned __int16	u16;
typedef unsigned __int32	u32;
typedef __int64				i64;
typedef double				f64;
typedef u32					b32;		// bool
typedef unsigned short		c16;		// wchar_t


struct SDK_SESSION_GUID 
{
	u32	Data1;
	u16 Data2;
	u16	Data3;
	u8	Data4[8];
};

struct SDK_SESSION_STRING
{
	u32		stringSize;			// size includes null terminator
	c16*	chars;
};

struct SDK_SESSION_METRONOME_INFO
{
	u32			mySize;
	i32			iSampleRate;
	f64			fBPM;
	f64			fOffsetMs;
	f64			fVolume;
	i32			iTickSpacing;
	i32			iOffset;
	i32			iSet;
	b32			bEnable;
	SDK_SESSION_STRING*	szDefaultTime;
	SDK_SESSION_STRING*	szPattern;
	u32			dwOutputPort;
	i32			iNoteLen;
};


struct SDK_SESSION_ENVELOPE_POINT_INFO
{
	u32	mySize;
	i64 dwAt;
	f64 value;
};


struct SDK_SESSION_BLOCK_ENVELOPE_INFO
{
	u32	mySize;
	u32 uiType; // (ET_PAN, ET_VOL, etc.)
	b32 bSplined;
	u32 numPoints;
	SDK_SESSION_ENVELOPE_POINT_INFO** points;
};


struct SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO
{
	u32	mySize;
	u32 dwID;					
	i64 rightMostSampleInEnvelopes;
	u32 numEnvelopes;
    SDK_SESSION_BLOCK_ENVELOPE_INFO** envelopes;
};


struct SDK_SESSION_MULTICHANNELENCODER_TRACK_INFO
{
	u32	mySize;
	b32 bUseFlatLines;
	f64 fFlatFade;
	f64 fFlatPan;
	i32 iSubLevel;				// sub level
	i32 iCenterLevel;				// center level
	i32 iTrackVolume;				// overall track volume
	u32 dwActiveSpeakers;		// bit field of active speakers
	i32 iPanningMode;				// special panning mode (currently only PM_STEREO or PM_MONO)
	b32 bFixedRouting;			// if true, all output goes to active speakers (otherwise use panning envelopes)
	i32 iTrack;
	SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO* envelopeGroup;
};


struct SDK_SESSION_MULTICHANNELENCODER_INFO
{
	u32				mySize;
	f64				fMasterVolume;
	SDK_SESSION_STRING* sessionName;
	u32				numTracks;
	SDK_SESSION_MULTICHANNELENCODER_TRACK_INFO**	tracks;
};


struct SDK_SESSION_LOOP_INFO
{
	u32	mySize;
	u32 dwID;
	f64 fNumBeats;
	f64 fNativeBpm;
	i32 iBaseNote;				// 0=non-voiced
	b32 bLoopSimple;
	b32 bLoopInSamples;
	u32 dwLoopSamples;		// for bLoopInSamples
	b32 bLoopInBeats;
	f64 fLoopEveryNBeats;	// for bLoopInBeats
	b32 bFollowSessionTempo;	// for bLoopInBeats
	f64 fLoopTempo;			// for bLoopInBeats
	b32 bTempoMatchFixed;
	b32 bTempoMatchStretch;
	b32 bTempoMatchResample;
	b32 bTempoMatchBeatSplice;
	i32 iBeatSpliceRate;		// 1 = whole, 2 = half beat... 16 = 1/16th beat
	b32 bCrossfade;
	f64 fSessionTempoUsed;
	i32 iStretchQuality;
	f64 fStretchSplicesPerBeat;
	f64 fStretchOverlapping;
	b32 bBeatSpliceAutoFind;
	f64 fBeatSpliceDb;
	f64 fBeatSpliceMs;
	u32 dwOrigFileIndent;
	f64 fTranspose;			 // This takes higher priority over iTranspose.
	b32 bLoopOnce;
};


struct SDK_SESSION_GROUP_INFO
{
	u32	mySize;
	i32 iIndex;
	i32 iColor;
};


struct SDK_SESSION_VERSION_INFO
{
	u32	mySize;
	f64 version;
	f64 buildNumber;
	SDK_SESSION_STRING* szVersionStringInfo;
};


struct SDK_SESSION_ENV_INFO
{
	u32	mySize;
	u32 uiType;
	b32 bSplined;
	SDK_SESSION_STRING* szName;
	u32	dwFlags;
	u32	LineColor;
	i64	dwSize;
	b32	bReadOnly;
	u32 numPoints;
	SDK_SESSION_ENVELOPE_POINT_INFO** points;
};


struct SDK_SESSION_AUDIO_PARAM
{
	u32	mySize;
	b32	bIsAutomatable;
	f64	parameterValueForRendering;
	b32	bSafeDuringWrite;
	SDK_SESSION_ENV_INFO*	envelope;
};


struct SDK_SESSION_AUDIO_COMPONENT
{
	u32	mySize;
	u32 componentType;
	i64 initialDelay;
	b32 isBypass;
	b32 isSerial;
	u32	numParams;
	SDK_SESSION_AUDIO_PARAM** audioParams;
};


struct SDK_SESSION_AUDIO_OUTPUT_COMPONENT
{
	u32	mySize;
	u32 componentType;
	i64 initialDelay;
	b32 isBypass;
	u32	numParams;
	SDK_SESSION_AUDIO_PARAM** audioParams;
	u32 pannerMode;
	b32 isSumDown;
	u32 outputChannels;
	i32 busID;
};


struct SDK_SESSION_AUDIO_EFFECT_COMPONENT
{
	u32	mySize;
	u32 componentType;
	i64 initialDelay;
	b32 isBypass;
	u32	numParams;
	SDK_SESSION_AUDIO_PARAM** audioParams;
	SDK_SESSION_GUID guid;
	SDK_SESSION_STRING* name;			// only used for reading in older .ses sessions
	SDK_SESSION_STRING* groupName;		// only used for reading in older .ses sessions
	SDK_SESSION_STRING* effectParamsString;
};


struct SDK_SESSION_AUDIO_EFFECT_COMPONENT_GROUP
{
	u32	mySize;
	u32 componentType;
	i64 initialDelay;
	b32 isBypass;
	b32 isSerial;
	u32	numParams;
	SDK_SESSION_AUDIO_PARAM** audioParams;
	SDK_SESSION_AUDIO_COMPONENT* inputGain;
	SDK_SESSION_AUDIO_COMPONENT* outputGain;
};


struct SDK_SESSION_AUDIO_SEND_COMPONENT
{
	u32	mySize;
	u32 componentType;
	i64 initialDelay;
	b32 isBypass;
	u32	numParams;
	SDK_SESSION_AUDIO_PARAM** audioParams;
	u32 pannerMode;
	b32 isSumDown;
	u32 numOutputChannels;
	i32 busID;
	b32 isPre;
};


struct SDK_SESSION_COMMON_TRACK_INFO
{
	u32	mySize;
	u32	id;
	u32	dwFlags;
	SDK_SESSION_STRING*	szTitle;
	u32	dwTrackHeight;
};


struct SDK_SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO
{
	u32				mySize;
	u32				audioComponentType;
	SDK_SESSION_STRING*	guidString;
	SDK_SESSION_STRING*	componentName;
	u32				parameterIndex;
	u32				laneUIHeight;
};


struct SDK_SESSION_AUDIO_TRACK_INFO
{
	u32	mySize;
	SDK_SESSION_COMMON_TRACK_INFO* baseTrackInfo;
	u32	trackType;
	u32	busID;
	u32	automationMode;
	SDK_SESSION_STRING*	inputDriver;
	u32	inputPort;
	SDK_SESSION_STRING*	inputPortName;
	SDK_SESSION_STRING*	outputDriver;
	u32	outputPort;
	SDK_SESSION_STRING*	outputPortName;
	b32			isPreMeter;
	b32			isPreEffect;
	b32			isFreeze;
	b32			isTrackEQUI;
	b32			isEffectsRackUI;
	SDK_SESSION_AUDIO_COMPONENT*					inputComponent;
	SDK_SESSION_AUDIO_COMPONENT*					eqComponent;
	SDK_SESSION_AUDIO_COMPONENT*					faderComponent;
	SDK_SESSION_AUDIO_COMPONENT*					muteComponent;
	SDK_SESSION_AUDIO_OUTPUT_COMPONENT*			outputComponent;
	SDK_SESSION_AUDIO_EFFECT_COMPONENT_GROUP*	effectComponentGroup;
	u32			numSends;
	SDK_SESSION_AUDIO_SEND_COMPONENT**	sendComponents;
	u32			numEffects;
	SDK_SESSION_AUDIO_EFFECT_COMPONENT** effectComponents;
	u32			numEffectSlots;
	b32*		hasEffectInSlot;
	b32			isShowingAutomationLanes;
	u32			trackHeight;
	u32			numAutomationLanes;
	SDK_SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO**	automationLanes;
	SDK_SESSION_AUDIO_EFFECT_COMPONENT*			midiHostEffect;

};


struct SDK_SESSION_AUDIO_TRACK_FULL_INFO
{
	u32	mySize;
	u32	trackType;
	u32	trackOrderIndex;
	SDK_SESSION_AUDIO_TRACK_INFO* track;
};


struct SDK_SESSION_COMMON_TRACK_FULL_INFO
{
	u32	mySize;
	u32	trackType;
	u32	trackOrderIndex;
	SDK_SESSION_COMMON_TRACK_INFO* track;
};


struct SDK_SESSION_ALL_TRACKS_INFO
{
	u32	mySize;
	u32	nextTrackID;
	SDK_SESSION_AUDIO_TRACK_FULL_INFO* masterTrack;
	u32	numAudioTracks;
	SDK_SESSION_AUDIO_TRACK_FULL_INFO** audioTracks;
	u32 numNonAudioTracks;
	SDK_SESSION_COMMON_TRACK_FULL_INFO** nonAudioTracks;
};


struct SDK_SESSION_CUE_INFO
{
	u32	mySize;
	u32 dwType;
	i64 qiOffset;
	i64 qiLength;
	SDK_SESSION_STRING* wszName;
	SDK_SESSION_STRING* wszDesc;
};


struct SDK_SESSION_COMMON_BLOCK_INFO
{
	u32	mySize;
	u32 dwBlockID;
	u32 dwEntityID;
	f64 fLeftVol;
	f64 fRightVol;
	i64 dwOffset;
	i64 dwSize;
	i64 dwFileIndent;
	u32 dwFlags;
	i32 iTrack;
	i32 iParentGroup;
	i32 iPunchGeneration;
	i32 iPreviousPunch;
	i32 iNextPunch;
	i32 iHue;
	i32	iParentBlock;
};


struct SDK_SESSION_WAVE_BLOCK_INFO
{
	u32	mySize;
	u32 dwBlockID;
	i64 iFadeInOffset;
	i64 iFadeOutOffset;
	i32 iFadeInShapeVal;
	i32 iFadeOutShapeVal;
	i32 iFadeInType;
	i32 iFadeOutType;
	b32 hasCrossfadeLeft;
	b32 hasCrossfadeRight;
	i32 iCrossfadeLeftLinkType;
	i32 iCrossfadeRightLinkType;
};

struct SDK_SESSION_VIDEO_BLOCK_INFO
{
	u32	mySize;
	u32 dwBlockID;
	u16 dwFlags;
	SDK_SESSION_STRING* szAudioFileName;
};

struct SDK_SESSION_BLOCK_INFO
{
	u32			mySize;
	u32			numCommonBlockInfo;
	SDK_SESSION_COMMON_BLOCK_INFO** commonBlockInfo;
	u32			numWaveBlockInfo;
	SDK_SESSION_WAVE_BLOCK_INFO** waveBlockInfo;
	u32			numVideoBlockInfo;
	SDK_SESSION_VIDEO_BLOCK_INFO** videoBlockInfo;
};

struct SDK_SESSION_ENTITY_INFO
{
	u32	mySize;
	u32 dwWaveformID;
	u32 dwFileFormat;			// old theFileType like stuff - format of file when saving
	i32 iCodecDelay;			// when opening compressed waves, chop this many samples off of start of data
	i64 dwTrueLength;			// when opening compressed waves, chop this many samples off of start of data
	u32 iEntityType;
	SDK_SESSION_STRING* szFilename;		// size varies when saved waveform filename, placed inside list
};

struct SDK_SESSION_MAIN_INFO
{
	u32	mySize;
	i32 iSampleRate;
	i64 qiNumSamples;
	f64 fMasterVolume;
	i64 qiMainOffset;
	b32 bRecovery;
	i32 iRulerFormat;
	i32 bPanningMode;
	f64 fVolEnvRange;			// For volume envelopes (always in 0 to 1.0 range), multiply by this factor for true volume
	i32 iNumberOfBlocks;
	SDK_SESSION_STRING* szFilename;
	SDK_SESSION_STRING* szMaster;
	SDK_SESSION_STRING* szASIODriver;
	i32 iTakeNumber;
};

struct SDK_SESSION_STATE_INFO
{
	u32	mySize;
	f64 fLeftSample;		// View
	f64 fRightSample;
	f64 fLoTrack;		// ViewTracks
	f64 fHiTrack;
	i64 qiLoSample;		// Selection
	i64 qiHiSample;
	f64 fNewLoTrack;
};

struct SDK_SESSION_TEMPO_INFO
{
	u32	mySize;
	f64 fBeatsPerMinute;
	i32 iBeatsPerBar;
	i32 iTicksPerQuarterNote;
	f64 fBeatOffsetMs;
	i32 iSessionBaseNote;
	i32 iNoteLen;
};

struct SDK_SESSION_NOTES_INFO
{
	u32	mySize;
	SDK_SESSION_STRING*		notesInfo;
};

struct FULL_SDK_SESSION_INFO
{
	u32										mySize;
	SDK_SESSION_MAIN_INFO*					sessionMainInfo;
	SDK_SESSION_VERSION_INFO*				versionInfo;
	SDK_SESSION_NOTES_INFO*					notesInfo;
	SDK_SESSION_STATE_INFO*					stateInfo;
	SDK_SESSION_TEMPO_INFO*					tempoInfo;
	SDK_SESSION_ALL_TRACKS_INFO*			tracksInfo;
	u32										numWaveEntities;
	SDK_SESSION_ENTITY_INFO** 				waveEntityInfo;
	u32										numVideoEntities;
	SDK_SESSION_ENTITY_INFO** 				videoEntityInfo;
	u32										numGroups;
	SDK_SESSION_GROUP_INFO** 				groupInfo;
	SDK_SESSION_BLOCK_INFO*					blockInfo;
	u32										numBlockEnvelopes;
	SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO**	blockEnvelopes;
	u32										numLoops;
	SDK_SESSION_LOOP_INFO** 				loopInfo;
	u32										numCues;
	SDK_SESSION_CUE_INFO** 					cueInfo;
	SDK_SESSION_METRONOME_INFO*				metronomeInfo;
	SDK_SESSION_MULTICHANNELENCODER_INFO*	multiChannelEncoderInfo;
};

#pragma pack()