/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2007 Adobe Systems Incorporated                       */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/


#pragma once


#include <string>
#include <map>


namespace AuditionSessionSDK
{


class XmlElement
{
public:

	typedef std::map<std::wstring, XmlElement>		ElementMap;
	typedef ElementMap::const_iterator				ElementIter;

	XmlElement();
	XmlElement(const std::wstring& inName);
	XmlElement(const std::wstring& inName, const std::wstring& inValue);
	XmlElement(const std::wstring& inName, const __int64& inValue);
	XmlElement(const std::wstring& inName, const int& inValue);
	XmlElement(const std::wstring& inName, const unsigned int& inValue);
	XmlElement(const std::wstring& inName, const unsigned long& inValue);
	XmlElement(const std::wstring& inName, const double& inValue);
	XmlElement(const std::wstring& inName, const bool& inValue);

	~XmlElement();

	bool ParseInto(const std::wstring& inString, std::wstring::size_type& ioFirstAvailableCharIndex);
	bool WriteToString(std::wstring& ioFileString) const;

	void			SetName(const std::wstring& inName);
	std::wstring	GetName() const;

	void	SetValue(const std::wstring& inValue);
	bool	GetValue(std::wstring& outString) const;
	bool	GetElementValue(const std::wstring& inElementName, std::wstring& outString) const;

	void	SetValue(const __int64& inValue);
	bool	GetValue(__int64& outValue) const;
	bool	GetElementValue(const std::wstring& inElementName, __int64& outValue) const;

	void	SetValue(const int& inValue);
	bool	GetValue(int& outValue) const;
	bool	GetElementValue(const std::wstring& inElementName, int& outValue) const;

	void	SetValue(const unsigned short& inValue);
	bool	GetValue(unsigned short& outValue) const;
	bool	GetElementValue(const std::wstring& inElementName, unsigned short& outValue) const;

	void	SetValue(const unsigned int& inValue);
	bool	GetValue(unsigned int& outValue) const;
	bool	GetElementValue(const std::wstring& inElementName, unsigned int& outValue) const;

	void	SetValue(const unsigned char& inValue);
	bool	GetValue(unsigned char& outValue) const;
	bool	GetElementValue(const std::wstring& inElementName, unsigned char& outValue) const;

	void	SetValue(const unsigned long& inValue);
	bool	GetValue(unsigned long& outValue) const;
	bool	GetElementValue(const std::wstring& inElementName, unsigned long& outValue) const;

	void	SetValue(const double& inValue);
	bool	GetValue(double& outValue) const;
	bool	GetElementValue(const std::wstring& inElementName, double& outValue) const;

	void	SetValue(const bool& inValue);
	bool	GetValue(bool& outValue) const;
	bool	GetElementValue(const std::wstring& inElementName, bool& outValue) const;

	// If the Element is found, returns true and outElementIter is the iterator pointing to the element
	// If the Element is not found, returns false, and outElementIter is invalid.
	bool	GetElement(const std::wstring& inElementName, ElementIter& outElementIter) const;

	// The element passed in is copied into our map
	// The return value is a direct reference to the item in our map
	// So if the desire is to add an element, and then continue work with that element,
	// the work should be done with the return val.
	XmlElement&	SetElement(const XmlElement& inElement);

private:

	std::wstring	mElementName;
	std::wstring	mValue;
	ElementMap		mElements;
};

class SimpleParser
{
public:

	SimpleParser();
	~SimpleParser();

	bool	Understands(const std::wstring& inFileName);
	bool	LoadFromFile(const std::wstring& inFileName);
	bool	WriteToFile(const std::wstring& inFileName);

	XmlElement element;
};


}	// namespace AuditionSessionSDK

