/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2003-2004 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

//
// XMLSession.h
//

#pragma once

#include "..\CPlugInSession.h"
#include "..\SessionSerializerStructs.h"
#include "SimpleParser.h"

#include <string>


// Forward declarations
struct COMPLETE_SESSION_INFO_NEW;
struct FULL_SDK_SESSION_INFO;

namespace AuditionSessionSDK
{
	class SessionIterpretter_FromSDK;
	class SessionIterpretter_ToSDK;
}


class XMLSession:public CPlugInSession
{
public:

	XMLSession();
	~XMLSession();

	HRESULT GetFlags(DWORD * pdwFlags);
	HRESULT Understands(LPCTSTR szName);
	HRESULT	ReadSession(LPCTSTR szFilename, FULL_SDK_SESSION_INFO** outSessionInfo);
	HRESULT	CloseSession(void);
	HRESULT	WriteSession(LPCTSTR szFilename, FULL_SDK_SESSION_INFO* inSessionInfo);

protected:
	bool	WriteEntireSession(const COMPLETE_SESSION_INFO_NEW& inCompleteSession);

	void	WriteMainSessionInfo(const SESSION_MAIN_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement);
	void	WriteVersionInfo(const VERSION_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement);
	void	WriteNotesInfo(const std::wstring& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement);
	void	WriteStateInfo(const SESSION_STATE_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement);
	void	WriteTempoInfo(const SESSION_TEMPO_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement);
	void	WriteTracksInfo(const SESSION_ALL_TRACKS_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement);
	void	WriteEntityInfo(const std::vector<SESSION_ENTITY_INFO_NEW>& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement, const std::wstring& inEntityType);
	void	WriteGroupInfo(const std::vector<SESSION_GROUP_INFO_NEW>& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement);
	void	WriteBlockInfo(const SESSION_BLOCK_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement);
	void	WriteBlockEnvelopesInfo(const std::vector<SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW>& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement);
	void	WriteLoopInfo(const std::vector<SESSION_LOOP_INFO_NEW>& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement);
	void	WriteCueInfo(const std::vector<SESSION_CUE_INFO_NEW>& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement);
	void	WriteMetronomeInfo(const SESSION_METRONOME_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement);
	void	WriteMultiChannelEncoderInfo(const SESSION_MULTICHANNELENCODER_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement);


	bool	ReadEntireSession(COMPLETE_SESSION_INFO_NEW& outCompleteSession);

	bool	ReadMainSessionInfo(SESSION_MAIN_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement);
	bool	ReadVersionInfo(VERSION_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement);
	bool	ReadNotesInfo(std::wstring& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement);
	bool	ReadStateInfo(SESSION_STATE_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement);
	bool	ReadTempoInfo(SESSION_TEMPO_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement);
	bool	ReadTracksInfo(SESSION_ALL_TRACKS_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement);
	bool	ReadEntityInfo(std::vector<SESSION_ENTITY_INFO_NEW>& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement, const std::wstring& inEntityType);
	bool	ReadGroupInfo(std::vector<SESSION_GROUP_INFO_NEW>& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement);
	bool	ReadBlockInfo(SESSION_BLOCK_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement);
	bool	ReadBlockEnvelopesInfo(std::vector<SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW>& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement);
	bool	ReadLoopInfo(std::vector<SESSION_LOOP_INFO_NEW>& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement);
	bool	ReadCueInfo(std::vector<SESSION_CUE_INFO_NEW>& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement);
	bool	ReadMetronomeInfo(SESSION_METRONOME_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement);
	bool	ReadMultiChannelEncoderInfo(SESSION_MULTICHANNELENCODER_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement);


private:
	AuditionSessionSDK::SessionIterpretter_ToSDK*	mInterpretterToSdk;
	std::wstring mSessionFilename;
	DWORD m_dwFlags;
};


