/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2003-2007 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

#define STRICT

#include <Windows.h>
#include "XML.h"
#include <assert.h>

using namespace AuditionSessionSDK;



namespace
{

	template<typename T, typename U>
		void ReadVector(std::vector<T>& outVector, const AuditionSessionSDK::XmlElement& inParentElement, const std::wstring& inVectorName, U processingFunction)
	{
		outVector.clear();

		XmlElement::ElementIter foundElementIter;
		if (!inParentElement.GetElement(inVectorName, foundElementIter))
		{
			// outVector is already empty.  Return...
			return;
		}

		for (unsigned long index = 0; true; ++index)
		{
			wchar_t tempString[70];
			std::wstring vectorIndexValue = _ultow(index, tempString, 10);
			vectorIndexValue.insert(0, L'_');

			XmlElement::ElementIter foundVectorItemIter;
			if (!foundElementIter->second.GetElement(inVectorName + vectorIndexValue, foundVectorItemIter))
			{
				// No more items in vector.  Break out of loop.
				break;
			}

			T newItem;
			if (processingFunction(newItem, foundVectorItemIter->second))
			{
				outVector.push_back(newItem);
			}
		}
	}


	bool ReadEnvelopePoint(SESSION_ENVELOPE_POINT_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"EnvelopePoint", elementIter))
		{
			return false;
		}

		return
			elementIter->second.GetElementValue(L"dwAt", outSessionInfo.dwAt) &&
			elementIter->second.GetElementValue(L"value", outSessionInfo.value);
	}

	bool ReadEnvelopeInfo(SESSION_ENV_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"EnvelopeInfo", elementIter))
		{
			return false;
		}

		bool returnVal =
			elementIter->second.GetElementValue(L"uiType", outSessionInfo.uiType) &&
			elementIter->second.GetElementValue(L"bSplined", outSessionInfo.bSplined) &&
			elementIter->second.GetElementValue(L"szName", outSessionInfo.szName) &&
			elementIter->second.GetElementValue(L"dwFlags", outSessionInfo.dwFlags) &&
			elementIter->second.GetElementValue(L"LineColor", outSessionInfo.LineColor) &&
			elementIter->second.GetElementValue(L"dwSize", outSessionInfo.dwSize) &&
			elementIter->second.GetElementValue(L"dwAt", outSessionInfo.bReadOnly);

		ReadVector(outSessionInfo.points, elementIter->second, L"EnvelopePointList", ReadEnvelopePoint);

		return returnVal;
	}

	bool ReadAudioParamInfo(SESSION_AUDIO_PARAM_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"AudioParamInfo", elementIter))
		{
			return false;
		}

		bool returnVal =
			elementIter->second.GetElementValue(L"bIsAutomatable", outSessionInfo.bIsAutomatable) &&
			elementIter->second.GetElementValue(L"parameterValueForRendering", outSessionInfo.parameterValueForRendering) &&
			elementIter->second.GetElementValue(L"bSafeDuringWrite", outSessionInfo.bSafeDuringWrite);

		if (returnVal)
		{
			returnVal = ReadEnvelopeInfo(outSessionInfo.envelope, elementIter->second);
		}

		return returnVal;
	}

	bool ReadAudioComponenetInfo(SESSION_AUDIO_COMPONENT_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement, const std::wstring inElementNameAppendage = L"")
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"AudioComponentInfo" + inElementNameAppendage, elementIter))
		{
			return false;
		}

		bool returnVal =
			elementIter->second.GetElementValue(L"componentType", outSessionInfo.componentType) &&
			elementIter->second.GetElementValue(L"initialDelay", outSessionInfo.initialDelay) &&
			elementIter->second.GetElementValue(L"isBypass", outSessionInfo.isBypass) &&
			elementIter->second.GetElementValue(L"isSerial", outSessionInfo.isSerial);

		ReadVector(outSessionInfo.audioParams, elementIter->second, L"AudioParamsList", ReadAudioParamInfo);

		return returnVal;
	}

	bool ReadAudioOutputComponenetInfo(SESSION_AUDIO_OUTPUT_COMPONENT_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"AudioOutputComponentInfo", elementIter))
		{
			return false;
		}

		bool returnVal =
			elementIter->second.GetElementValue(L"componentType", outSessionInfo.componentType) &&
			elementIter->second.GetElementValue(L"initialDelay", outSessionInfo.initialDelay) &&
			elementIter->second.GetElementValue(L"isBypass", outSessionInfo.isBypass) &&
			elementIter->second.GetElementValue(L"pannerMode", outSessionInfo.pannerMode) &&
			elementIter->second.GetElementValue(L"isSumDown", outSessionInfo.isSumDown) &&
			elementIter->second.GetElementValue(L"outputChannels", outSessionInfo.outputChannels) &&
			elementIter->second.GetElementValue(L"busID", outSessionInfo.busID);

		ReadVector(outSessionInfo.audioParams, elementIter->second, L"AudioParamsList", ReadAudioParamInfo);

		return returnVal;
	}

	bool ReadAudioEffectComponentGroupInfo(SESSION_AUDIO_EFFECT_COMPONENT_GROUP_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"AudioEffectComponentGroupInfo", elementIter))
		{
			return false;
		}

		bool returnVal =
			elementIter->second.GetElementValue(L"componentType", outSessionInfo.componentType) &&
			elementIter->second.GetElementValue(L"initialDelay", outSessionInfo.initialDelay) &&
			elementIter->second.GetElementValue(L"isBypass", outSessionInfo.isBypass) &&
			elementIter->second.GetElementValue(L"isSerial", outSessionInfo.isSerial);

		ReadVector(outSessionInfo.audioParams, elementIter->second, L"AudioParamsList", ReadAudioParamInfo);

		if (returnVal)
		{
			returnVal = ReadAudioComponenetInfo(outSessionInfo.inputGain, elementIter->second, L"_InputGain");
		}

		if (returnVal)
		{
			returnVal = ReadAudioComponenetInfo(outSessionInfo.outputGain, elementIter->second, L"_OutputGain");
		}

		return returnVal;

	}

	bool ReadAudioSendComponenetInfo(SESSION_AUDIO_SEND_COMPONENT_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"AudioSendComponentInfo", elementIter))
		{
			return false;
		}

		bool returnVal =
			elementIter->second.GetElementValue(L"componentType", outSessionInfo.componentType) &&
			elementIter->second.GetElementValue(L"initialDelay", outSessionInfo.initialDelay) &&
			elementIter->second.GetElementValue(L"isBypass", outSessionInfo.isBypass) &&
			elementIter->second.GetElementValue(L"pannerMode", outSessionInfo.pannerMode) &&
			elementIter->second.GetElementValue(L"isSumDown", outSessionInfo.isSumDown) &&
			elementIter->second.GetElementValue(L"numOutputChannels", outSessionInfo.numOutputChannels) &&
			elementIter->second.GetElementValue(L"busID", outSessionInfo.busID) &&
			elementIter->second.GetElementValue(L"isPre", outSessionInfo.isPre);

		ReadVector(outSessionInfo.audioParams, elementIter->second, L"AudioParamsList", ReadAudioParamInfo);

		return returnVal;
	}

	bool ReadAudioEffectComponenetInfo(SESSION_AUDIO_EFFECT_COMPONENT_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"AudioEffectComponentInfo", elementIter))
		{
			return false;
		}

		bool returnVal =
			elementIter->second.GetElementValue(L"componentType", outSessionInfo.componentType) &&
			elementIter->second.GetElementValue(L"initialDelay", outSessionInfo.initialDelay) &&
			elementIter->second.GetElementValue(L"isBypass", outSessionInfo.isBypass) &&
			elementIter->second.GetElementValue(L"guid_1", outSessionInfo.guid.Data1) &&
			elementIter->second.GetElementValue(L"guid_2", outSessionInfo.guid.Data2) &&
			elementIter->second.GetElementValue(L"guid_3", outSessionInfo.guid.Data3) &&
			elementIter->second.GetElementValue(L"name", outSessionInfo.name) &&
			elementIter->second.GetElementValue(L"groupName", outSessionInfo.groupName) &&
			elementIter->second.GetElementValue(L"effectParamsString", outSessionInfo.effectParamsString);

		ReadVector(outSessionInfo.audioParams, elementIter->second, L"AudioParamsList", ReadAudioParamInfo);

		for (int index = 0; index < 8; ++index)
		{
			wchar_t tempString[70];
			std::wstring indexString = _ultow(index + 4, tempString, 10);

			if (returnVal)
			{
				returnVal = elementIter->second.GetElementValue(L"guid_" + indexString, outSessionInfo.guid.Data4[index]);
			}
		}

		return returnVal;
	}

	bool ReadEffectSlot(bool& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"EffectSlot", elementIter))
		{
			return false;
		}

		return elementIter->second.GetElementValue(L"isEffectInSlot", outSessionInfo);
	}

	bool ReadTrackAutomationLaneInfo(SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"TrackAutomationLaneInfo", elementIter))
		{
			return false;
		}

		return
			elementIter->second.GetElementValue(L"audioComponentType", outSessionInfo.audioComponentType) &&
			elementIter->second.GetElementValue(L"guidString", outSessionInfo.guidString) &&
			elementIter->second.GetElementValue(L"componentName", outSessionInfo.componentName) &&
			elementIter->second.GetElementValue(L"parameterIndex", outSessionInfo.parameterIndex) &&
			elementIter->second.GetElementValue(L"laneUIHeight", outSessionInfo.laneUIHeight);
	}

	bool ReadCommonTrackInfo(SESSION_COMMON_TRACK_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"CommonTrackInfo", elementIter))
		{
			return false;
		}

		return
			elementIter->second.GetElementValue(L"id", outSessionInfo.id) &&
			elementIter->second.GetElementValue(L"dwFlags", outSessionInfo.dwFlags) &&
			elementIter->second.GetElementValue(L"szTitle", outSessionInfo.szTitle) &&
			elementIter->second.GetElementValue(L"dwTrackHeight", outSessionInfo.dwTrackHeight);
	}

	bool ReadAudioTrackInfo(SESSION_AUDIO_TRACK_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"AudioTrackInfo", elementIter))
		{
			return false;
		}

		bool returnVal =
			elementIter->second.GetElementValue(L"trackType", outSessionInfo.trackType) &&
			elementIter->second.GetElementValue(L"busID", outSessionInfo.busID) &&
			elementIter->second.GetElementValue(L"automationMode", outSessionInfo.automationMode) &&
			elementIter->second.GetElementValue(L"inputDriver", outSessionInfo.inputDriver) &&
			elementIter->second.GetElementValue(L"inputPort", outSessionInfo.inputPort) &&
			elementIter->second.GetElementValue(L"inputPortName", outSessionInfo.inputPortName) &&
			elementIter->second.GetElementValue(L"outputDriver", outSessionInfo.outputDriver) &&
			elementIter->second.GetElementValue(L"outputPort", outSessionInfo.outputPort) &&
			elementIter->second.GetElementValue(L"outputPortName", outSessionInfo.outputPortName) &&
			elementIter->second.GetElementValue(L"isPreMeter", outSessionInfo.isPreMeter) &&
			elementIter->second.GetElementValue(L"isPreEffect", outSessionInfo.isPreEffect) &&
			elementIter->second.GetElementValue(L"isFreeze", outSessionInfo.isFreeze) &&
			elementIter->second.GetElementValue(L"isTrackEQUI", outSessionInfo.isTrackEQUI) &&
			elementIter->second.GetElementValue(L"isEffectsRackUI", outSessionInfo.isEffectsRackUI) &&
			elementIter->second.GetElementValue(L"isShowingAutomationLanes", outSessionInfo.isShowingAutomationLanes) &&
			elementIter->second.GetElementValue(L"trackHeight", outSessionInfo.trackHeight);

		if (returnVal)
		{
			returnVal = ReadCommonTrackInfo(outSessionInfo.baseTrackInfo, elementIter->second);
		}
		if (returnVal)
		{
			returnVal = ReadAudioComponenetInfo(outSessionInfo.inputComponent, elementIter->second, L"_Input");
		}
		if (returnVal)
		{
			returnVal = ReadAudioComponenetInfo(outSessionInfo.eqComponent, elementIter->second, L"_EQ");
		}
		if (returnVal)
		{
			returnVal = ReadAudioComponenetInfo(outSessionInfo.faderComponent, elementIter->second, L"_Fader");
		}
		if (returnVal)
		{
			returnVal = ReadAudioComponenetInfo(outSessionInfo.muteComponent, elementIter->second, L"_Mute");
		}
		if (returnVal)
		{
			returnVal = ReadAudioOutputComponenetInfo(outSessionInfo.outputComponent, elementIter->second);
		}
		if (returnVal)
		{
			returnVal = ReadAudioEffectComponentGroupInfo(outSessionInfo.effectComponentGroup, elementIter->second);
		}
		
		ReadVector(outSessionInfo.sendComponents, elementIter->second, L"SendComponents", ReadAudioSendComponenetInfo);
		ReadVector(outSessionInfo.effectComponents, elementIter->second, L"EffectComponents", ReadAudioEffectComponenetInfo);

		if (returnVal)
		{
			returnVal = ReadAudioEffectComponenetInfo(outSessionInfo.midiHostEffect, elementIter->second);
		}

		ReadVector(outSessionInfo.hasEffectInSlot, elementIter->second, L"EffectSlots", ReadEffectSlot);
		ReadVector(outSessionInfo.automationLanes, elementIter->second, L"AutomationLaneInfo", ReadTrackAutomationLaneInfo);

		return returnVal;
	}

	bool ReadAudioTrackFull(SESSION_AUDIO_TRACK_FULL_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"AudioTrack", elementIter))
		{
			return false;
		}

		bool returnVal =
			elementIter->second.GetElementValue(L"trackType", outSessionInfo.trackType) &&
			elementIter->second.GetElementValue(L"trackOrderIndex", outSessionInfo.trackOrderIndex);

		if (returnVal)
		{
			returnVal = ReadAudioTrackInfo(outSessionInfo.track, elementIter->second);
		}

		return returnVal;
	}

	bool ReadNonAudioTrackFull(SESSION_COMMON_TRACK_FULL_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"NonAudioTrack", elementIter))
		{
			return false;
		}

		bool returnVal =
			elementIter->second.GetElementValue(L"trackType", outSessionInfo.trackType) &&
			elementIter->second.GetElementValue(L"trackOrderIndex", outSessionInfo.trackOrderIndex);

		if (returnVal)
		{
			returnVal = ReadCommonTrackInfo(outSessionInfo.track, elementIter->second);
		}

		return returnVal;
	}

	bool ReadEntityInfo_Helper(SESSION_ENTITY_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"EntityInfo", elementIter))
		{
			return false;
		}

		return
			elementIter->second.GetElementValue(L"dwWaveformID", outSessionInfo.dwWaveformID) &&
			elementIter->second.GetElementValue(L"dwFileFormat", outSessionInfo.dwFileFormat) &&
			elementIter->second.GetElementValue(L"iCodecDelay", outSessionInfo.iCodecDelay) &&
			elementIter->second.GetElementValue(L"dwTrueLength", outSessionInfo.dwTrueLength) &&
			elementIter->second.GetElementValue(L"iEntityType", outSessionInfo.iEntityType) &&
			elementIter->second.GetElementValue(L"wszFilename", outSessionInfo.wszFilename);
	}

	bool ReadGroupInfo_Helper(SESSION_GROUP_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"GroupInfo", elementIter))
		{
			return false;
		}

		return
			elementIter->second.GetElementValue(L"iIndex", outSessionInfo.iIndex) &&
			elementIter->second.GetElementValue(L"iColor", outSessionInfo.iColor);
	}

	bool ReadCommonBlockInfo(SESSION_COMMON_BLOCK_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"CommonBlockInfo", elementIter))
		{
			return false;
		}

		return
			elementIter->second.GetElementValue(L"dwBlockID", outSessionInfo.dwBlockID) &&
			elementIter->second.GetElementValue(L"dwEntityID", outSessionInfo.dwEntityID) &&
			elementIter->second.GetElementValue(L"fLeftVol", outSessionInfo.fLeftVol) &&
			elementIter->second.GetElementValue(L"fRightVol", outSessionInfo.fRightVol) &&
			elementIter->second.GetElementValue(L"dwOffset", outSessionInfo.dwOffset) &&
			elementIter->second.GetElementValue(L"dwSize", outSessionInfo.dwSize) &&
			elementIter->second.GetElementValue(L"dwFileIndent", outSessionInfo.dwFileIndent) &&
			elementIter->second.GetElementValue(L"dwFlags", outSessionInfo.dwFlags) &&
			elementIter->second.GetElementValue(L"iTrack", outSessionInfo.iTrack) &&
			elementIter->second.GetElementValue(L"iParentGroup", outSessionInfo.iParentGroup) &&
			elementIter->second.GetElementValue(L"iPunchGeneration", outSessionInfo.iPunchGeneration) &&
			elementIter->second.GetElementValue(L"iPreviousPunch", outSessionInfo.iPreviousPunch) &&
			elementIter->second.GetElementValue(L"iNextPunch", outSessionInfo.iNextPunch) &&
			elementIter->second.GetElementValue(L"iHue", outSessionInfo.iHue) &&
			elementIter->second.GetElementValue(L"iParentBlock", outSessionInfo.iParentBlock);
	}

	bool ReadWaveBlockInfo(SESSION_WAVE_BLOCK_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"WaveBlockInfo", elementIter))
		{
			return false;
		}

		return
			elementIter->second.GetElementValue(L"dwBlockID", outSessionInfo.dwBlockID) &&
			elementIter->second.GetElementValue(L"iFadeInOffset", outSessionInfo.iFadeInOffset) &&
			elementIter->second.GetElementValue(L"iFadeOutOffset", outSessionInfo.iFadeOutOffset) &&
			elementIter->second.GetElementValue(L"iFadeInShapeVal", outSessionInfo.iFadeInShapeVal) &&
			elementIter->second.GetElementValue(L"iFadeOutShapeVal", outSessionInfo.iFadeOutShapeVal) &&
			elementIter->second.GetElementValue(L"iFadeInType", outSessionInfo.iFadeInType) &&
			elementIter->second.GetElementValue(L"iFadeOutType", outSessionInfo.iFadeOutType) &&
			elementIter->second.GetElementValue(L"hasCrossfadeLeft", outSessionInfo.hasCrossfadeLeft) &&
			elementIter->second.GetElementValue(L"hasCrossfadeRight", outSessionInfo.hasCrossfadeRight) &&
			elementIter->second.GetElementValue(L"iCrossfadeLeftLinkType", outSessionInfo.iCrossfadeLeftLinkType) &&
			elementIter->second.GetElementValue(L"iCrossfadeRightLinkType", outSessionInfo.iCrossfadeRightLinkType);
	}

	bool ReadVideoBlockInfo(SESSION_VIDEO_BLOCK_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"VideoBlockInfo", elementIter))
		{
			return false;
		}

		return
			elementIter->second.GetElementValue(L"dwBlockID", outSessionInfo.dwBlockID) &&
			elementIter->second.GetElementValue(L"dwFlags", outSessionInfo.dwFlags) &&
			elementIter->second.GetElementValue(L"wszAudioFileName", outSessionInfo.wszAudioFileName);
	}

	bool ReadBlockEnvelopeInfo(SESSION_BLOCK_ENVELOPE_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"BlockEnvelopeInfo", elementIter))
		{
			return false;
		}

		ReadVector(outSessionInfo.points, elementIter->second, L"EnvelopePointList", ReadEnvelopePoint);

		return
			elementIter->second.GetElementValue(L"uiType", outSessionInfo.uiType) &&
			elementIter->second.GetElementValue(L"bSplined", outSessionInfo.bSplined);
	}

	bool ReadBlockEnvelopeGroupInfo(SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"BlockEnvelopeGroupInfo", elementIter))
		{
			return false;
		}

		ReadVector(outSessionInfo.envelopes, elementIter->second, L"Envelopes", ReadBlockEnvelopeInfo);

		return
			elementIter->second.GetElementValue(L"dwID", outSessionInfo.dwID) &&
			elementIter->second.GetElementValue(L"rightMostSampleInEnvelope", outSessionInfo.rightMostSampleInEnvelope);
	}

	bool ReadLoopInfo_Helper(SESSION_LOOP_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"LoopInfo", elementIter))
		{
			return false;
		}

		return
			elementIter->second.GetElementValue(L"dwID", outSessionInfo.dwID) &&
			elementIter->second.GetElementValue(L"fNumBeats", outSessionInfo.fNumBeats) &&
			elementIter->second.GetElementValue(L"fNativeBpm", outSessionInfo.fNativeBpm) &&
			elementIter->second.GetElementValue(L"iBaseNote", outSessionInfo.iBaseNote) &&
			elementIter->second.GetElementValue(L"bLoopSimple", outSessionInfo.bLoopSimple) &&
			elementIter->second.GetElementValue(L"bLoopInSamples", outSessionInfo.bLoopInSamples) &&
			elementIter->second.GetElementValue(L"dwLoopSamples", outSessionInfo.dwLoopSamples) &&
			elementIter->second.GetElementValue(L"bLoopInBeats", outSessionInfo.bLoopInBeats) &&
			elementIter->second.GetElementValue(L"fLoopEveryNBeats", outSessionInfo.fLoopEveryNBeats) &&
			elementIter->second.GetElementValue(L"bFollowSessionTempo", outSessionInfo.bFollowSessionTempo) &&
			elementIter->second.GetElementValue(L"fLoopTempo", outSessionInfo.fLoopTempo) &&
			elementIter->second.GetElementValue(L"bTempoMatchFixed", outSessionInfo.bTempoMatchFixed) &&
			elementIter->second.GetElementValue(L"bTempoMatchStretch", outSessionInfo.bTempoMatchStretch) &&
			elementIter->second.GetElementValue(L"bTempoMatchResample", outSessionInfo.bTempoMatchResample) &&
			elementIter->second.GetElementValue(L"bTempoMatchBeatSplice", outSessionInfo.bTempoMatchBeatSplice) &&
			elementIter->second.GetElementValue(L"iBeatSpliceRate", outSessionInfo.iBeatSpliceRate) &&
			elementIter->second.GetElementValue(L"bCrossfade", outSessionInfo.bCrossfade) &&
			elementIter->second.GetElementValue(L"fSessionTempoUsed", outSessionInfo.fSessionTempoUsed) &&
			elementIter->second.GetElementValue(L"iStretchQuality", outSessionInfo.iStretchQuality) &&
			elementIter->second.GetElementValue(L"fStretchSplicesPerBeat", outSessionInfo.fStretchSplicesPerBeat) &&
			elementIter->second.GetElementValue(L"fStretchOverlapping", outSessionInfo.fStretchOverlapping) &&
			elementIter->second.GetElementValue(L"bBeatSpliceAutoFind", outSessionInfo.bBeatSpliceAutoFind) &&
			elementIter->second.GetElementValue(L"fBeatSpliceDb", outSessionInfo.fBeatSpliceDb) &&
			elementIter->second.GetElementValue(L"fBeatSpliceMs", outSessionInfo.fBeatSpliceMs) &&
			elementIter->second.GetElementValue(L"dwOrigFileIndent", outSessionInfo.dwOrigFileIndent) &&
			elementIter->second.GetElementValue(L"fTranspose", outSessionInfo.fTranspose) &&
			elementIter->second.GetElementValue(L"bLoopOnce", outSessionInfo.bLoopOnce);
	}

	bool ReadCueInfo_Helper(SESSION_CUE_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"CueInfo", elementIter))
		{
			return false;
		}

		return
			elementIter->second.GetElementValue(L"dwType", outSessionInfo.dwType) &&
			elementIter->second.GetElementValue(L"qiOffset", outSessionInfo.qiOffset) &&
			elementIter->second.GetElementValue(L"qiLength", outSessionInfo.qiLength) &&
			elementIter->second.GetElementValue(L"wszName", outSessionInfo.wszName) &&
			elementIter->second.GetElementValue(L"wszDesc", outSessionInfo.wszDesc);
	}

	bool ReadMultiChannelEncoderTrackInfo(SESSION_MULTICHANNELENCODER_TRACK_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
	{
		XmlElement::ElementIter elementIter;
		if (!inParentElement.GetElement(L"MultiChannelEncoderTrackInfo", elementIter))
		{
			return false;
		}

		bool returnVal =
			elementIter->second.GetElementValue(L"bUseFlatLines", outSessionInfo.bUseFlatLines) &&
			elementIter->second.GetElementValue(L"fFlatFade", outSessionInfo.fFlatFade) &&
			elementIter->second.GetElementValue(L"fFlatPan", outSessionInfo.fFlatPan) &&
			elementIter->second.GetElementValue(L"iSubLevel", outSessionInfo.iSubLevel) &&
			elementIter->second.GetElementValue(L"iCenterLevel", outSessionInfo.iCenterLevel) &&
			elementIter->second.GetElementValue(L"iTrackVolume", outSessionInfo.iTrackVolume) &&
			elementIter->second.GetElementValue(L"dwActiveSpeakers", outSessionInfo.dwActiveSpeakers) &&
			elementIter->second.GetElementValue(L"iPanningMode", outSessionInfo.iPanningMode) &&
			elementIter->second.GetElementValue(L"bFixedRouting", outSessionInfo.bFixedRouting) &&
			elementIter->second.GetElementValue(L"iTrack", outSessionInfo.iTrack);

		if (returnVal)
		{
			returnVal = ReadBlockEnvelopeGroupInfo(outSessionInfo.envelopeGroup, elementIter->second);
		}

		return returnVal;
	}

}		// private namespace



bool XMLSession::ReadEntireSession(COMPLETE_SESSION_INFO_NEW& outCompleteSession)
{
	SimpleParser parser;
	if (!parser.LoadFromFile(mSessionFilename) || parser.element.GetName().compare(L"AuditionSession") != 0)
	{
		return false;
	}

	outCompleteSession.sessionMainInfo.first = ReadMainSessionInfo(outCompleteSession.sessionMainInfo.second, parser.element);

	outCompleteSession.versionInfo.first = ReadVersionInfo(outCompleteSession.versionInfo.second, parser.element);

	outCompleteSession.notesInfo.first = ReadNotesInfo(outCompleteSession.notesInfo.second, parser.element);

	outCompleteSession.stateInfo.first = ReadStateInfo(outCompleteSession.stateInfo.second, parser.element);

	outCompleteSession.tempoInfo.first = ReadTempoInfo(outCompleteSession.tempoInfo.second, parser.element);

	outCompleteSession.tracksInfo.first = ReadTracksInfo(outCompleteSession.tracksInfo.second, parser.element);

	outCompleteSession.waveEntityInfo.first = ReadEntityInfo(outCompleteSession.waveEntityInfo.second, parser.element, L"Wave");

	outCompleteSession.videoEntityInfo.first = ReadEntityInfo(outCompleteSession.videoEntityInfo.second, parser.element, L"Video");

	outCompleteSession.groupInfo.first = ReadGroupInfo(outCompleteSession.groupInfo.second, parser.element);

	outCompleteSession.blockInfo.first = ReadBlockInfo(outCompleteSession.blockInfo.second, parser.element);

	outCompleteSession.blockEnvelopes.first = ReadBlockEnvelopesInfo(outCompleteSession.blockEnvelopes.second, parser.element);

	outCompleteSession.loopInfo.first = ReadLoopInfo(outCompleteSession.loopInfo.second, parser.element);

	outCompleteSession.cueInfo.first = ReadCueInfo(outCompleteSession.cueInfo.second, parser.element);

	outCompleteSession.metronomeInfo.first = ReadMetronomeInfo(outCompleteSession.metronomeInfo.second, parser.element);

	outCompleteSession.multiChannelEncoderInfo.first = ReadMultiChannelEncoderInfo(outCompleteSession.multiChannelEncoderInfo.second, parser.element);

	// We need to at least have the session header info...
	return outCompleteSession.sessionMainInfo.first;
}


bool XMLSession::ReadMainSessionInfo(SESSION_MAIN_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
{
	XmlElement::ElementIter elementIter;
	if (!inParentElement.GetElement(L"SessionHeaderInfo", elementIter))
	{
		return false;
	}

	return
		elementIter->second.GetElementValue(L"iSampleRate", outSessionInfo.iSampleRate) &&
		elementIter->second.GetElementValue(L"qiNumSamples", outSessionInfo.qiNumSamples) &&
		elementIter->second.GetElementValue(L"fMasterVolume", outSessionInfo.fMasterVolume) &&
		elementIter->second.GetElementValue(L"qiMainOffset", outSessionInfo.qiMainOffset) &&
		elementIter->second.GetElementValue(L"iRulerFormat", outSessionInfo.iRulerFormat) &&
		elementIter->second.GetElementValue(L"bPanningMode", outSessionInfo.bPanningMode) &&
		elementIter->second.GetElementValue(L"fVolEnvRange", outSessionInfo.fVolEnvRange) &&
		elementIter->second.GetElementValue(L"wszMaster", outSessionInfo.wszMaster) &&
		elementIter->second.GetElementValue(L"wszASIODriver", outSessionInfo.wszASIODriver) &&
		elementIter->second.GetElementValue(L"iTakeNumber", outSessionInfo.iTakeNumber) &&
		elementIter->second.GetElementValue(L"bRecovery", outSessionInfo.bRecovery) &&
		elementIter->second.GetElementValue(L"wszFilename", outSessionInfo.wszFilename) &&
		elementIter->second.GetElementValue(L"iNumberOfBlocks", outSessionInfo.iNumberOfBlocks);
}

bool XMLSession::ReadVersionInfo(VERSION_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
{
	XmlElement::ElementIter elementIter;
	if (!inParentElement.GetElement(L"VersionInfo", elementIter))
	{
		return false;
	}

	return
		elementIter->second.GetElementValue(L"version", outSessionInfo.version) &&
		elementIter->second.GetElementValue(L"buildNumber", outSessionInfo.buildNumber) &&
		elementIter->second.GetElementValue(L"szVersionStringInfo", outSessionInfo.szVersionStringInfo);
}

bool XMLSession::ReadNotesInfo(std::wstring& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
{
	XmlElement::ElementIter elementIter;
	if (!inParentElement.GetElement(L"NotesInfo", elementIter))
	{
		return false;
	}

	return
		elementIter->second.GetElementValue(L"Notes", outSessionInfo);
}

bool XMLSession::ReadStateInfo(SESSION_STATE_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
{
	XmlElement::ElementIter elementIter;
	if (!inParentElement.GetElement(L"StateInfo", elementIter))
	{
		return false;
	}

	return
		elementIter->second.GetElementValue(L"fLeftSample", outSessionInfo.fLeftSample) &&
		elementIter->second.GetElementValue(L"fRightSample", outSessionInfo.fRightSample) &&
		elementIter->second.GetElementValue(L"fLoTrack", outSessionInfo.fLoTrack) &&
		elementIter->second.GetElementValue(L"fHiTrack", outSessionInfo.fHiTrack) &&
		elementIter->second.GetElementValue(L"fNewLoTrack", outSessionInfo.fNewLoTrack) &&
		elementIter->second.GetElementValue(L"qiLoSample", outSessionInfo.qiLoSample) &&
		elementIter->second.GetElementValue(L"qiHiSample", outSessionInfo.qiHiSample);
}

bool XMLSession::ReadTempoInfo(SESSION_TEMPO_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
{
	XmlElement::ElementIter elementIter;
	if (!inParentElement.GetElement(L"TempoInfo", elementIter))
	{
		return false;
	}

	return
		elementIter->second.GetElementValue(L"fBeatsPerMinute", outSessionInfo.fBeatsPerMinute) &&
		elementIter->second.GetElementValue(L"iBeatsPerBar", outSessionInfo.iBeatsPerBar) &&
		elementIter->second.GetElementValue(L"iTicksPerQuarterNote", outSessionInfo.iTicksPerQuarterNote) &&
		elementIter->second.GetElementValue(L"fBeatOffsetMs", outSessionInfo.fBeatOffsetMs) &&
		elementIter->second.GetElementValue(L"iSessionBaseNote", outSessionInfo.iSessionBaseNote) &&
		elementIter->second.GetElementValue(L"iNoteLen", outSessionInfo.iNoteLen);
}

bool XMLSession::ReadTracksInfo(SESSION_ALL_TRACKS_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
{
	XmlElement::ElementIter elementIter;
	if (!inParentElement.GetElement(L"TracksInfo", elementIter))
	{
		return false;
	}

	ReadVector(outSessionInfo.audioTracks, elementIter->second, L"AudioTracks", ReadAudioTrackFull);
	ReadVector(outSessionInfo.nonAudioTracks, elementIter->second, L"NonAudioTracks", ReadNonAudioTrackFull);

	bool returnVal = 
		elementIter->second.GetElementValue(L"nextTrackID", outSessionInfo.nextTrackID);

	if (returnVal)
	{
		returnVal = ReadAudioTrackFull(outSessionInfo.masterTrack, elementIter->second);
	}

	return returnVal;
}

bool XMLSession::ReadEntityInfo(std::vector<SESSION_ENTITY_INFO_NEW>& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement, const std::wstring& inEntityType)
{
	ReadVector(outSessionInfo, inParentElement, L"EntityInfo_" + inEntityType, ReadEntityInfo_Helper);

	return true;
}

bool XMLSession::ReadGroupInfo(std::vector<SESSION_GROUP_INFO_NEW>& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
{
	ReadVector(outSessionInfo, inParentElement, L"GroupInfo", ReadGroupInfo_Helper);

	return true;
}

bool XMLSession::ReadBlockInfo(SESSION_BLOCK_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
{
	ReadVector(outSessionInfo.commonBlockInfo, inParentElement, L"CommonBlockInfo", ReadCommonBlockInfo);
	ReadVector(outSessionInfo.waveBlockInfo, inParentElement, L"WaveBlockInfo", ReadWaveBlockInfo);
	ReadVector(outSessionInfo.videoBlockInfo, inParentElement, L"VideoBlockInfo", ReadVideoBlockInfo);

	return true;
}

bool XMLSession::ReadBlockEnvelopesInfo(std::vector<SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW>& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
{
	ReadVector(outSessionInfo, inParentElement, L"BlockEnvelopes", ReadBlockEnvelopeGroupInfo);

	return true;
}

bool XMLSession::ReadLoopInfo(std::vector<SESSION_LOOP_INFO_NEW>& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
{
	ReadVector(outSessionInfo, inParentElement, L"LoopInfo", ReadLoopInfo_Helper);

	return true;
}

bool XMLSession::ReadCueInfo(std::vector<SESSION_CUE_INFO_NEW>& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
{
	ReadVector(outSessionInfo, inParentElement, L"CueInfo", ReadCueInfo_Helper);

	return true;
}

bool XMLSession::ReadMetronomeInfo(SESSION_METRONOME_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
{
	XmlElement::ElementIter elementIter;
	if (!inParentElement.GetElement(L"MetronomeInfo", elementIter))
	{
		return false;
	}

	return
		elementIter->second.GetElementValue(L"iSampleRate", outSessionInfo.iSampleRate) &&
		elementIter->second.GetElementValue(L"fBPM", outSessionInfo.fBPM) &&
		elementIter->second.GetElementValue(L"fOffsetMs", outSessionInfo.fOffsetMs) &&
		elementIter->second.GetElementValue(L"fVolume", outSessionInfo.fVolume) &&
		elementIter->second.GetElementValue(L"iTickSpacing", outSessionInfo.iTickSpacing) &&
		elementIter->second.GetElementValue(L"iOffset", outSessionInfo.iOffset) &&
		elementIter->second.GetElementValue(L"iSet", outSessionInfo.iSet) &&
		elementIter->second.GetElementValue(L"bEnable", outSessionInfo.bEnable) &&
		elementIter->second.GetElementValue(L"szDefaultTime", outSessionInfo.szDefaultTime) &&
		elementIter->second.GetElementValue(L"szPattern", outSessionInfo.szPattern) &&
		elementIter->second.GetElementValue(L"dwOutputPort", outSessionInfo.dwOutputPort) &&
		elementIter->second.GetElementValue(L"iNoteLen", outSessionInfo.iNoteLen);
}

bool XMLSession::ReadMultiChannelEncoderInfo(SESSION_MULTICHANNELENCODER_INFO_NEW& outSessionInfo, const AuditionSessionSDK::XmlElement& inParentElement)
{
	XmlElement::ElementIter elementIter;
	if (!inParentElement.GetElement(L"MultiChannelEncoderInfo", elementIter))
	{
		return false;
	}

	ReadVector(outSessionInfo.tracks, elementIter->second, L"Tracks", ReadMultiChannelEncoderTrackInfo);

	return
		elementIter->second.GetElementValue(L"sessionName", outSessionInfo.sessionName) &&
		elementIter->second.GetElementValue(L"fMasterVolume", outSessionInfo.fMasterVolume);
}

