/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2003-2007 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

#define STRICT

#include <windows.h>
#include "XML.h"
#include <assert.h>

using namespace AuditionSessionSDK;



namespace
{

	template<typename T, typename U>
		void WriteVector(const std::vector<T>& inVector, AuditionSessionSDK::XmlElement& ioParentElement, const std::wstring& inVectorName, U processingFunction)
	{

		XmlElement& vectorElement = ioParentElement.SetElement(XmlElement(inVectorName));

		const std::vector<T>::const_iterator beginIter = inVector.begin();
		const std::vector<T>::const_iterator endIter = inVector.end();
		std::vector<T>::const_iterator iter = beginIter;
		for ( ; iter != endIter; ++iter)
		{
			wchar_t tempString[70];
			std::wstring vectorIndexValue = _ultow(iter - beginIter, tempString, 10);
			vectorIndexValue.insert(0, L'_');

			XmlElement& vectorItemElement = vectorElement.SetElement(XmlElement(inVectorName + vectorIndexValue));

			processingFunction(*iter, vectorItemElement);
		}
	}


	void WriteEnvelopePoint(const SESSION_ENVELOPE_POINT_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"EnvelopePoint"));

		mainElement.SetElement(XmlElement(L"dwAt", inSessionInfo.dwAt));
		mainElement.SetElement(XmlElement(L"value", inSessionInfo.value));
	}

	void WriteEnvelopeInfo(const SESSION_ENV_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"EnvelopeInfo"));

		mainElement.SetElement(XmlElement(L"uiType", inSessionInfo.uiType));
		mainElement.SetElement(XmlElement(L"bSplined", inSessionInfo.bSplined));
		mainElement.SetElement(XmlElement(L"szName", inSessionInfo.szName));
		mainElement.SetElement(XmlElement(L"dwFlags", inSessionInfo.dwFlags));
		mainElement.SetElement(XmlElement(L"LineColor", inSessionInfo.LineColor));
		mainElement.SetElement(XmlElement(L"dwSize", inSessionInfo.dwSize));
		mainElement.SetElement(XmlElement(L"dwAt", inSessionInfo.bReadOnly));

		WriteVector(inSessionInfo.points, mainElement, L"EnvelopePointList", WriteEnvelopePoint);
	}

	void WriteAudioParamInfo(const SESSION_AUDIO_PARAM_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"AudioParamInfo"));

		mainElement.SetElement(XmlElement(L"bIsAutomatable", inSessionInfo.bIsAutomatable));
		mainElement.SetElement(XmlElement(L"parameterValueForRendering", inSessionInfo.parameterValueForRendering));
		mainElement.SetElement(XmlElement(L"bSafeDuringWrite", inSessionInfo.bSafeDuringWrite));

		WriteEnvelopeInfo(inSessionInfo.envelope, mainElement);
	}

	void WriteAudioComponenetInfo(const SESSION_AUDIO_COMPONENT_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement, const std::wstring inElementNameAppendage = L"")
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"AudioComponentInfo" + inElementNameAppendage));

		mainElement.SetElement(XmlElement(L"componentType", inSessionInfo.componentType));
		mainElement.SetElement(XmlElement(L"initialDelay", inSessionInfo.initialDelay));
		mainElement.SetElement(XmlElement(L"isBypass", inSessionInfo.isBypass));
		mainElement.SetElement(XmlElement(L"isSerial", inSessionInfo.isSerial));

		WriteVector(inSessionInfo.audioParams, mainElement, L"AudioParamsList", WriteAudioParamInfo);
	}

	void WriteAudioOutputComponenetInfo(const SESSION_AUDIO_OUTPUT_COMPONENT_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"AudioOutputComponentInfo"));

		mainElement.SetElement(XmlElement(L"componentType", inSessionInfo.componentType));
		mainElement.SetElement(XmlElement(L"initialDelay", inSessionInfo.initialDelay));
		mainElement.SetElement(XmlElement(L"isBypass", inSessionInfo.isBypass));

		WriteVector(inSessionInfo.audioParams, mainElement, L"AudioParamsList", WriteAudioParamInfo);

		mainElement.SetElement(XmlElement(L"pannerMode", inSessionInfo.pannerMode));
		mainElement.SetElement(XmlElement(L"isSumDown", inSessionInfo.isSumDown));
		mainElement.SetElement(XmlElement(L"outputChannels", inSessionInfo.outputChannels));
		mainElement.SetElement(XmlElement(L"busID", inSessionInfo.busID));
	}

	void WriteAudioEffectComponentGroupInfo(const SESSION_AUDIO_EFFECT_COMPONENT_GROUP_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"AudioEffectComponentGroupInfo"));

		mainElement.SetElement(XmlElement(L"componentType", inSessionInfo.componentType));
		mainElement.SetElement(XmlElement(L"initialDelay", inSessionInfo.initialDelay));
		mainElement.SetElement(XmlElement(L"isBypass", inSessionInfo.isBypass));
		mainElement.SetElement(XmlElement(L"isSerial", inSessionInfo.isSerial));

		WriteVector(inSessionInfo.audioParams, mainElement, L"AudioParamsList", WriteAudioParamInfo);

		WriteAudioComponenetInfo(inSessionInfo.inputGain, mainElement, L"_InputGain");
		WriteAudioComponenetInfo(inSessionInfo.outputGain, mainElement, L"_OutputGain");
	}

	void WriteAudioSendComponenetInfo(const SESSION_AUDIO_SEND_COMPONENT_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"AudioSendComponentInfo"));

		mainElement.SetElement(XmlElement(L"componentType", inSessionInfo.componentType));
		mainElement.SetElement(XmlElement(L"initialDelay", inSessionInfo.initialDelay));
		mainElement.SetElement(XmlElement(L"isBypass", inSessionInfo.isBypass));

		WriteVector(inSessionInfo.audioParams, mainElement, L"AudioParamsList", WriteAudioParamInfo);

		mainElement.SetElement(XmlElement(L"pannerMode", inSessionInfo.pannerMode));
		mainElement.SetElement(XmlElement(L"isSumDown", inSessionInfo.isSumDown));
		mainElement.SetElement(XmlElement(L"numOutputChannels", inSessionInfo.numOutputChannels));
		mainElement.SetElement(XmlElement(L"busID", inSessionInfo.busID));
		mainElement.SetElement(XmlElement(L"isPre", inSessionInfo.isPre));
	}

	void WriteAudioEffectComponenetInfo(const SESSION_AUDIO_EFFECT_COMPONENT_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"AudioEffectComponentInfo"));

		mainElement.SetElement(XmlElement(L"componentType", inSessionInfo.componentType));
		mainElement.SetElement(XmlElement(L"initialDelay", inSessionInfo.initialDelay));
		mainElement.SetElement(XmlElement(L"isBypass", inSessionInfo.isBypass));

		WriteVector(inSessionInfo.audioParams, mainElement, L"AudioParamsList", WriteAudioParamInfo);

		mainElement.SetElement(XmlElement(L"guid_1", inSessionInfo.guid.Data1));
		mainElement.SetElement(XmlElement(L"guid_2", inSessionInfo.guid.Data2));
		mainElement.SetElement(XmlElement(L"guid_3", inSessionInfo.guid.Data3));
		for (int index = 0; index < 8; ++index)
		{
			wchar_t tempString[70];
			std::wstring indexString = _ultow(index + 4, tempString, 10);

			mainElement.SetElement(XmlElement(L"guid_" + indexString, inSessionInfo.guid.Data4[index]));
		}
		mainElement.SetElement(XmlElement(L"name", inSessionInfo.name));
		mainElement.SetElement(XmlElement(L"groupName", inSessionInfo.groupName));
		mainElement.SetElement(XmlElement(L"effectParamsString", inSessionInfo.effectParamsString));
	}

	void WriteEffectSlot(const bool& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"EffectSlot"));

		mainElement.SetElement(XmlElement(L"isEffectInSlot", inSessionInfo));
	}

	void WriteTrackAutomationLaneInfo(const SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"TrackAutomationLaneInfo"));

		mainElement.SetElement(XmlElement(L"audioComponentType", inSessionInfo.audioComponentType));
		mainElement.SetElement(XmlElement(L"guidString", inSessionInfo.guidString));
		mainElement.SetElement(XmlElement(L"componentName", inSessionInfo.componentName));
		mainElement.SetElement(XmlElement(L"parameterIndex", inSessionInfo.parameterIndex));
		mainElement.SetElement(XmlElement(L"laneUIHeight", inSessionInfo.laneUIHeight));
	}

	void WriteCommonTrackInfo(const SESSION_COMMON_TRACK_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"CommonTrackInfo"));

		mainElement.SetElement(XmlElement(L"id", inSessionInfo.id));
		mainElement.SetElement(XmlElement(L"dwFlags", inSessionInfo.dwFlags));
		mainElement.SetElement(XmlElement(L"szTitle", inSessionInfo.szTitle));
		mainElement.SetElement(XmlElement(L"dwTrackHeight", inSessionInfo.dwTrackHeight));
	}

	void WriteAudioTrackInfo(const SESSION_AUDIO_TRACK_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"AudioTrackInfo"));

		WriteCommonTrackInfo(inSessionInfo.baseTrackInfo, mainElement);

		mainElement.SetElement(XmlElement(L"trackType", inSessionInfo.trackType));
		mainElement.SetElement(XmlElement(L"busID", inSessionInfo.busID));
		mainElement.SetElement(XmlElement(L"automationMode", inSessionInfo.automationMode));
		mainElement.SetElement(XmlElement(L"inputDriver", inSessionInfo.inputDriver));
		mainElement.SetElement(XmlElement(L"inputPort", inSessionInfo.inputPort));
		mainElement.SetElement(XmlElement(L"inputPortName", inSessionInfo.inputPortName));
		mainElement.SetElement(XmlElement(L"outputDriver", inSessionInfo.outputDriver));
		mainElement.SetElement(XmlElement(L"outputPort", inSessionInfo.outputPort));
		mainElement.SetElement(XmlElement(L"outputPortName", inSessionInfo.outputPortName));
		mainElement.SetElement(XmlElement(L"isPreMeter", inSessionInfo.isPreMeter));
		mainElement.SetElement(XmlElement(L"isPreEffect", inSessionInfo.isPreEffect));
		mainElement.SetElement(XmlElement(L"isFreeze", inSessionInfo.isFreeze));
		mainElement.SetElement(XmlElement(L"isTrackEQUI", inSessionInfo.isTrackEQUI));
		mainElement.SetElement(XmlElement(L"isEffectsRackUI", inSessionInfo.isEffectsRackUI));

		WriteAudioComponenetInfo(inSessionInfo.inputComponent, mainElement, L"_Input");
		WriteAudioComponenetInfo(inSessionInfo.eqComponent, mainElement, L"_EQ");
		WriteAudioComponenetInfo(inSessionInfo.faderComponent, mainElement, L"_Fader");
		WriteAudioComponenetInfo(inSessionInfo.muteComponent, mainElement, L"_Mute");
		WriteAudioOutputComponenetInfo(inSessionInfo.outputComponent, mainElement);
		WriteAudioEffectComponentGroupInfo(inSessionInfo.effectComponentGroup, mainElement);

		WriteVector(inSessionInfo.sendComponents, mainElement, L"SendComponents", WriteAudioSendComponenetInfo);
		WriteVector(inSessionInfo.effectComponents, mainElement, L"EffectComponents", WriteAudioEffectComponenetInfo);

		WriteAudioEffectComponenetInfo(inSessionInfo.midiHostEffect, mainElement);

		WriteVector(inSessionInfo.hasEffectInSlot, mainElement, L"EffectSlots", WriteEffectSlot);

		mainElement.SetElement(XmlElement(L"isShowingAutomationLanes", inSessionInfo.isShowingAutomationLanes));
		mainElement.SetElement(XmlElement(L"trackHeight", inSessionInfo.trackHeight));

		WriteVector(inSessionInfo.automationLanes, mainElement, L"AutomationLaneInfo", WriteTrackAutomationLaneInfo);
	}

	void WriteAudioTrackFull(const SESSION_AUDIO_TRACK_FULL_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"AudioTrack"));

		mainElement.SetElement(XmlElement(L"trackType", inSessionInfo.trackType));
		mainElement.SetElement(XmlElement(L"trackOrderIndex", inSessionInfo.trackOrderIndex));

		WriteAudioTrackInfo(inSessionInfo.track, mainElement);
	}

	void WriteNonAudioTrackFull(const SESSION_COMMON_TRACK_FULL_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"NonAudioTrack"));

		mainElement.SetElement(XmlElement(L"trackType", inSessionInfo.trackType));
		mainElement.SetElement(XmlElement(L"trackOrderIndex", inSessionInfo.trackOrderIndex));

		WriteCommonTrackInfo(inSessionInfo.track, mainElement);
	}

	void WriteEntityInfo_Helper(const SESSION_ENTITY_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"EntityInfo"));

		mainElement.SetElement(XmlElement(L"dwWaveformID", inSessionInfo.dwWaveformID));
		mainElement.SetElement(XmlElement(L"dwFileFormat", inSessionInfo.dwFileFormat));
		mainElement.SetElement(XmlElement(L"iCodecDelay", inSessionInfo.iCodecDelay));
		mainElement.SetElement(XmlElement(L"dwTrueLength", inSessionInfo.dwTrueLength));
		mainElement.SetElement(XmlElement(L"iEntityType", inSessionInfo.iEntityType));
		mainElement.SetElement(XmlElement(L"wszFilename", inSessionInfo.wszFilename));
	}

	void WriteGroupInfo_Helper(const SESSION_GROUP_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"GroupInfo"));

		mainElement.SetElement(XmlElement(L"iIndex", inSessionInfo.iIndex));
		mainElement.SetElement(XmlElement(L"iColor", inSessionInfo.iColor));
	}

	void WriteCommonBlockInfo(const SESSION_COMMON_BLOCK_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"CommonBlockInfo"));

		mainElement.SetElement(XmlElement(L"dwBlockID", inSessionInfo.dwBlockID));
		mainElement.SetElement(XmlElement(L"dwEntityID", inSessionInfo.dwEntityID));
		mainElement.SetElement(XmlElement(L"fLeftVol", inSessionInfo.fLeftVol));
		mainElement.SetElement(XmlElement(L"fRightVol", inSessionInfo.fRightVol));
		mainElement.SetElement(XmlElement(L"dwOffset", inSessionInfo.dwOffset));
		mainElement.SetElement(XmlElement(L"dwSize", inSessionInfo.dwSize));
		mainElement.SetElement(XmlElement(L"dwFileIndent", inSessionInfo.dwFileIndent));
		mainElement.SetElement(XmlElement(L"dwFlags", inSessionInfo.dwFlags));
		mainElement.SetElement(XmlElement(L"iTrack", inSessionInfo.iTrack));
		mainElement.SetElement(XmlElement(L"iParentGroup", inSessionInfo.iParentGroup));
		mainElement.SetElement(XmlElement(L"iPunchGeneration", inSessionInfo.iPunchGeneration));
		mainElement.SetElement(XmlElement(L"iPreviousPunch", inSessionInfo.iPreviousPunch));
		mainElement.SetElement(XmlElement(L"iNextPunch", inSessionInfo.iNextPunch));
		mainElement.SetElement(XmlElement(L"iHue", inSessionInfo.iHue));
		mainElement.SetElement(XmlElement(L"iParentBlock", inSessionInfo.iParentBlock));
	}

	void WriteWaveBlockInfo(const SESSION_WAVE_BLOCK_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"WaveBlockInfo"));

		mainElement.SetElement(XmlElement(L"dwBlockID", inSessionInfo.dwBlockID));
		mainElement.SetElement(XmlElement(L"iFadeInOffset", inSessionInfo.iFadeInOffset));
		mainElement.SetElement(XmlElement(L"iFadeOutOffset", inSessionInfo.iFadeOutOffset));
		mainElement.SetElement(XmlElement(L"iFadeInShapeVal", inSessionInfo.iFadeInShapeVal));
		mainElement.SetElement(XmlElement(L"iFadeOutShapeVal", inSessionInfo.iFadeOutShapeVal));
		mainElement.SetElement(XmlElement(L"iFadeInType", inSessionInfo.iFadeInType));
		mainElement.SetElement(XmlElement(L"iFadeOutType", inSessionInfo.iFadeOutType));
		mainElement.SetElement(XmlElement(L"hasCrossfadeLeft", inSessionInfo.hasCrossfadeLeft));
		mainElement.SetElement(XmlElement(L"hasCrossfadeRight", inSessionInfo.hasCrossfadeRight));
		mainElement.SetElement(XmlElement(L"iCrossfadeLeftLinkType", inSessionInfo.iCrossfadeLeftLinkType));
		mainElement.SetElement(XmlElement(L"iCrossfadeRightLinkType", inSessionInfo.iCrossfadeRightLinkType));
	}

	void WriteVideoBlockInfo(const SESSION_VIDEO_BLOCK_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"VideoBlockInfo"));

		mainElement.SetElement(XmlElement(L"dwBlockID", inSessionInfo.dwBlockID));
		mainElement.SetElement(XmlElement(L"dwFlags", inSessionInfo.dwFlags));
		mainElement.SetElement(XmlElement(L"wszAudioFileName", inSessionInfo.wszAudioFileName));
	}

	void WriteBlockEnvelopeInfo(const SESSION_BLOCK_ENVELOPE_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"BlockEnvelopeInfo"));

		mainElement.SetElement(XmlElement(L"uiType", inSessionInfo.uiType));
		mainElement.SetElement(XmlElement(L"bSplined", inSessionInfo.bSplined));

		WriteVector(inSessionInfo.points, mainElement, L"EnvelopePointList", WriteEnvelopePoint);
	}

	void WriteBlockEnvelopeGroupInfo(const SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"BlockEnvelopeGroupInfo"));

		mainElement.SetElement(XmlElement(L"dwID", inSessionInfo.dwID));

		WriteVector(inSessionInfo.envelopes, mainElement, L"Envelopes", WriteBlockEnvelopeInfo);

		mainElement.SetElement(XmlElement(L"rightMostSampleInEnvelope", inSessionInfo.rightMostSampleInEnvelope));
	}

	void WriteLoopInfo_Helper(const SESSION_LOOP_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"LoopInfo"));

		mainElement.SetElement(XmlElement(L"dwID", inSessionInfo.dwID));
		mainElement.SetElement(XmlElement(L"fNumBeats", inSessionInfo.fNumBeats));
		mainElement.SetElement(XmlElement(L"fNativeBpm", inSessionInfo.fNativeBpm));
		mainElement.SetElement(XmlElement(L"iBaseNote", inSessionInfo.iBaseNote));
		mainElement.SetElement(XmlElement(L"bLoopSimple", inSessionInfo.bLoopSimple));
		mainElement.SetElement(XmlElement(L"bLoopInSamples", inSessionInfo.bLoopInSamples));
		mainElement.SetElement(XmlElement(L"dwLoopSamples", inSessionInfo.dwLoopSamples));
		mainElement.SetElement(XmlElement(L"bLoopInBeats", inSessionInfo.bLoopInBeats));
		mainElement.SetElement(XmlElement(L"fLoopEveryNBeats", inSessionInfo.fLoopEveryNBeats));
		mainElement.SetElement(XmlElement(L"bFollowSessionTempo", inSessionInfo.bFollowSessionTempo));
		mainElement.SetElement(XmlElement(L"fLoopTempo", inSessionInfo.fLoopTempo));
		mainElement.SetElement(XmlElement(L"bTempoMatchFixed", inSessionInfo.bTempoMatchFixed));
		mainElement.SetElement(XmlElement(L"bTempoMatchStretch", inSessionInfo.bTempoMatchStretch));
		mainElement.SetElement(XmlElement(L"bTempoMatchResample", inSessionInfo.bTempoMatchResample));
		mainElement.SetElement(XmlElement(L"bTempoMatchBeatSplice", inSessionInfo.bTempoMatchBeatSplice));
		mainElement.SetElement(XmlElement(L"iBeatSpliceRate", inSessionInfo.iBeatSpliceRate));
		mainElement.SetElement(XmlElement(L"bCrossfade", inSessionInfo.bCrossfade));
		mainElement.SetElement(XmlElement(L"fSessionTempoUsed", inSessionInfo.fSessionTempoUsed));
		mainElement.SetElement(XmlElement(L"iStretchQuality", inSessionInfo.iStretchQuality));
		mainElement.SetElement(XmlElement(L"fStretchSplicesPerBeat", inSessionInfo.fStretchSplicesPerBeat));
		mainElement.SetElement(XmlElement(L"fStretchOverlapping", inSessionInfo.fStretchOverlapping));
		mainElement.SetElement(XmlElement(L"bBeatSpliceAutoFind", inSessionInfo.bBeatSpliceAutoFind));
		mainElement.SetElement(XmlElement(L"fBeatSpliceDb", inSessionInfo.fBeatSpliceDb));
		mainElement.SetElement(XmlElement(L"fBeatSpliceMs", inSessionInfo.fBeatSpliceMs));
		mainElement.SetElement(XmlElement(L"dwOrigFileIndent", inSessionInfo.dwOrigFileIndent));
		mainElement.SetElement(XmlElement(L"fTranspose", inSessionInfo.fTranspose));
		mainElement.SetElement(XmlElement(L"bLoopOnce", inSessionInfo.bLoopOnce));
	}

	void WriteCueInfo_Helper(const SESSION_CUE_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"CueInfo"));

		mainElement.SetElement(XmlElement(L"dwType", inSessionInfo.dwType));
		mainElement.SetElement(XmlElement(L"qiOffset", inSessionInfo.qiOffset));
		mainElement.SetElement(XmlElement(L"qiLength", inSessionInfo.qiLength));
		mainElement.SetElement(XmlElement(L"wszName", inSessionInfo.wszName));
		mainElement.SetElement(XmlElement(L"wszDesc", inSessionInfo.wszDesc));
	}

	void WriteMultiChannelEncoderTrackInfo(const SESSION_MULTICHANNELENCODER_TRACK_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
	{
		XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"MultiChannelEncoderTrackInfo"));

		mainElement.SetElement(XmlElement(L"bUseFlatLines", inSessionInfo.bUseFlatLines));
		mainElement.SetElement(XmlElement(L"fFlatFade", inSessionInfo.fFlatFade));
		mainElement.SetElement(XmlElement(L"fFlatPan", inSessionInfo.fFlatPan));
		mainElement.SetElement(XmlElement(L"iSubLevel", inSessionInfo.iSubLevel));
		mainElement.SetElement(XmlElement(L"iCenterLevel", inSessionInfo.iCenterLevel));
		mainElement.SetElement(XmlElement(L"iTrackVolume", inSessionInfo.iTrackVolume));
		mainElement.SetElement(XmlElement(L"dwActiveSpeakers", inSessionInfo.dwActiveSpeakers));
		mainElement.SetElement(XmlElement(L"iPanningMode", inSessionInfo.iPanningMode));
		mainElement.SetElement(XmlElement(L"bFixedRouting", inSessionInfo.bFixedRouting));
		mainElement.SetElement(XmlElement(L"iTrack", inSessionInfo.iTrack));

		WriteBlockEnvelopeGroupInfo(inSessionInfo.envelopeGroup, mainElement);
	}

}		// private namespace


bool XMLSession::WriteEntireSession(const COMPLETE_SESSION_INFO_NEW& inCompleteSession)
{
	SimpleParser parser;
	parser.element.SetName(L"AuditionSession");

	assert(inCompleteSession.sessionMainInfo.first);
	if (inCompleteSession.sessionMainInfo.first)
	{
		WriteMainSessionInfo(inCompleteSession.sessionMainInfo.second, parser.element);
	}

	if (inCompleteSession.versionInfo.first)
	{
		WriteVersionInfo(inCompleteSession.versionInfo.second, parser.element);
	}

	if (inCompleteSession.notesInfo.first)
	{
		WriteNotesInfo(inCompleteSession.notesInfo.second, parser.element);
	}

	if (inCompleteSession.stateInfo.first)
	{
		WriteStateInfo(inCompleteSession.stateInfo.second, parser.element);
	}

	if (inCompleteSession.tempoInfo.first)
	{
		WriteTempoInfo(inCompleteSession.tempoInfo.second, parser.element);
	}

	if (inCompleteSession.tracksInfo.first)
	{
		WriteTracksInfo(inCompleteSession.tracksInfo.second, parser.element);
	}

	if (inCompleteSession.waveEntityInfo.first)
	{
		WriteEntityInfo(inCompleteSession.waveEntityInfo.second, parser.element, L"Wave");
	}

	if (inCompleteSession.videoEntityInfo.first)
	{
		WriteEntityInfo(inCompleteSession.videoEntityInfo.second, parser.element, L"Video");
	}

	if (inCompleteSession.groupInfo.first)
	{
		WriteGroupInfo(inCompleteSession.groupInfo.second, parser.element);
	}

	if (inCompleteSession.blockInfo.first)
	{
		WriteBlockInfo(inCompleteSession.blockInfo.second, parser.element);
	}

	if (inCompleteSession.blockEnvelopes.first)
	{
		WriteBlockEnvelopesInfo(inCompleteSession.blockEnvelopes.second, parser.element);
	}

	if (inCompleteSession.loopInfo.first)
	{
		WriteLoopInfo(inCompleteSession.loopInfo.second, parser.element);
	}

	if (inCompleteSession.cueInfo.first)
	{
		WriteCueInfo(inCompleteSession.cueInfo.second, parser.element);
	}

	if (inCompleteSession.metronomeInfo.first)
	{
		WriteMetronomeInfo(inCompleteSession.metronomeInfo.second, parser.element);
	}

	if (inCompleteSession.multiChannelEncoderInfo.first)
	{
		WriteMultiChannelEncoderInfo(inCompleteSession.multiChannelEncoderInfo.second, parser.element);
	}

	return parser.WriteToFile(mSessionFilename);
}


void XMLSession::WriteMainSessionInfo(const SESSION_MAIN_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
{
	XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"SessionHeaderInfo"));

	mainElement.SetElement(XmlElement(L"iSampleRate", inSessionInfo.iSampleRate));
	mainElement.SetElement(XmlElement(L"qiNumSamples", inSessionInfo.qiNumSamples));
	mainElement.SetElement(XmlElement(L"fMasterVolume", inSessionInfo.fMasterVolume));
	mainElement.SetElement(XmlElement(L"qiMainOffset", inSessionInfo.qiMainOffset));
	mainElement.SetElement(XmlElement(L"iRulerFormat", inSessionInfo.iRulerFormat));
	mainElement.SetElement(XmlElement(L"bPanningMode", inSessionInfo.bPanningMode));
	mainElement.SetElement(XmlElement(L"fVolEnvRange", inSessionInfo.fVolEnvRange));
	mainElement.SetElement(XmlElement(L"wszMaster", inSessionInfo.wszMaster));
	mainElement.SetElement(XmlElement(L"wszASIODriver", inSessionInfo.wszASIODriver));
	mainElement.SetElement(XmlElement(L"iTakeNumber", inSessionInfo.iTakeNumber));
	mainElement.SetElement(XmlElement(L"bRecovery", inSessionInfo.bRecovery));
	mainElement.SetElement(XmlElement(L"wszFilename", inSessionInfo.wszFilename));
	mainElement.SetElement(XmlElement(L"iNumberOfBlocks", inSessionInfo.iNumberOfBlocks));
}

void XMLSession::WriteVersionInfo(const VERSION_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
{
	XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"VersionInfo"));

	mainElement.SetElement(XmlElement(L"version", inSessionInfo.version));
	mainElement.SetElement(XmlElement(L"buildNumber", inSessionInfo.buildNumber));
	mainElement.SetElement(XmlElement(L"szVersionStringInfo", inSessionInfo.szVersionStringInfo));
}

void XMLSession::WriteNotesInfo(const std::wstring& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
{
	XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"NotesInfo"));

	mainElement.SetElement(XmlElement(L"Notes", inSessionInfo));
}

void XMLSession::WriteStateInfo(const SESSION_STATE_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
{
	XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"StateInfo"));

	mainElement.SetElement(XmlElement(L"fLeftSample", inSessionInfo.fLeftSample));
	mainElement.SetElement(XmlElement(L"fRightSample", inSessionInfo.fRightSample));
	mainElement.SetElement(XmlElement(L"fLoTrack", inSessionInfo.fLoTrack));
	mainElement.SetElement(XmlElement(L"fHiTrack", inSessionInfo.fHiTrack));
	mainElement.SetElement(XmlElement(L"fNewLoTrack", inSessionInfo.fNewLoTrack));
	mainElement.SetElement(XmlElement(L"qiLoSample", inSessionInfo.qiLoSample));
	mainElement.SetElement(XmlElement(L"qiHiSample", inSessionInfo.qiHiSample));
}

void XMLSession::WriteTempoInfo(const SESSION_TEMPO_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
{
	XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"TempoInfo"));

	mainElement.SetElement(XmlElement(L"fBeatsPerMinute", inSessionInfo.fBeatsPerMinute));
	mainElement.SetElement(XmlElement(L"iBeatsPerBar", inSessionInfo.iBeatsPerBar));
	mainElement.SetElement(XmlElement(L"iTicksPerQuarterNote", inSessionInfo.iTicksPerQuarterNote));
	mainElement.SetElement(XmlElement(L"fBeatOffsetMs", inSessionInfo.fBeatOffsetMs));
	mainElement.SetElement(XmlElement(L"iSessionBaseNote", inSessionInfo.iSessionBaseNote));
	mainElement.SetElement(XmlElement(L"iNoteLen", inSessionInfo.iNoteLen));
}

void XMLSession::WriteTracksInfo(const SESSION_ALL_TRACKS_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
{
	XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"TracksInfo"));

	mainElement.SetElement(XmlElement(L"nextTrackID", inSessionInfo.nextTrackID));

	WriteAudioTrackFull(inSessionInfo.masterTrack, mainElement);

	WriteVector(inSessionInfo.audioTracks, mainElement, L"AudioTracks", WriteAudioTrackFull);
	WriteVector(inSessionInfo.nonAudioTracks, mainElement, L"NonAudioTracks", WriteNonAudioTrackFull);
}

void XMLSession::WriteEntityInfo(const std::vector<SESSION_ENTITY_INFO_NEW>& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement, const std::wstring& inEntityType)
{
	WriteVector(inSessionInfo, ioParentElement, L"EntityInfo_" + inEntityType, WriteEntityInfo_Helper);
}

void XMLSession::WriteGroupInfo(const std::vector<SESSION_GROUP_INFO_NEW>& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
{
	WriteVector(inSessionInfo, ioParentElement, L"GroupInfo", WriteGroupInfo_Helper);
}

void XMLSession::WriteBlockInfo(const SESSION_BLOCK_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
{
	WriteVector(inSessionInfo.commonBlockInfo, ioParentElement, L"CommonBlockInfo", WriteCommonBlockInfo);
	WriteVector(inSessionInfo.waveBlockInfo, ioParentElement, L"WaveBlockInfo", WriteWaveBlockInfo);
	WriteVector(inSessionInfo.videoBlockInfo, ioParentElement, L"VideoBlockInfo", WriteVideoBlockInfo);
}

void XMLSession::WriteBlockEnvelopesInfo(const std::vector<SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW>& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
{
	WriteVector(inSessionInfo, ioParentElement, L"BlockEnvelopes", WriteBlockEnvelopeGroupInfo);
}

void XMLSession::WriteLoopInfo(const std::vector<SESSION_LOOP_INFO_NEW>& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
{
	WriteVector(inSessionInfo, ioParentElement, L"LoopInfo", WriteLoopInfo_Helper);
}

void XMLSession::WriteCueInfo(const std::vector<SESSION_CUE_INFO_NEW>& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
{
	WriteVector(inSessionInfo, ioParentElement, L"CueInfo", WriteCueInfo_Helper);
}

void XMLSession::WriteMetronomeInfo(const SESSION_METRONOME_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
{
	XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"MetronomeInfo"));

	mainElement.SetElement(XmlElement(L"iSampleRate", inSessionInfo.iSampleRate));
	mainElement.SetElement(XmlElement(L"fBPM", inSessionInfo.fBPM));
	mainElement.SetElement(XmlElement(L"fOffsetMs", inSessionInfo.fOffsetMs));
	mainElement.SetElement(XmlElement(L"fVolume", inSessionInfo.fVolume));
	mainElement.SetElement(XmlElement(L"iTickSpacing", inSessionInfo.iTickSpacing));
	mainElement.SetElement(XmlElement(L"iOffset", inSessionInfo.iOffset));
	mainElement.SetElement(XmlElement(L"iSet", inSessionInfo.iSet));
	mainElement.SetElement(XmlElement(L"bEnable", inSessionInfo.bEnable));
	mainElement.SetElement(XmlElement(L"szDefaultTime", inSessionInfo.szDefaultTime));
	mainElement.SetElement(XmlElement(L"szPattern", inSessionInfo.szPattern));
	mainElement.SetElement(XmlElement(L"dwOutputPort", inSessionInfo.dwOutputPort));
	mainElement.SetElement(XmlElement(L"iNoteLen", inSessionInfo.iNoteLen));
}

void XMLSession::WriteMultiChannelEncoderInfo(const SESSION_MULTICHANNELENCODER_INFO_NEW& inSessionInfo, AuditionSessionSDK::XmlElement& ioParentElement)
{
	XmlElement& mainElement = ioParentElement.SetElement(XmlElement(L"MultiChannelEncoderInfo"));

	WriteVector(inSessionInfo.tracks, mainElement, L"Tracks", WriteMultiChannelEncoderTrackInfo);
	mainElement.SetElement(XmlElement(L"sessionName", inSessionInfo.sessionName));
	mainElement.SetElement(XmlElement(L"fMasterVolume", inSessionInfo.fMasterVolume));
}


